//LightRay3D SDK (C) 2002-2006 Tzivenis Sakis

#include <stdio.h>
#include "myassert.h"
#include <malloc.h>
#include <windows.h>

#include "plugapi.h"
#include "api.h"
#include "cmesh.h"
#include "cskin.h"
#include "cbone.h"
#include "array.h"
#include "expq3.h"
#include "interface.h"
#include "dlg.h"

#include "quake3.h"

#include <list>
#include <vector>



struct UV_Coord
{
	UV_Coord() : filled( false ) {};

	float	u,v;
	bool	filled;
	int		tvi;
};
struct i_Triangle
{
	union
	{
		struct 
		{
			int a,b,c;
		};
		int e[3];
	};
};

int cexpq3model::Export(const char *filename)
{
	int i;
	md3header_t header;
	cnode* tag_head=NULL;
	cnode* tag_torso=NULL;
	cnode* tag_weapon=NULL;
	cnode* tag_relative=NULL;

	memset(&header,0,sizeof(md3header_t));
	header.id=MD3_ID_HEADER;
	header.version=MD3_ALIAS_VERSION;
	header.flags=0;//???
	//These will be used to set anim length
	int minKeyTime=1000000,maxKeyTime=0;//init values.1000000 a really big number
	header.numboneframes=1;
	for(i=0;i<Interface::NumNodes();i++)
	{
		cnode *node=Interface::GetNodeAt(i);
		if(node->GetFlags()&FLAG_HIDDEN || node->GetTypeID()!=TYPE_BONE)
			continue;
		//Get keys' min,max frame(time)
		int start=node->GetAnimPtr()->GetFirstKeyTime();
		int end=node->GetAnimPtr()->GetLastKeyTime();
		if(start<minKeyTime) minKeyTime=start;
		if(end>maxKeyTime) maxKeyTime=end;
	}
	if(minKeyTime>maxKeyTime)
	{
		minKeyTime=maxKeyTime=Interface::GetAnimStart();
	}
	else
	{
		//Clamp to anim bar's length
		if(minKeyTime<Interface::GetAnimStart()) minKeyTime=Interface::GetAnimStart();
		else if(minKeyTime>Interface::GetAnimEnd()) minKeyTime=Interface::GetAnimEnd();
		if(maxKeyTime<Interface::GetAnimStart()) maxKeyTime=Interface::GetAnimStart();
		else if(maxKeyTime>Interface::GetAnimEnd()) maxKeyTime=Interface::GetAnimEnd();
	}

	//tags
	header.numtags=0;
	for(i=0;i<Interface::NumNodes();i++)
	{
		cnode* node=Interface::GetNodeAt(i);
		if(node->GetFlags()&FLAG_HIDDEN) continue;

		if(strstr(node->GetName(),"tag_"))
		{
			if(strcmp(node->GetName(),"tag_head")==0) tag_head=node;
			else if(strcmp(node->GetName(),"tag_torso")==0) tag_torso=node;
			else if(strcmp(node->GetName(),"tag_weapon")==0) tag_weapon=node;

			header.numtags++;
		}
	}
	//nummeshes
	header.nummeshes=0;
	for(i=0;i<Interface::NumNodes();i++)
	{
		cnode* node=Interface::GetNodeAt(i);
		if(node->GetFlags()&FLAG_HIDDEN) continue;

		if(node->GetTypeID()==TYPE_MESH||node->GetTypeID()==TYPE_SKIN)
		{
			//Pass tag meshes/skins
			if(strstr(node->GetName(),"tag_")==NULL) header.nummeshes++;
		}
	}
	//numskins
	header.numskins=0;
	for(i=0;i<Interface::NumNodes();i++)
	{
		cnode* node=Interface::GetNodeAt(i);
		if(node->GetFlags()&FLAG_HIDDEN) continue;

		if(node->GetTypeID()==TYPE_MESH||node->GetTypeID()==TYPE_SKIN)
		{
			if(node->GetMaterial())	header.numskins++;
		}
	}

	if(header.nummeshes==0)
	{
		MessageBox(::GetActiveWindow(),"No meshes/skins in scene to export!","MD3 Export",MB_ICONERROR);
		return 0;
	}

	//Parse options
	g_q3_startframe=minKeyTime;
	g_q3_endframe=maxKeyTime;
	g_q3_bHasTags=(header.numtags>0)?TRUE:FALSE;
	if(ExportQuake3Options()==0) return 1;
	header.numboneframes=(g_q3_endframe-g_q3_startframe)+1;
	if(g_q3_bHasTags)
	{
		if(g_q3_tag_relative==TAG_RELATIVE_NONE) tag_relative=NULL;
		else if(g_q3_tag_relative==TAG_RELATIVE_TORSO) tag_relative=tag_torso;
		else if(g_q3_tag_relative==TAG_RELATIVE_HEAD) tag_relative=tag_head;
		else if(g_q3_tag_relative==TAG_RELATIVE_WEAPON) tag_relative=tag_weapon;
		else if(g_q3_tag_relative==TAG_RELATIVE_USER)
		{
			tag_relative=Interface::FindNode(g_q3_tag_name);
		}
	}

	// open a file for writing
	FILE *	file;
	long	pos_current;
	char *	subname;
	int ival;
	float fval;

	file = fopen( filename, "wb" );
	if(!file)
	{
		MessageBox(::GetActiveWindow(),"Failed to open file for writing!","MD3 Export",MB_ICONERROR);
		return 0;
	}

	// write out sync pattern
	fwrite(&header.id, 1, sizeof(header.id), file );

	// write out version
	fwrite(&header.version, 1, sizeof(header.version), file );


	// Once we have the filename we mangle it.  We search for
	// 'models' and if found we remove the filename to that part.
	// Thus if we have "C:/quake3/baseq3/models/mapobjects/model.md3"
	// It becomes "models/mapobjects/model.md3"
	// otherwise we just leave it.
	subname = strstr( filename, "models" );
	if( !g_q3_bUseModels || subname == NULL )
		fwrite( filename, 64, 1, file );	// full model filename
	else
		fwrite( subname, 64, 1, file );		// filename from 'models' on.
	
	// write out header
	fwrite( &header.flags, 1, sizeof(header.flags),	file );// flags
	fwrite( &header.numboneframes, 1, sizeof(header.numboneframes),	file );// framenum
	fwrite( &header.numtags, 1, sizeof(header.numtags),	file );// tagnum
	fwrite( &header.nummeshes, 1, sizeof(header.nummeshes),	file );// meshnum
	fwrite( &header.numskins, 1, sizeof(header.numskins),	file );// skinnum
	ival=108;fwrite( &ival, 1, sizeof(ival),	file );// headersize-boneframes offs

	long pos_tagstart	= ftell( file );
	ival=0;fwrite(&ival,1,sizeof(ival),file);// tagstart
	long pos_tagend		= ftell( file );
	ival=256;fwrite(&ival,1,sizeof(ival),file);// tagend
	long pos_filesize	= ftell( file );
	ival=512;fwrite(&ival,1,sizeof(ival),file);// filesize

	//Write boneframes: some per frame object info.
	for( i = 0; i < header.numboneframes; i++ )
	{
		fval=-1;fwrite(&fval,1,sizeof(fval),file);	// min	X
		fval=-1;fwrite(&fval,1,sizeof(fval),file);	//		Y
		fval=-1;fwrite(&fval,1,sizeof(fval),file);	//		Z
		fval=1;fwrite(&fval,1,sizeof(fval),file);	// max	X
		fval=1;fwrite(&fval,1,sizeof(fval),file);	//		Y
		fval=1;fwrite(&fval,1,sizeof(fval),file);	//		Z
		fval=0;fwrite(&fval,1,sizeof(fval),file);	// pos	X
		fval=0;fwrite(&fval,1,sizeof(fval),file);	//		Y
		fval=0;fwrite(&fval,1,sizeof(fval),file);	//		Z
		fval=1;fwrite(&fval,1,sizeof(fval),file);	// Scale

		fwrite( "from LightRay3D", 16, 1, file );	// Creator
	}

	// fill in tagStart
	pos_current = ftell( file );
	fseek( file, pos_tagstart, SEEK_SET );
	fwrite( &pos_current, 1, sizeof(pos_current), file );
	fseek( file, pos_current, SEEK_SET );

	//Write tags: for each frame write out each tag
	cpoint3	row;
	for( i = g_q3_startframe; i <= g_q3_endframe; i++ ) // number of frames
	{
		for(int n=0;n<Interface::NumNodes();n++)
		{
			cnode* node=Interface::GetNodeAt(n);
			if(node->GetFlags()&FLAG_HIDDEN) continue;

			if(strstr(node->GetName(),"tag_")==NULL) continue;

			cmatrix4 tm	= node->GetNodeTM(i);
			cmatrix4 tmrot	= node->GetNodeTM(i,TM_ROT);

			fwrite( node->GetName(), 12, 1, file );	// tagname
			fwrite("unknown", 52, 1, file );		// unknown
			
			// If a relative_tag exists we transform any tags into the
			// relative_tag's space before writing them out.
			if( tag_relative != NULL )
			{
				cmatrix4 scratch = tag_relative->GetNodeTM(i);
				tm = tm*Inverse( scratch );
				scratch = tag_relative->GetNodeTM(i,TM_ROT);
				tmrot = tmrot*Inverse( scratch );
			}
			
			row = tm.GetRow3(3);
			
			// tag_floor is special.  We move everything down so its on the floor proper
		//	if( (g_relative_to_tag == tag_floor) && (relative_tag >= 0) )
		//		row.z += FLOOR_LEVEL;
			
			//Position
			fwrite( &row.x, 1, sizeof(float), file );	//		X
			fwrite( &row.y, 1, sizeof(float), file );	//		Y
			fwrite( &row.z, 1, sizeof(float), file );	//		Z
			
			// Rotation Matrix
			row = tmrot.GetRow3(0);
			fwrite( &row.x, 1, sizeof(float), file );	//
			fwrite( &row.y, 1, sizeof(float), file );	// row 0
			fwrite( &row.z, 1, sizeof(float), file );	//
			
			row = tmrot.GetRow3(1);
			fwrite( &row.x, 1, sizeof(float), file );	//
			fwrite( &row.y, 1, sizeof(float), file );	// row 1
			fwrite( &row.z, 1, sizeof(float), file );	//
			
			row = tmrot.GetRow3(2);
			fwrite( &row.x, 1, sizeof(float), file );	//
			fwrite( &row.y, 1, sizeof(float), file );	// row 2
			fwrite( &row.z, 1, sizeof(float), file );	//
		}
	}

	// fill in tagEnd
	pos_current = ftell( file );
	fseek( file, pos_tagend, SEEK_SET );
	fwrite(&pos_current, 1, sizeof(pos_current), file);
	fseek( file, pos_current, SEEK_SET );

	//Write Meshes
	char mapname[256];

	// Constants
	const cpoint3 x_axis( 1, 0, 0 );
	const cpoint3 z_axis( 0, 0, 1 );

	for(i=0;i<Interface::NumNodes();i++)
	{
		cnode* node=Interface::GetNodeAt(i);
		if(node->GetFlags()&FLAG_HIDDEN) continue;

		if(node->GetTypeID()!=TYPE_MESH&&node->GetTypeID()!=TYPE_SKIN)
			continue;

		cimage *tex=NULL;
		cmaterial *mat=node->GetMaterial();
		if(mat && mat->IsValidImage(MAT_IMG_DIFFUSE)) tex=mat->GetImage(MAT_IMG_DIFFUSE);

		// start position
		long pos_meshstart = ftell( file );

		// identifier
		ival=MD3_ID_HEADER;
		fwrite( &ival, 1, sizeof(ival), file );

		// mesh name
		fwrite( node->GetName(), 64, 1, file );
		ival=0;fwrite( &ival, 1, sizeof(ival),	file );// flags

		fwrite( &header.numboneframes, 1, sizeof(header.numboneframes), file );	// framenum

		if( tex ) ival=1;
		else ival=0;
		fwrite( &ival, 1, sizeof(ival), file);// skinnum
	
		long pos_vertexnum = ftell( file );
		ival=0;fwrite(&ival, 1, sizeof(ival), file);	// vertexnum
		if(node->GetTypeID()==TYPE_MESH) ival=((cmesh*)node)->NumFaces();
		else /*if(node->GetTypeID()==TYPE_SKIN)*/ ival=((cskin*)node)->NumFaces();
		fwrite(&ival, 1, sizeof(ival), file);	// trianglenum

		long pos_trianglestart = ftell( file );
		ival=0;fwrite(&ival, 1, sizeof(ival), file);	// trianglestart

		ival=108;fwrite(&ival, 1, sizeof(ival), file);	// headersize

		long pos_texvecstart = ftell( file );
		ival=0;fwrite(&ival, 1, sizeof(ival), file);	// texvecstart

		long pos_vertexstart = ftell( file );
		ival=16;fwrite(&ival, 1, sizeof(ival), file);	// vertexstart
		
		long pos_meshsize = ftell( file );
		ival=32;fwrite(&ival, 1, sizeof(ival), file);	// meshsize

		if( tex )	// write out a single 'skin'
		{
			// Once we have the filename we mangle it.  We search for
			// 'models' and if found we remove the filename to that part.
			// Thus if we have "C:/quake3/baseq3/models/mapobjects/texture.tga"
			// It becomes "models/mapobjects/texture.tga"
			// otherwise we just leave it.
			tex->GetPathName(mapname);
			subname = strstr( mapname, "models" );
			if( !g_q3_bUseModels || subname == NULL )
				fwrite( mapname, 64, 1, file );	// full texture filename
			else
				fwrite( subname, 64, 1, file );	// filename from 'models' on.
		}

		// Get info for triangles
		std::vector<i_Triangle>	vTriangles;
		if(node->GetTypeID()==TYPE_MESH) ival=((cmesh*)node)->NumFaces();
		else /*if(node->GetTypeID()==TYPE_SKIN)*/ ival=((cskin*)node)->NumFaces();
		vTriangles.resize( ival );

		for( int f = 0; f < ival; f++ )
		{
			cface face;
			if(node->GetTypeID()==TYPE_MESH) face=((cmesh*)node)->GetFace(f);
			else /*if(node->GetTypeID()==TYPE_SKIN)*/ face=((cskin*)node)->GetFace(f);

			vTriangles[f].a = face.v[0];
			vTriangles[f].b = face.v[1];
			vTriangles[f].c = face.v[2];
		}

		// Someplace to store extra vertices.  An extra vertex is simply a reference
		// to the 'real' vertex.  Also keep track of the current index of the extra's.
		std::list<int>	lExtraVerts;
		if(node->GetTypeID()==TYPE_MESH) ival=((cmesh*)node)->NumVerts();
		else /*if(node->GetTypeID()==TYPE_SKIN)*/ ival=((cskin*)node)->NumVerts();
		int	nextVert=ival;

		// Write out texture uv for each vertex.
		// In LR3D it is possible that there is not a one to one correspondence
		// between texture UVs and vertices.  In that case we try our best
		// and warn the user.
		bool fudged_uv = false;
		std::vector<UV_Coord>	uv_map;
		uv_map.resize( ival );

		// fill in uv_map by going through each tvFace of the mesh and getting
		// the appropriate TVert values.  Unless all the faces aren't textured
		// of course.  In which case we forget it and just make up our own uv
		// mapping with a bunch of (0.5,0.5) coords.
		if(node->GetTypeID()==TYPE_MESH) ival=((cmesh*)node)->NumTVerts();
		else /*if(node->GetTypeID()==TYPE_SKIN)*/ ival=((cskin*)node)->NumTVerts();
		if( ival )
		{
			if(node->GetTypeID()==TYPE_MESH) ival=((cmesh*)node)->NumFaces();
			else /*if(node->GetTypeID()==TYPE_SKIN)*/ ival=((cskin*)node)->NumFaces();

			for( int f = 0; f < ival; f++ )//faces
			{
				cface face;
				if(node->GetTypeID()==TYPE_MESH) face=((cmesh*)node)->GetFace(f);
				else /*if(node->GetTypeID()==TYPE_SKIN)*/ face=((cskin*)node)->GetFace(f);

				for( int j = 0; j < 3; j++ )
				{
					int _v	= face.v[j];//mesh.faces[i].getVert( j );
					int _tv	= face.tv[j];//mesh.tvFace[i].t[j];

					if( !uv_map[_v].filled )
					{
						cpoint3 tv;
						if(node->GetTypeID()==TYPE_MESH) tv=((cmesh*)node)->GetTVert(_tv);
						else /*if(node->GetTypeID()==TYPE_SKIN)*/ tv=((cskin*)node)->GetTVert(_tv);

						// this spot not yet taken
						uv_map[_v].u = tv.x;//(mesh.getTVert(_tv)).x;
						uv_map[_v].v = tv.y;//(mesh.getTVert(_tv)).y;
						uv_map[_v].filled = true;
						uv_map[_v].tvi = _tv;
					}
					else
					{
						// that spot already taken!
						// first check to see if the vertex already there is adequate.
						if( _tv == uv_map[_v].tvi )	continue;

						if(node->GetTypeID()==TYPE_MESH) ival=((cmesh*)node)->NumVerts();
						else /*if(node->GetTypeID()==TYPE_SKIN)*/ ival=((cskin*)node)->NumVerts();
						int _uvm = ival;//mesh.getNumVerts();
						bool found_one = false;
						// check if any of the vertices already added are adequate.
						for( std::list<int>::iterator ev_i = lExtraVerts.begin(); ev_i != lExtraVerts.end(); ev_i++ )
						{
							cpoint3 tv;
							if(node->GetTypeID()==TYPE_MESH) tv=((cmesh*)node)->GetTVert(_tv);
							else /*if(node->GetTypeID()==TYPE_SKIN)*/ tv=((cskin*)node)->GetTVert(_tv);

							if( *ev_i == _v && 
								uv_map[_uvm].u == tv.x &&//mesh.getTVert(_tv).x &&
								uv_map[_uvm].v == tv.y)//mesh.getTVert(_tv).y )
							{
								vTriangles[i].e[j] = _uvm;
								found_one = true;
								break;
							}
							_uvm++;
						}

						// if we didn't find a suitable vertex yet
						// we'll need to add a vertex so that everything is ok.
						if( !found_one )
						{
							cpoint3 tv;
							if(node->GetTypeID()==TYPE_MESH) tv=((cmesh*)node)->GetTVert(_tv);
							else /*if(node->GetTypeID()==TYPE_SKIN)*/ tv=((cskin*)node)->GetTVert(_tv);

							UV_Coord new_uv;
							
							fudged_uv = true;
							lExtraVerts.push_back( _v );
							vTriangles[i].e[j] = nextVert;
							new_uv.filled = true;
							new_uv.u = tv.x;//(mesh.getTVert(_tv)).x;
							new_uv.v = tv.y;//(mesh.getTVert(_tv)).y;
							new_uv.tvi = _tv;
							uv_map.push_back( new_uv );
							nextVert++;
						}
					}
				}
			}
		}
		else
		{
			if(node->GetTypeID()==TYPE_MESH) ival=((cmesh*)node)->NumVerts();
			else /*if(node->GetTypeID()==TYPE_SKIN)*/ ival=((cskin*)node)->NumVerts();

			for( int v = 0; v < ival; v++ )
			{
				cpoint3 n;// = mesh.getVert( i );
				if(node->GetTypeID()==TYPE_MESH) n=((cmesh*)node)->GetVertPoint(v);
				else /*if(node->GetTypeID()==TYPE_SKIN)*/ n=((cskin*)node)->GetVertPoint(v);


				cpoint3 _elv_t = n; _elv_t.y = 0;
				cpoint3 _rot_t = n; _rot_t.z = 0;
				cpoint3 _elv_n = Normalize( _elv_t );
				cpoint3 _rot_n = Normalize( _rot_t );
				/*
#ifdef _MAX2
				Point3 _elv_n = Normalize( _elv_t );
				Point3 _rot_n = Normalize( _rot_t );
#else				
				Point3 _elv_n = _elv_t.Normalize();
				Point3 _rot_n = _rot_t.Normalize();
#endif
				*/
				float _elv	= DotProd( z_axis, _elv_n );
				float _rot	= DotProd( x_axis, _rot_n );
				
				_elv = acos(_elv) / (2*PI) * 2;

				if( n.y >= 0 )
					_rot = acos(_rot) / (2*PI);
				else
					_rot = (1.0f - (acos(_rot)/(2*PI)));

				uv_map[i].u = _rot;
				uv_map[i].v = _elv;
			}
			if( g_q3_bEnableWarnings )
				MessageBox( GetActiveWindow(), "No UV mapping found for this object.\n"
												"Creating spherical mapping coordinates instead.", node->GetName(), MB_ICONWARNING);
		}

		if( fudged_uv && g_q3_bEnableWarnings )
		{
			MessageBox( GetActiveWindow(), "UV map may contain errors for this object.\n"
											"(There was not a one to one correspondence\n"
											"between texture vertices and true vertices.)", node->GetName(), MB_ICONWARNING);
		}

		// fill in trianglestart
		pos_current = ftell( file );
		fseek( file, pos_trianglestart, SEEK_SET );
		ival=pos_current - pos_meshstart;
		fwrite(&ival, 1, sizeof(ival), file);
		fseek( file, pos_current, SEEK_SET );

		// Write out triangles.
		if(node->GetTypeID()==TYPE_MESH) ival=((cmesh*)node)->NumFaces();
		else /*if(node->GetTypeID()==TYPE_SKIN)*/ ival=((cskin*)node)->NumFaces();
		for( f = 0; f < ival; f++ )
		{
			fwrite(&vTriangles[f].a, 1, sizeof(vTriangles[f].a), file);// vertex index
			fwrite(&vTriangles[f].b, 1, sizeof(vTriangles[f].b), file);// for 3 vertices
			fwrite(&vTriangles[f].c, 1, sizeof(vTriangles[f].c), file);// of triangle
		}

		// fill in texvecstart
		pos_current = ftell( file );
		fseek( file, pos_texvecstart, SEEK_SET );
		ival=pos_current - pos_meshstart;
		fwrite(&ival, 1, sizeof(ival), file);
		fseek( file, pos_current, SEEK_SET );

		// Write out UV mapping coords.
		for( int itv = 0; itv < uv_map.size(); itv++ )
		{
			fval=uv_map[itv].u;fwrite(&fval, 1, sizeof(float), file);// texture coord u,v
			fval=uv_map[itv].v;fwrite(&fval, 1, sizeof(float), file);// for vertex
		}

		// fill in vertexstart
		pos_current = ftell( file );
		fseek( file, pos_vertexstart, SEEK_SET );
		ival=pos_current - pos_meshstart;
		fwrite(&ival, 1, sizeof(ival), file);
		fseek( file, pos_current, SEEK_SET );

		// fill in vertexnum
		pos_current = ftell( file );
		fseek( file, pos_vertexnum, SEEK_SET );
		fwrite(&nextVert, 1, sizeof(nextVert), file);
		fseek( file, pos_current, SEEK_SET );

		// write out for each frame the position of each vertex
		short x, y, z;
		bool no_normals = false;
	//	if( !_mesh.normalsBuilt ) no_normals = true;
		
		// Premake an array with all the appropo verts and norms.
		std::vector<cpoint3> allVerts;
		std::vector<cpoint3> allNorms;
		allVerts.resize( nextVert );
		allNorms.resize( nextVert );

		//Parse all verts
		if(node->GetTypeID()==TYPE_MESH)
		{
			for( int j = 0; j < ((cmesh*)node)->NumVerts(); j++ )
			{
				allVerts[j] = ((cmesh*)node)->GetVertPoint(j);
				if( !no_normals )
					allNorms[j] = ((cmesh*)node)->GetNormal(j);
			}
			for( std::list<int>::iterator v_i = lExtraVerts.begin(); v_i != lExtraVerts.end(); v_i++ )
			{
				allVerts[j] = ((cmesh*)node)->GetVertPoint(*v_i);
				if( !no_normals )
					allNorms[j] = ((cmesh*)node)->GetNormal(*v_i);
				j++;
			}
		}

		//For each frame
		for( int t = g_q3_startframe; t <= g_q3_endframe; t++ ) // number of frames
		{
			cmatrix4 _tm = node->GetNodeTM(t);
			cmatrix4 _tmrot = node->GetNodeTM(t,TM_ROT);

			// If a relative_tag exists we transform vertices into the tags
			// space before writing them out.
			if( tag_relative != NULL )
			{
				cmatrix4 scratch = tag_relative->GetNodeTM(t);
				_tm = _tm*Inverse( scratch );
				scratch = tag_relative->GetNodeTM(t,TM_ROT);
				_tmrot = _tmrot*Inverse( scratch );
			}

			//Parse all verts for this frame
			if(node->GetTypeID()==TYPE_SKIN)
			{
				Array<cpoint3> varray;
				((cskin*)node)->GetVertArray(t,varray);
				Array<cpoint3> narray;
				((cskin*)node)->GetNormalArray(t,narray);

				for( int j = 0; j < ((cskin*)node)->NumVerts(); j++ )
				{
					allVerts[j] = varray[j];//((cskin*)node)->GetVert(j);
					if( !no_normals )
						allNorms[j] = narray[j];//((cskin*)node)->GetNormal(j);
				}
				for( std::list<int>::iterator v_i = lExtraVerts.begin(); v_i != lExtraVerts.end(); v_i++ )
				{
					allVerts[j] = varray[*v_i];//((cskin*)node)->GetVert(*v_i);
					if( !no_normals )
						allNorms[j] = narray[*v_i];//((cskin*)node)->GetNormal(*v_i);
					j++;
				}
			}


			//Write verts,norms
			for( int j = 0; j < nextVert; j++ ) // number of vertices
			{
				cpoint3 v = _tm*allVerts[j];

				// tag_floor is special.  We move everything down so its on the floor proper
			//	if( (g_relative_to_tag == tag_floor) && (relative_tag >= 0) )
			//		v.z += FLOOR_LEVEL;
				
				x = (short)(v.x * 64.0f);
				y = (short)(v.y * 64.0f);
				z = (short)(v.z * 64.0f);
				
				fwrite(&x, 1, sizeof(short), file);
				fwrite(&y, 1, sizeof(short), file);
				fwrite(&z, 1, sizeof(short), file);
				
				unsigned char _elevation, _rotation;
				
				if( no_normals )
				{
					short zero=0x0000;
					fwrite(&zero, 1, sizeof(short), file);
				}
				else
				{ 
					cpoint3 un_n	= allNorms[j];
					cpoint3 n = _tmrot*un_n;
					cpoint3 _elv_t = n; _elv_t.y = 0;
					cpoint3 _rot_t = n; _rot_t.z = 0;
					cpoint3 _elv_n = Normalize( _elv_t );
					cpoint3 _rot_n = Normalize( _rot_t );
					float _elv	= DotProd( z_axis, _elv_n );
					float _rot	= DotProd( x_axis, _rot_n );
					
					_elevation	= (unsigned char)( acos(_elv) / (2*PI) * 255 );
					
					if( n.y >= 0 )
						_rotation	= (unsigned char)( acos(_rot) / (2*PI) * 255 );
					else
						_rotation	= (unsigned char)( (1.0f - (acos(_rot)/(2*PI))) * 255 );
					
					fwrite(&_elevation, 1, 1, file);// normal elevation angle: 0 up, 63 sideways and 80 down.
					fwrite(&_rotation, 1, 1, file);// normal rotation: 0 straight along X, 40 straight along Y, 7f opposite X, c0 opposite Y
				}
			} // vertices

		}//frames

		if( no_normals && g_q3_bEnableWarnings )
		{
			MessageBox( GetActiveWindow(), "No normals for this object.\n"
											"Environment maps will be wrong.", node->GetName(), MB_ICONWARNING);
		}

		// fill in meshsize
		pos_current = ftell( file );
		fseek( file, pos_meshsize, SEEK_SET );
		ival=pos_current - pos_meshstart;
		fwrite(&ival, 1, sizeof(ival), file);
		fseek( file, pos_current, SEEK_SET );

	}//meshes

	// fill in filesize
	pos_current = ftell( file );
	fseek( file, pos_filesize, SEEK_SET );
	fwrite(&pos_current, 1, sizeof(pos_current), file);
	fseek( file, pos_current, SEEK_SET );

	fclose(file);


	return 1;
}