//LightRay3D SDK (C) 2002-2006 Tzivenis Sakis

#include <stdio.h>
#include "myassert.h"
#include <malloc.h>
#include <windows.h>

#include "plugapi.h"
#include "api.h"
#include "cmesh.h"
#include "impq2.h"
#include "interface.h"
#include "uniquename.h"//materials
#include "dlg.h"


//*.MD2
#define IDALIASHEADER	(('2'<<24)+('P'<<16)+('D'<<8)+'I')
#define ALIAS_VERSION	8

typedef struct
{
	short	s;
	short	t;
} dstvert_t;

typedef struct
{
	short	index_xyz[3];
	short	index_st[3];
} dtriangle_t;

typedef struct
{
	unsigned char	v[3];			// scaled byte to fit in frame mins/maxs
	char	lightnormalindex;
} dtrivertx_t;

typedef struct
{
	float		scale[3];	// multiply byte verts by this
	float		translate[3];	// then add this
	char		name[16];	// frame name from grabbing
	dtrivertx_t	verts[1];	// variable sized
} daliasframe_t;

typedef struct
{
	int			ident;
	int			version;

	int			skinwidth;
	int			skinheight;
	int			framesize;		// byte size of each frame

	int			num_skins;
	int			num_xyz;
	int			num_st;			// greater than num_xyz for seams
	int			num_tris;
	int			num_glcmds;		// dwords in strip/fan command list
	int			num_frames;

	int			ofs_skins;		// each skin is a MAX_SKINNAME string
	int			ofs_st;			// byte offset from start for stverts
	int			ofs_tris;		// offset for dtriangles
	int			ofs_frames;		// offset for first frame
	int			ofs_glcmds;	
	int			ofs_end;		// end of file
} dmdl_t;


static cmaterial* AddTexMaterial(const char* imagefile);

int cimpq2model::Import(const char *filename)
{
	dmdl_t  m;
	daliasframe_t *d;
	dtriangle_t   *dt;
	dstvert_t     *dst;
	char          skin[64];

	cmesh *mesh;
	int     i,iframe;

	FILE *f=fopen(filename,"rb");
	if (!f) return 0;
	
	fread(&m,1,sizeof(m),f);
	if (m.ident!=IDALIASHEADER || m.version!=ALIAS_VERSION)
	{
		fclose(f);
		MessageBox(::GetActiveWindow(),"Invalid Header!","MD2 File Error",0);
		return 0;
	}

	iframe=ChooseFrame(m.num_frames);

	mesh=CreateNewMesh();
	assert(mesh);
	if(!mesh->Allocate(m.num_xyz,m.num_st,m.num_tris))
	{
		fclose(f);
		mesh->Delete();
		return 0;
	}

	//Verts
	d=(daliasframe_t*)malloc(m.framesize);
	assert(d);
	fseek(f,m.ofs_frames+(iframe*m.framesize),SEEK_SET);
	fread(d,1,m.framesize,f);
	for (i=0;i<m.num_xyz;i++)
	{
		cpoint3 v;
		v.x=((float)d->verts[i].v[0])*d->scale[0]+d->translate[0];
		v.y=((float)d->verts[i].v[1])*d->scale[1]+d->translate[1];
		v.z=((float)d->verts[i].v[2])*d->scale[2]+d->translate[2];
		mesh->SetVert(i,v);
	}
	free(d);
	
	//st Verts
	dst=(dstvert_t*)malloc(m.num_st*sizeof(dstvert_t));
	assert(dst);
	fseek(f,m.ofs_st,SEEK_SET);
	fread(dst,1,m.num_st*sizeof(dstvert_t),f);
	//Before setting 'em find max u,v values
	float u=0,v=0;
	for (i=0;i<m.num_st;i++)
	{
		if((float)abs(dst[i].s)>fabs(u)) u=(float)dst[i].s;
		if((float)abs(dst[i].t)>fabs(v)) v=(float)dst[i].t;
	}
	for (i=0;i<m.num_st;i++)
	{
		cpoint3 tv;
		tv.x=(float)dst[i].s/u;//128.0f
		tv.y=(float)dst[i].t/v;//128.0f
		mesh->SetTVert(i,tv);
	}
	free(dst);
	
	//Tris
	dt=(dtriangle_t*)malloc(m.num_tris*sizeof(dtriangle_t));
	assert(dt);
	fseek(f,m.ofs_tris,SEEK_SET);
	fread(dt,1,m.num_tris*sizeof(dtriangle_t),f);
	for (i=0;i<m.num_tris;i++)
	{
		cface face;
		for(int j=0;j<3;j++)
		{
			face.v[j]=dt[i].index_xyz[j];
			face.tv[j]=dt[i].index_st[j];
		}
		// Enable correct edges.
		face.wFlags = EDGE_ALL;
		mesh->SetFace(i,face);
	}
	free(dt);
	
	mesh->Smooth();//just to build normals

	//Skin
	if(m.num_skins)
	{
		fseek(f,m.ofs_skins,SEEK_SET);
		fread(skin,1,64,f);
		
		cmaterial *mat=AddTexMaterial(skin);
		if(mat) mesh->SetMeshMaterial(mat);
	}
	
	fclose(f);

	Interface::AddToScene(mesh);
	
	return 1;
}


cmaterial* AddTexMaterial(const char* imagefile)
{
	char texture[256];
	strcpy(texture,imagefile);
	strlwr(texture);

	cimage image;
	if(!image.Load(texture)) return NULL;
	
	cmaterial *mat=CreateNewMaterial();
	assert(mat);
	mat->SetImage(image,MAT_IMG_DIFFUSE);

	cbitarray matarray;
	Interface::GetUnusedMaterials(matarray);//get the indices of the unused mats
	if(matarray.NumBitsSet())//if there is one or more empty mats
	{
		for(int im=0;im<matarray.GetSize();im++)//search for unused entry
		{
			if(matarray[im])//bit is 1 so this is empty
			{
				cmaterial *emptymat=Interface::GetMaterialAt(im);//get pointer
				emptymat->Copy(mat);//just copy material
				//name doesn't get copied so we must set it if not the same
				if(strcmp(emptymat->GetName(),mat->GetName()))
				{
					UniqueNameForMaterial(mat->GetName());//make the name unique first
					emptymat->SetName(mat->GetName());
				}
				mat->Delete();//delete the array mat since it's copied
				//after deleting set the scene's emptymat pointer back
				mat=emptymat;
				matarray.Set(im,0);//entry is used now,so set bit to 0
				break;
			}
		}
	}
	else Interface::AddToScene(mat);//No empty entries.Just add new.

//	char ach[255]="";
//	mat->GetImage(MAT_IMG_DIFFUSE)->GetName(ach);
//	MessageBox(::GetActiveWindow(),ach,"Debug Image",0);

	return mat;
}
