//LightRay3D SDK (C) 2002-2006 Tzivenis Sakis

#include <stdio.h>
#include "myassert.h"
#include <malloc.h>
#include <windows.h>

#include "plugapi.h"
#include "api.h"
#include "cmesh.h"
#include "cbone.h"
#include "impq3.h"
#include "interface.h"
#include "uniquename.h"//materials
#include "dlg.h"

#include "quake3.h"


static cmaterial* AddTexMaterial(const char* imagefile);

int cimpq3model::Import(const char *filename)
{
	BYTE				*fdata,*data,*mdata;
	md3header_t			*header;
	md3mesh_file_t		*mesh_header;
	md3tag_t			*tags=NULL;
	char				skin[64]="";

	cmesh **mesh=NULL;
	int nmeshes=0;

	int i,j,size,iframe;//,numv,numtv,numf;

	FILE *f=fopen(filename,"rb");
	if (!f) return 0;
	fseek(f, 0, SEEK_END);
	size = ftell(f);
	fseek(f, 0, SEEK_SET);

	fdata=(BYTE*)malloc(size);
	assert(fdata);
	fread(fdata,1,size,f);
	fclose(f);

	data=fdata;

	//Check header
	header = (md3header_t *)data;
	if (header->id != MD3_ID_HEADER || header->version != MD3_ALIAS_VERSION)
	{
		MessageBox(::GetActiveWindow(),"Invalid Header!","MD3 File Error",0);
		free(fdata);
		return 0;
	}

	// Check frames
	if (!(header->numboneframes && header->filesize > header->bone_offs))
	{
		MessageBox(::GetActiveWindow(),"No frames found!","MD3 File Error",0);
		free(fdata);
		return 0;
	}

	// Meshes number of elements
//	numv=numtv=numf=0;
	nmeshes=0;
	if (header->nummeshes && (header->filesize > header->mesh_offs))
	{
		data = fdata + header->mesh_offs;

		for (i = 0; i < header->nummeshes; i++)
		{
			// mesh header
			mesh_header = (md3mesh_file_t *) data;

			if (mesh_header->id != MD3_ID_HEADER)
			{
				MessageBox(::GetActiveWindow(),"Invalid mesh header id!","MD3 File Error",0);
			//	free(fdata);
			//	return 0;
				goto abort;
			}

		//	numv+=mesh_header->numverts;
		//	numtv+=mesh_header->numverts;
		//	numf+=mesh_header->numtris;

			cmesh *m=CreateNewMesh();
			assert(m);
			if(!m->Allocate(mesh_header->numverts,mesh_header->numverts,mesh_header->numtris))
			{
			//	free(fdata);
			//	m->Delete();
			//	return 0;
				m->Delete();
				goto abort;
			}
			m->SetName(mesh_header->name);
			mesh=(cmesh**)realloc(mesh,(nmeshes+1)*sizeof(cmesh*));
			assert(mesh);
			mesh[nmeshes]=m;
			nmeshes++;

			data += mesh_header->meshsize;
		}
	}
	/*
	if(!numv || !numf)
	{
		MessageBox(::GetActiveWindow(),"Zero number of mesh elements!","MD3 File Error",0);
		free(fdata);
		return 0;
	}
	*/
	if(!nmeshes)
	{
		MessageBox(::GetActiveWindow(),"No meshes found in file!","MD3 File Error",0);
		goto abort;
	}

	iframe=ChooseFrame(header->numboneframes);

	/*
	//Mesh
	mesh=CreateNewMesh();
	assert(mesh);
	if(!mesh->Allocate(numv,numtv,numf))
	{
		free(fdata);
		mesh->Delete();
		return 0;
	}
	*/

	// Tags
	if (header->numtags && (header->filesize > header->tag_offs))
	{
		//goto frame
		data = fdata + header->tag_offs + (iframe * header->numtags * sizeof(md3tag_t));

		tags = (md3tag_t *) malloc(header->numtags * sizeof(md3tag_t));
		assert(tags);
		
		for (i = 0; i < header->numtags; i++)
		{
			// name
			memcpy(tags[i].name, data, 64);
			data += 64;
			
			// position
			tags[i].pos[0] = *((float*)data);
			data += sizeof(float);
			tags[i].pos[1] = *((float*)data);
			data += sizeof(float);
			tags[i].pos[2] = *((float*)data);
			data += sizeof(float);
			
			// rotation
			tags[i].rot[0] = *((float*)data);
			data += sizeof(float);
			tags[i].rot[1] = *((float*)data);
			data += sizeof(float);
			tags[i].rot[2] = *((float*)data);
			data += sizeof(float);
			tags[i].rot[3] = *((float*)data);
			data += sizeof(float);
			tags[i].rot[4] = *((float*)data);
			data += sizeof(float);
			tags[i].rot[5] = *((float*)data);
			data += sizeof(float);
			tags[i].rot[6] = *((float*)data);
			data += sizeof(float);
			tags[i].rot[7] = *((float*)data);
			data += sizeof(float);
			tags[i].rot[8] = *((float*)data);
			data += sizeof(float);
		}
	}

	// Meshes elements
	if (header->nummeshes && (header->filesize > header->mesh_offs))
	{
		mdata = fdata + header->mesh_offs;

	//	numv=numtv=numf=0;
		for (i = 0; i < header->nummeshes; i++)
		{
			// mesh header
			mesh_header = (md3mesh_file_t *) mdata;

			if (mesh_header->id == MD3_ID_HEADER)
			{
				// vertices
				if (mesh_header->numverts && (mesh_header->meshsize > mesh_header->vert_offs))
				{
					//goto frame - verts
					data = mdata + mesh_header->vert_offs + (iframe * mesh_header->numverts * 8);

					for (j = 0; j < mesh_header->numverts; j++)
					{
						cpoint3 v;

						v.x=(float)(*((short *)data)) * MD3_XYZ_SCALE;
						data += sizeof(short);
						v.y=(float)(*((short *)data)) * MD3_XYZ_SCALE;
						data += sizeof(short);
						v.z=(float)(*((short *)data)) * MD3_XYZ_SCALE;
						data += sizeof(short);
						//pass normals(2 bytes)
						data+=2;

						mesh[i]->SetVert(j/*+numv*/,v);
					}
				}

				// texcoords
				if (mesh_header->numverts && (mesh_header->meshsize > mesh_header->tc_offs))
				{
					data = mdata + mesh_header->tc_offs;

					for (j = 0; j < mesh_header->numverts; j++)
					{
						cpoint3 tv;
						tv.x=*((float *)data);
						data += sizeof(float);
						tv.y=*((float *)data);
						data += sizeof(float);
						mesh[i]->SetTVert(j/*+numtv*/,tv);
					}
				}

				// triangles
				if (mesh_header->numtris && (mesh_header->meshsize > mesh_header->elem_offs))
				{
					data = mdata + mesh_header->elem_offs;

					for (j = 0; j < mesh_header->numtris; j++)
					{
						cface face;
						face.v[0]=face.tv[0]=*((long *)data);//+numv;
						data += sizeof(long);
						face.v[1]=face.tv[1]=*((long *)data);//+numv;
						data += sizeof(long);
						face.v[2]=face.tv[2]=*((long *)data);//+numv;
						data += sizeof(long);
						// Enable correct edges.
						face.wFlags = EDGE_ALL;
						mesh[i]->SetFace(j/*+numf*/,face);
					}
				}

				// skins
				if (mesh_header->numskins && (mesh_header->meshsize > mesh_header->skin_offs))
				{
					BYTE* skindata = mdata + mesh_header->skin_offs;
					//Load skin here
					memcpy(skin,skindata,64);

				//	char texpath[256];
				//	memset(texpath,0,256);
				//	strcpy(texpath,filename);
				//	char *ptr=strstr(texpath,"models");
				//	if(ptr)
				//	{
				//		strcpy(ptr,skin);
				//		cmaterial *mat=AddTexMaterial(texpath);
				//		if(mat) mesh[i]->SetMeshMaterial(mat);
				//	}
					cmaterial *mat=AddTexMaterial(skin);
					if(mat) mesh[i]->SetMeshMaterial(mat);
				}

			}//valid MD3_ID_HEADER
			else assert(0);

		//	numv+=mesh_header->numverts;
		//	numtv+=mesh_header->numverts;
		//	numf+=mesh_header->numtris;

			mdata += mesh_header->meshsize;
		}//meshes loop
	}//meshes valid

	//Choose tag
	int itag;itag=-1;
	for(i=0;i<header->numtags;i++)
		if(stricmp(tags[i].name,"tag_torso")==0) {itag=i;break;}
	for(i=0;itag==-1&&i<header->numtags;i++)
		if(stricmp(tags[i].name,"tag_head")==0) {itag=i;break;}
	for(i=0;itag==-1&&i<header->numtags;i++)
		if(stricmp(tags[i].name,"tag_weapon")==0) {itag=i;break;}

	//Correct mesh relative to tag
	if(itag!=-1)
	{
		cmatrix4 tm;
		tm.SetRow(0,MakePt3(tags[itag].rot[0],tags[itag].rot[1],tags[itag].rot[2]));
		tm.SetRow(1,MakePt3(tags[itag].rot[3],tags[itag].rot[4],tags[itag].rot[5]));
		tm.SetRow(2,MakePt3(tags[itag].rot[6],tags[itag].rot[7],tags[itag].rot[8]));
		tm.SetRow(3,MakePt3(tags[itag].pos[0],tags[itag].pos[1],tags[itag].pos[2]));
		cmatrix4 invtm=Inverse(tm);
		for(i=0;i<nmeshes;i++)
		{
			for(int iv=0;iv<mesh[i]->NumVerts();iv++)
			{
				cpoint3 v=mesh[i]->GetVertPoint(iv);
				v=invtm*v;
				mesh[i]->SetVert(iv,v);
			}
		}
	}
	//Create new tags if not already in scene
	for(i=0;i<header->numtags;i++)
	{
		cnode* tagnode=Interface::FindNode(tags[i].name);
		if(!tagnode)
		{
			cmatrix4 tm;
			tm.SetRow(0,MakePt3(tags[i].rot[0],tags[i].rot[1],tags[i].rot[2]));
			tm.SetRow(1,MakePt3(tags[i].rot[3],tags[i].rot[4],tags[i].rot[5]));
			tm.SetRow(2,MakePt3(tags[i].rot[6],tags[i].rot[7],tags[i].rot[8]));

			cbone* bone=CreateNewBone();
			bone->SetName(tags[i].name);
			bone->SetPosition(TIME_IDLE,MakePt3(tags[i].pos[0],tags[i].pos[1],tags[i].pos[2]));
			bone->SetRotation(TIME_IDLE,QuatFromMatrix(tm));
			Interface::AddToScene(bone);
		}
	}
	//Meshes relative to loaded tags
	cnode *tag;tag=NULL;
	if(itag>-1) tag=Interface::FindNode(tags[itag].name);
	if(tag)
	{
		for(i=0;i<nmeshes;i++)
		{
			mesh[i]->SetPosition(TIME_IDLE,tag->GetPosition(TIME_IDLE));
			mesh[i]->SetRotation(TIME_IDLE,tag->GetRotation(TIME_IDLE));
		}
	}

	for(i=0;i<nmeshes;i++)
	{
		mesh[i]->Smooth();//just to build normals
		Interface::AddToScene(mesh[i]);
	}

#ifndef NDEBUG	
	//Stats
	char ach[256];
	sprintf(ach,"file: %s\nframes: %d\nskin images: %d\ntags: %d\nmeshes: %d\n",
		header->filename,header->numboneframes,header->numskins,
		header->numtags,header->nummeshes);
	MessageBox(::GetActiveWindow(),ach,"Stats",0);

//	sprintf(ach,"tags_offs:%d, meshes_ofs:%d, bones_offs:%d",
//		header->tag_offs,header->mesh_offs,header->bone_offs);
//	MessageBox(::GetActiveWindow(),ach,"Debug",0);

#endif

	free(fdata);
	if(tags) free(tags);
	if(mesh) free(mesh);

	return 1;

abort:
	free(fdata);
	if(tags) free(tags);
	if(mesh)
	{
		for(i=0;i<nmeshes;i++)
			mesh[i]->Delete();
		free(mesh);
	}
	
	return 0;
}


cmaterial* AddTexMaterial(const char* imagefile)
{
	char texture[256];
	strcpy(texture,imagefile);
	strlwr(texture);

	//First search for same texture(may load parts of model that share same image)
	for(int i=0;i<Interface::NumMaterials();i++)
	{
		cmaterial *m=Interface::GetMaterialAt(i);

		char imgname[256];m->GetImage(MAT_IMG_DIFFUSE)->GetPathName(imgname);
		strlwr(imgname);

		if(strstr(imgname,texture)) return m;
		//Try both TGA-JPG
		if(strstr(texture,".tga"))
			strcpy(strrchr(texture,'.'),".jpg");
		else
			strcpy(strrchr(texture,'.'),".tga");
		if(strstr(imgname,texture)) return m;
	}

	cimage image;
	if(!image.Load(texture))
	{
		//Try both TGA-JPG
		if(strstr(texture,".tga"))
			strcpy(strrchr(texture,'.'),".jpg");
		else
			strcpy(strrchr(texture,'.'),".tga");

		if(!image.Load(texture)) return NULL;
	}

	
	cmaterial *mat=CreateNewMaterial();
	assert(mat);
	mat->SetImage(image,MAT_IMG_DIFFUSE);

	cbitarray matarray;
	Interface::GetUnusedMaterials(matarray);//get the indices of the unused mats
	if(matarray.NumBitsSet())//if there is one or more empty mats
	{
		for(int im=0;im<matarray.GetSize();im++)//search for unused entry
		{
			if(matarray[im])//bit is 1 so this is empty
			{
				cmaterial *emptymat=Interface::GetMaterialAt(im);//get pointer
				emptymat->Copy(mat);//just copy material
				//name doesn't get copied so we must set it if not the same
				if(strcmp(emptymat->GetName(),mat->GetName()))
				{
					UniqueNameForMaterial(mat->GetName());//make the name unique first
					emptymat->SetName(mat->GetName());
				}
				mat->Delete();//delete the array mat since it's copied
				//after deleting set the scene's emptymat pointer back
				mat=emptymat;
				matarray.Set(im,0);//entry is used now,so set bit to 0
				break;
			}
		}
	}
	else Interface::AddToScene(mat);//No empty entries.Just add new.

//	char ach[255]="";
//	mat->GetImage(MAT_IMG_DIFFUSE)->GetName(ach);
//	MessageBox(::GetActiveWindow(),ach,"Debug Image",0);

	return mat;
}