// Lightray3D (C) 2002-2006 Tzivenis Sakis

#ifndef ICREATE_H
#define ICREATE_H

#include "iclass.h"
#include "glwnd.h"

//Standard categories(Note: !!ORDER IS ESSENTIAL!!)
#define CATEGORY_BONES		"Bones"
#define CATEGORY_CAMERAS	"Cameras"
#define CATEGORY_SHAPES		"Shapes"
#define CATEGORY_PRIMITIVES	"Primitives"
#define CATEGORY_LIGHTS		"Lights"
#define CATEGORY_FREEFORM	"Free-Form"
#define NUM_STANDARD_CATEGORIES 6


class CGLWindow;

class COREAPI ccreateinterface : public cclassinterface
{
public:
	//From cclassinterface
	//{
	char *GetCategory(){return "";}
	char *GetName(){return "";}
	//NOTE:Normally this is always TRUE
	BOOL IsPublic(){return TRUE;}//Return FALSE if this class isn't shown in Control Panel
	//Descendants must return their object's standard type(e.g. primitives shall return TYPE_MESH)
	UINT GetType(){return 0;}
	//Descendants must return their object's own type(e.g. primitives shall return PRIM_PLANE,PRIM_CUBE etc.)
	UINT GetSubType(){return 0;}
	//Descendants must return their object(e.g. primitives shall return cplane,ccube etc.)
	void* Create(){return NULL;}
	//If having a dialog must handle messages
	BOOL DlgProc(HWND hDlg,UINT uMsg,WPARAM wParam,LPARAM lParam){return FALSE;}
	//(ver.>=1.3.6 update1):Generic rendering(see base class cclassinterface)
	void Display(int attime,CGLWindow*,UINT renderflags){}
	//}



	virtual void Init(){}//called on entering creation mode
	virtual void Done(){}//called on exiting creation mode

	//Mouse funcs
	//(ver.>=1.3.3):Changed return parameter from void to BOOL
	//Returns TRUE if handled, FALSE if not.

	//(ver.>=1.3.6 update1):Called before Left button down.
	//Return FALSE if creation needs more left clicks.
	virtual BOOL CanCreateNew(){return TRUE;}

	//Left button down
	virtual BOOL BeginDrag(CGLWindow *glwnd,int scrx,int scry,
						   float x,float y,float z,
						   float dirx,float diry,float dirz,
						   float upx,float upy,float upz){return FALSE;}
	//Left button down and drag
	virtual BOOL Drag(CGLWindow *glwnd,int scrx,int scry,int scrdx,int scrdy,
					  float x,float y,float z,float szx,float szy){return FALSE;}
	//Left button up
	virtual BOOL EndDrag(CGLWindow *glwnd,int scrx,int scry,int scrdx,int scrdy,
						 float x,float y,float z,float szx,float szy){return FALSE;}
	//(ver.>=1.3.3);These two are added
	//No buttons down.Just cursor moving.
	virtual BOOL DragIdle(CGLWindow *glwnd,int scrx,int scry,int scrdx,int scrdy,
						  float x,float y,float z,float szx,float szy){return FALSE;}
	//Right mouse button down.Stop "current" creation process.
	virtual BOOL Stop(CGLWindow *glwnd,int scrx,int scry,int scrdx,int scrdy,
					  float x,float y,float z,float szx,float szy){return FALSE;}

	//(ver.>=1.3.6 update1):Ask for view mouse capture while idle dragging.
	virtual BOOL DragIdleMouseCapture(){return FALSE;}

	//(ver.>=1.3.6 update1):HBITMAP icon for listbox( 24x24 )
	virtual void* GetHBitmap(){return NULL;}
	virtual void DeleteHBitmap(void*hbmp){}
};

COREAPI BOOL RegisterInterface(ccreateinterface*);
COREAPI void UnregisterCreateInterfaces();
COREAPI int GetNumCreateInterfaces();
COREAPI ccreateinterface* GetCreateInterface(int i);
COREAPI ccreateinterface* GetCreateInterfaceByName(const char*);
COREAPI ccreateinterface* GetCreateInterfaceClass(UINT type,UINT subtype);
//(ver.>=1.3.6 update1)
COREAPI void SetActiveCreateInterface(ccreateinterface*);
COREAPI ccreateinterface* GetActiveCreateInterface();


COREAPI void *CreateInterfaceCreate(UINT type,UINT subtype);

#endif //ICREATE_H