// Lightray3D (C) 2002-2006 Tzivenis Sakis

#ifndef INTERFACE_H
#define INTERFACE_H

//Added Fog (versions >=1.3)

//View windows ids
#define I_TOP_LEFT_VIEW		0
#define I_TOP_RIGHT_VIEW	1
#define I_BOTTOM_LEFT_VIEW	2
#define I_BOTTOM_RIGHT_VIEW	3
#define I_NUM_VIEWS			4

//Pick object from scene list dialog flags
#define I_PICK_LIST_SINGLE		0
#define I_PICK_LIST_MULTI		1
#define I_PICK_LIST_LIGHTS		1
#define I_PICK_LIST_CAMERAS		2
#define I_PICK_LIST_BONES		4
//(ver.>=1.3.5):Detailed Geom Objects
//#define I_PICK_LIST_GEOM		8
//#define I_PICK_LIST_ALL			15
#define I_PICK_LIST_MESHES		8
#define I_PICK_LIST_SKINS		16
#define I_PICK_LIST_SHAPES		32
#define I_PICK_LIST_GEOM		(I_PICK_LIST_MESHES|I_PICK_LIST_SKINS|I_PICK_LIST_SHAPES)
#define I_PICK_LIST_ALL			63


class cnode;
class cmaterial;
class ccolor;
class cpoint3;
class cimage;
class CGLWindow;
class cbitarray;
class cimportinterface;
class cexportinterface;


//Scene List Callbacks.
//SCENE_LIST_VALIDATE will be called for each object selected in SceneList.Return TRUE if this object is valid,FALSE if invalid.
//SCENE_LIST_PICKED will be called for each object when Select button is pressed.
typedef int SCENE_LIST_VALIDATE(cnode*);
typedef void SCENE_LIST_PICKED(cnode*);
/*
//(ver.>=1.3.3):Cursor Picking Callbacks
typedef void PICKCALLBACK (CGLWindow*glwnd,int x,int y);
typedef void SCENELISTCALLBACK (void);
*/

//(ver.>=1.3.5): GUI - subclassing dialogs,controls
//Messages send after initdialog.Handle them in dialog's WindowProc
#ifndef WM_USER
#define WM_USER						0x0400
#endif
#define WM_LR3D_HOOK_SUBCLASS			WM_USER+4096//request subclassing permission - wParam==BOOL* pointer - lParam==0 if dlg else control's ID
#define WM_LR3D_HOOK_DRAWITEM			WM_USER+4097//let user handle extra painting
#define WM_LR3D_HOOK_BTNTYPE			WM_USER+4098//request button type
#define WM_LR3D_HOOK_HANDLECTLCOLOR		WM_USER+4099//let user handle WM_ONCTLCOLOR message

class COREAPI Interface
{
public:

	static unsigned long GetAppVersion();//returns app build version.
	static void* GetMainHINSTANCE();//returns application's HINSTANCE
	static void* GetMainHWND();//returns application's main HWND
	//Application's installation path (e.g. C:\\Lightray3D)
	static const char* GetMainPath();

	//(ver.>=1.3.5):Color picker dialog, Image File Open dialog
	static BOOL DlgPickColor(COLORREF& col);
	static BOOL DlgLoadImage(char* filename);//filename must be at least 255 chrs long
	static int JPGQualityDlg();//If using sxImgLib.Returns quality %(0-100)

	//(ver.>=1.3.5):Image Viewer Window
	//NOTE: image window allocates memory for image bits which are converted to 32bpp.You can free original image bits,palette mem.
	static BOOL CreateImageWnd(int sx=320,int sy=240);//Must first create it.If already created sx,sy will NOT size wnd.
	static void CloseImageWnd();
	//bRGB2BGR if TRUE will swap r,b components / bFlipVert if TRUE will flip image vertically.
	static BOOL ShowImage(cmaterial* mat,int imgtype);//imgtype: one of materials's: MAT_IMG_DIFFUSE,MAT_IMAGE_OPACITY etc.(see cmaterial.h)
	static BOOL ShowImage(cimage* image,char *wndtitle,BOOL bRGB2BGR=FALSE,BOOL bFlipVert=TRUE);
	static BOOL ShowImage(const char* filename,char *wndtitle,BOOL bRGB2BGR=FALSE,BOOL bFlipVert=TRUE);//load an image file
	static BOOL ShowImage(void *bits,void *palette,int sizex,int sizey,int bpp,BOOL bRGB2BGR=FALSE,BOOL bFlipVert=TRUE);

	/*
	//(ver.>=1.3.3):Added picking callbacks
	void PickCallback(PICKCALLBACK* func);
	void SceneListCallback(SCENELISTCALLBACK* func);
	*/
	//(ver.>=1.3.5):Same as AddToScene(cnode*) func except that we use a cnode* array for multiple insertions(i.e >1000).
	static void AddNodeArrayToScene(cnode **array,int nelems,
									BOOL bRawInsertion=FALSE);//If bRaw==TRUE then no checkings are made(unique name etc.).Speeds up things.
	//(ver.>=1.3.5):Same as AddToScene(cmaterial*) func except that we use a cmaterial* array for multiple insertions(i.e >1000).
	static void AddMatArrayToScene(cmaterial **array,int nelems,
									BOOL bRawInsertion=FALSE);//If bRaw==TRUE then no checkings are made(unique name etc.).Speeds up things.

	//Add a new node in scene if not already added
	//NOTE: If in create mode, node gets selected too
	static void AddToScene(cnode *node,BOOL bRawInsertion=FALSE);//(ver.>=1.3.5):If bRaw==TRUE then no checkings are made(unique name etc.).Speeds up things.
	//Remove from scene.Set TRUE if you delete,will remove,notify other nodes,detach children,detached from parent.
	static void RemoveFromScene(cnode *node,BOOL bIsBeingDeleted=TRUE);
	//Add a new material in scene if not already added
	static void AddToScene(cmaterial *mat,BOOL bRawInsertion=FALSE);//(ver.>=1.3.5):If bRaw==TRUE then no checkings are made(unique name etc.).Speeds up things.
	//Will return single node selected else NULL if none or more than one selected
	static cnode* GetSelNode();
	//Returns index in list.-1 if not found
	static int Find(cnode *node);
	//Find node by name
	static cnode* FindNode(const char *name);
	//Get node at index
	static cnode* GetNodeAt(int index);//index is temporary,node may change position in list
	//Returns index in list.-1 if not found
	static int Find(cmaterial *mat);
	//Find material by name
	static cmaterial* FindMaterial(const char *name);
	//Get material at index
	static cmaterial* GetMaterialAt(int index);//index is temporary,material may change position in list
	//If material is used by a node or its sub-elements returns TRUE
	static BOOL IsMaterialUsed(cmaterial *mat);
	//Use this to get unused materials state.(useful for replacing materials instead of adding while importing)
	static void GetUnusedMaterials(cbitarray& array);
	//Set ambient color for scene
	static void SetSceneAmbient(const ccolor& c);
	//Get ambient color of scene
	static ccolor GetSceneAmbient();
	//Set background color for scene
	static void SetSceneBkColor(const ccolor& c);
	//Get background color of scene
	static ccolor GetSceneBkColor();
	//Set background image for scene
//	static void SetSceneBkImage(const cimage&);
//	static BOOL SetSceneBkImage(const char*filename);
	//Get background image of scene
	static cimage* GetSceneBkImage();//Loading image example:GetSceneBkImage()->Load(filename);
	//Fog values(versions>=1.3)
	static void SetFogValues(const cpoint3& fogcolor,float fogdensity,float fogstart,float fogend);
	static void GetFogValues(cpoint3& fogcolor,float& fogdensity,float& fogstart,float& fogend);
	static void EnableFog(BOOL);
	static BOOL IsFogEnabled();
	//Will return current sub-level mode(if a node is selected)
	static int GetSubLevel();
	//Will return nodes selection or sub-elements selection center according to current state
	static cpoint3 GetSelCenter();
	//Scene notes
	static void CopyNotes(const char* notes,int size);//will replace current notes array
	static void AddNotes(const char* notes,int size);//will append notes sizing it accordingly
	static void ClearNotes();//(ver.>=1.3.3):will clear notes
	static char* GetNotes();//will return a temp pointer
	static int NotesSize();
	// Pick object from scene list(ValidateObject callback should return TRUE if node is OK, PickedObject(will be called after ValidateObject returns TRUE)
	//								should handle operations on passed picked node)
	// Returns FALSE if CANCEL button is pressed.
	// ValidateObject and PickedObject callbacks must not be NULL!!
	static BOOL PickFromList(UINT singleormulti,UINT objtypes,char *title,char *button,SCENE_LIST_VALIDATE* ValidateObject,SCENE_LIST_PICKED* PickedObject);//int(*ValidateObject)(cnode*),void(*PickedObject)(cnode*));

	//Use these funcs to select,hide and not flags as you would do with (e.g)mesh verts,faces,edges
	static void Select(cnode *node);//Must not be hidden(~FLAG_HIDDEN)
	static void Deselect(cnode *node);
	static void DeselectAll();
	static void Hide(cnode *node);//Will get rid of FLAG_SELECTED
	static void Unhide(cnode *node);
	static void UnhideAll();

	static int NumNodes();//Returns number of all nodes in scene
	static int NumSelNodes();//Returns number of selected nodes in scene
	static int NumHiddenNodes();//Returns number of hidden nodes in scene
	static int NumMaterials();//Returns number of materials in scene
	static int NumUsedMaterials();//Returns number of materials used by nodes in scene

	//(ver.>=1.3.5):Nodes hierarchy indices in scene list
	static BOOL GetNodesHierarchy(cbitarray& array);

	//Update all viewports
	static void UpdateViews();
	//Update active/focused viewport only
	static void UpdateActiveView();
	//Active glview
	static CGLWindow* GetActiveGLView();
	//By id
	static CGLWindow* GetGLView(int id);

	//Reset animation to default values
	static void ResetAnim();
	//Set animation current frame(start<=frame<=end otherwise will be clamped)
	static void SetAnimTime(int frame);
	//Set animation start frame(start<=frame<=end otherwise will be clamped)
	static void SetAnimStart(int frame);
	//Set animation end frame(0<=frame otherwise will be clamped)
	static void SetAnimEnd(int frame);
	//Set animation ticks per frame(0<ticksperframe)
	static void SetAnimTPF(int ticksperframe);
	//Set anim REC flag.SetPosition,SetRotation etc funcs will be recorded
	static void SetAnimOn(BOOL);
	static BOOL IsAnimOn();
	//NOTE:anim.h functions can be used instead of these below
	static int GetAnimTime();
	static int GetAnimStart();
	static int GetAnimEnd();
	static int GetAnimTPF();
	//(ver.>=1.3.3):Animation segment.First,Last key times.
	//Can set NULL to one of parameters if not needed.
	static BOOL GetAnimSeg(int *first,int *last);//FALSE if no keys set

	//(ver.>=1.3.3):World CS BBox for nodes.Returns false if no nodes in scene.
	static BOOL GetWorldBBox(int attime,cpoint3& min,cpoint3& max,BOOL bSelectedOnly=FALSE,BOOL bExcludeHidden=TRUE);

	//(ver.>=1.3.4)
	//(ver.>=1.3.5):Let user choose between one or all views & selected or all objects
	//Defaults are all views/all objects
	static void FocusViews(int iview=I_NUM_VIEWS,BOOL bSelOnly=FALSE);//when importing this func already called by app
	//Scene update funcs
	static void UpdateNodes();//i.e 'UpdateTimeIdle' for all nodes
	static void UpdateMisc();//i.e Update Sel Tripod graph
	static void UpdateSelPos();//(ver.>=1.3.6):Update Sel Tripod,calc bbox

	//(ver.>=1.3.4):Application funcs
	static void UpdateAnimBar();
	static void UpdateControlPanel();
	static void UpdateToolbars();
	static void MaximizeActiveView(BOOL bMax);
	static void MaximizeView(int id);//by id
	static BOOL IsActiveViewMaximized();
	static BOOL IsViewMaximized(int id);//by id
	static BOOL IsViewEnabled(int id);
	static void EnableView(int id,BOOL bEnable);
	static void SetActiveView(int id);//set focus
	static int ActiveViewID();//get focused view id
	static int SetCameraToView(int id,cnode* node,BOOL bRedraw);//set a ccamera to viewport.returns FALSE if fails
	static void RemoveCameraFromView(int id,BOOL bRedraw);//remove camera from view if one exists.
	//io
	static void New(BOOL bUpdateViews,BOOL bKeepMats=FALSE,BOOL bResetViews=FALSE);//New project
	static BOOL Open(const char*filename);//Open project
	static BOOL Save();//save current project
	static BOOL SaveAs(const char*filename);//save as
	//(ver.>=1.3.5):Import/Export operations
	static void Import();
	static void Export();
	static BOOL ImportFile(const char* filename, cimportinterface* ii);
	static BOOL ExportFile(const char* filename, cexportinterface* ei);
	//(ver.>=1.3.5):Replaced SetProjectName()/GetProjectName() with SetProjectPathName()/GetProjectPathName()
	static void SetProjectPathName(const char* name);//name must include path
	static BOOL GetProjectPathName(char* name);//name must be at least 255 characters long
											//returns FALSE if 'Untitled' -no project.
	//(ver.>=1.3.5):SetProjectName()/GetProjectName() now sets/returns filename(no path,no extension)
	static void SetProjectName(const char* name);//name must include path
	static BOOL GetProjectName(char* name);//name must be at least 255 characters long
											//returns FALSE if 'Untitled' -no project.
	//(ver.>=1.3.5):Added path(no filename)
	static void SetProjectPath(const char* path);//name must include path
	static BOOL GetProjectPath(char* path);//name must be 255 characters long
											//returns FALSE if 'Untitled' -no project.
};

#endif //INTERFACE_H