// Lightray3D (C) 2002-2006 Tzivenis Sakis

//----------------------------------------------------------------
// Pixelformat.h
//
// RGBA colour macros
//
//----------------------------------------------------------------

#ifndef PIXELFORMAT_H
#define PIXELFORMAT_H

#define ABGR_BYTE_ORDER

#ifdef ABGR_BYTE_ORDER

//--------------------------------------------------------------
//ABGR byte order
//--------------------------------------------------------------

//Byte position of components
#define RGB_RED_POS		0
#define RGB_GREEN_POS	1
#define RGB_BLUE_POS	2
#define RGB_ALPHA_POS	3


//24,32 bits
#define RGB_RED_MASK                   0x000000FF
#define RGB_GREEN_MASK                 0x0000FF00
#define RGB_BLUE_MASK                  0x00FF0000
#define RGB_ALPHA_MASK                 0xFF000000
#define RGB_RED_SHIFT                  0
#define RGB_GREEN_SHIFT                8
#define RGB_BLUE_SHIFT                 16
#define RGB_ALPHA_SHIFT                24

//5-6-5 16 bits
#define RGB_565_RED_MASK               0x001F
#define RGB_565_GREEN_MASK             0x07E0
#define RGB_565_BLUE_MASK              0xF800
#define RGB_565_RED_SCALE              8
#define RGB_565_GREEN_SCALE            4
#define RGB_565_BLUE_SCALE             8
#define RGB_565_RED_SHIFT              0
#define RGB_565_GREEN_SHIFT            5
#define RGB_565_BLUE_SHIFT             11

//5-5-5 16 bits
#define RGB_555_RED_MASK               0x001F
#define RGB_555_GREEN_MASK             0x03E0
#define RGB_555_BLUE_MASK              0x7C00
#define RGB_555_RED_SCALE              8
#define RGB_555_GREEN_SCALE            8
#define RGB_555_BLUE_SCALE             8
#define RGB_555_RED_SHIFT              0
#define RGB_555_GREEN_SHIFT            5
#define RGB_555_BLUE_SHIFT             10


//24,32 bits
#define RGB_RED(rgb)                   ((rgb >> RGB_RED_SHIFT) & 0xff)
#define RGB_GREEN(rgb)                 ((rgb >> RGB_GREEN_SHIFT) & 0xff)
#define RGB_BLUE(rgb)                  ((rgb >> RGB_BLUE_SHIFT) & 0xff)
#define RGB_ALPHA(rgb)                 ((rgb >> RGB_ALPHA_SHIFT) & 0xff)

#define RGB_COLOR(r, g, b)             ((unsigned long) ((r<<RGB_RED_SHIFT) | (g<<RGB_GREEN_SHIFT) | (b<<RGB_BLUE_SHIFT)))
#define ARGB_COLOR(r, g, b, a)         ((unsigned long) ((r<<RGB_RED_SHIFT) | (g<<RGB_GREEN_SHIFT) | (b<<RGB_BLUE_SHIFT) | (a<<RGB_ALPHA_SHIFT)))

#define RGB_SETRED(rgb,x)              ((x << RGB_RED_SHIFT)   | (rgb & (~RGB_RED_MASK)))//0xffffff00))
#define RGB_SETGREEN(rgb,x)            ((x << RGB_GREEN_SHIFT) | (rgb & (~RGB_GREEN_MASK)))//0xffff00ff))
#define RGB_SETBLUE(rgb,x)             ((x << RGB_BLUE_SHIFT)  | (rgb & (~RGB_BLUE_MASK)))//0xff00ffff))
#define RGB_SETALPHA(rgba,x)           ((x << RGB_ALPHA_SHIFT) | (rgba & (~RGB_ALPHA_MASK)))//0x00ffffff))

//5-6-5 16 bits
#define RGB_RED16_565(rgb)             (((rgb & RGB_565_RED_MASK) >> RGB_565_RED_SHIFT) * RGB_565_RED_SCALE)
#define RGB_GREEN16_565(rgb)           (((rgb & RGB_565_GREEN_MASK) >> RGB_565_GREEN_SHIFT) * RGB_565_GREEN_SCALE)
#define RGB_BLUE16_565(rgb)            (((rgb & RGB_565_BLUE_MASK) >> RGB_565_BLUE_SHIFT) * RGB_565_BLUE_SCALE)
#define RGB_COLOR16_565(r, g, b)       ((unsigned long) (((r/RGB_565_RED_SCALE) << RGB_565_RED_SHIFT) | ((g/RGB_565_GREEN_SCALE) << RGB_565_GREEN_SHIFT) | ((b/RGB_565_BLUE_SCALE) << RGB_565_BLUE_SHIFT)))
#define RGB_FROM32_565(rgb)            ((unsigned long) ((((rgb&RGB_RED_MASK)/RGB_565_RED_SCALE) << RGB_565_RED_SHIFT) | (((rgb&RGB_GREEN_MASK)/RGB_565_GREEN_SCALE) << RGB_565_GREEN_SHIFT) | (((rgb&RGB_BLUE_MASK)/RGB_565_BLUE_SCALE) << RGB_565_BLUE_SHIFT)))
#define RGB_FROM16_565(rgb)            ((unsigned long) ((RGB_RED16_565(rgb) << RGB_RED_SHIFT) | (RGB_GREEN16_565(rgb) << RGB_GREEN_SHIFT) | (RGB_BLUE16_565(rgb) << RGB_BLUE_SHIFT)))

//5-5-5 16 bits
#define RGB_RED16_555(rgb)             (((rgb & RGB_555_RED_MASK) >> RGB_555_RED_SHIFT) * RGB_555_RED_SCALE)
#define RGB_GREEN16_555(rgb)           (((rgb & RGB_555_GREEN_MASK) >> RGB_555_GREEN_SHIFT) * RGB_555_GREEN_SCALE)
#define RGB_BLUE16_555(rgb)            (((rgb & RGB_555_BLUE_MASK) >> RGB_555_BLUE_SHIFT) * RGB_555_BLUE_SCALE)
#define RGB_COLOR16_555(r, g, b)       ((unsigned long) (((r/RGB_555_RED_SCALE) << RGB_555_RED_SHIFT) | ((g/RGB_555_GREEN_SCALE) << RGB_555_GREEN_SHIFT) | ((b/RGB_555_BLUE_SCALE) << RGB_555_BLUE_SHIFT)))
#define RGB_FROM32_555(rgb)            ((unsigned long) ((((rgb&RGB_RED_MASK)/RGB_555_RED_SCALE) << RGB_555_RED_SHIFT) | (((rgb&RGB_GREEN_MASK)/RGB_555_GREEN_SCALE) << RGB_555_GREEN_SHIFT) | (((rgb&RGB_BLUE_MASK)/RGB_555_BLUE_SCALE) << RGB_555_BLUE_SHIFT)))
#define RGB_FROM16_555(rgb)            ((unsigned long) ((RGB_RED16_555(rgb) << RGB_RED_SHIFT) | (RGB_GREEN16_555(rgb) << RGB_GREEN_SHIFT) | (RGB_BLUE16_555(rgb) << RGB_BLUE_SHIFT)))


#else
//--------------------------------------------------------------
//ARGB byte order
//--------------------------------------------------------------

//Byte position of components
#define RGB_RED_POS		2
#define RGB_GREEN_POS	1
#define RGB_BLUE_POS	0
#define RGB_ALPHA_POS	3

//24,32 bits
#define RGB_RED_MASK                   0x00FF0000
#define RGB_GREEN_MASK                 0x0000FF00
#define RGB_BLUE_MASK                  0x000000FF
#define RGB_ALPHA_MASK                 0xFF000000
#define RGB_RED_SHIFT                  16
#define RGB_GREEN_SHIFT                8
#define RGB_BLUE_SHIFT                 0
#define RGB_ALPHA_SHIFT                24

//5-6-5 16 bits
#define RGB_565_RED_MASK               0xF800
#define RGB_565_GREEN_MASK             0x07E0
#define RGB_565_BLUE_MASK              0x001F
#define RGB_565_RED_SCALE              8
#define RGB_565_GREEN_SCALE            4
#define RGB_565_BLUE_SCALE             8
#define RGB_565_RED_SHIFT              11
#define RGB_565_GREEN_SHIFT            5
#define RGB_565_BLUE_SHIFT             0

//5-5-5 16 bits
#define RGB_555_RED_MASK               0x7C00
#define RGB_555_GREEN_MASK             0x03E0
#define RGB_555_BLUE_MASK              0x001F
#define RGB_555_RED_SCALE              8
#define RGB_555_GREEN_SCALE            8
#define RGB_555_BLUE_SCALE             8
#define RGB_555_RED_SHIFT              10
#define RGB_555_GREEN_SHIFT            5
#define RGB_555_BLUE_SHIFT             0

//24,32 bits
#define RGB_RED(rgb)                   ((rgb >> RGB_RED_SHIFT) & 0xff)
#define RGB_GREEN(rgb)                 ((rgb >> RGB_GREEN_SHIFT) & 0xff)
#define RGB_BLUE(rgb)                  ((rgb >> RGB_BLUE_SHIFT) & 0xff)
#define RGB_ALPHA(rgb)                 ((rgb >> RGB_ALPHA_SHIFT) & 0xff)

#define RGB_COLOR(r, g, b)             ((unsigned long) ((r<<RGB_RED_SHIFT) | (g<<RGB_GREEN_SHIFT) | (b<<RGB_BLUE_SHIFT)))
#define RGBA_COLOR(r, g, b, a)         ((unsigned long) ((r<<RGB_RED_SHIFT) | (g<<RGB_GREEN_SHIFT) | (b<<RGB_BLUE_SHIFT) | (a<<RGB_ALPHA_SHIFT)))

#define RGB_SETRED(rgb,x)              ((x << RGB_RED_SHIFT)   | (rgb & (~RGB_RED_MASK)))
#define RGB_SETGREEN(rgb,x)            ((x << RGB_GREEN_SHIFT) | (rgb & (~RGB_GREEN_MASK)))
#define RGB_SETBLUE(rgb,x)             ((x << RGB_BLUE_SHIFT)  | (rgb & (~RGB_BLUE_MASK)))
#define RGB_SETALPHA(rgba,x)           ((x << RGB_ALPHA_SHIFT) | (rgba & (~RGB_ALPHA_MASK)))

//5-6-5 16 bits
#define RGB_RED16_565(rgb)             (((rgb & RGB_565_RED_MASK) >> RGB_565_RED_SHIFT) * RGB_565_RED_SCALE)
#define RGB_GREEN16_565(rgb)           (((rgb & RGB_565_GREEN_MASK) >> RGB_565_GREEN_SHIFT) * RGB_565_GREEN_SCALE)
#define RGB_BLUE16_565(rgb)            (((rgb & RGB_565_BLUE_MASK) >> RGB_565_BLUE_SHIFT) * RGB_565_BLUE_SCALE)
#define RGB_COLOR16_565(r, g, b)       ((unsigned long) (((r/RGB_565_RED_SCALE) << RGB_565_RED_SHIFT) | ((g/RGB_565_GREEN_SCALE) << RGB_565_GREEN_SHIFT) | ((b/RGB_565_BLUE_SCALE) << RGB_565_BLUE_SHIFT)))
#define RGB_FROM32_565(rgb)            ((unsigned long) ((((rgb&RGB_RED_MASK)/RGB_565_RED_SCALE) << RGB_565_RED_SHIFT) | (((rgb&RGB_GREEN_MASK)/RGB_565_GREEN_SCALE) << RGB_565_GREEN_SHIFT) | (((rgb&RGB_BLUE_MASK)/RGB_565_BLUE_SCALE) << RGB_565_BLUE_SHIFT)))
#define RGB_FROM16_565(rgb)            ((unsigned long) ((RGB_RED16_565(rgb) << RGB_RED_SHIFT) | (RGB_GREEN16_565(rgb) << RGB_GREEN_SHIFT) | (RGB_BLUE16_565(rgb) << RGB_BLUE_SHIFT)))

//5-5-5 16 bits
#define RGB_RED16_555(rgb)             (((rgb & RGB_555_RED_MASK) >> RGB_555_RED_SHIFT) * RGB_555_RED_SCALE)
#define RGB_GREEN16_555(rgb)           (((rgb & RGB_555_GREEN_MASK) >> RGB_555_GREEN_SHIFT) * RGB_555_GREEN_SCALE)
#define RGB_BLUE16_555(rgb)            (((rgb & RGB_555_BLUE_MASK) >> RGB_555_BLUE_SHIFT) * RGB_555_BLUE_SCALE)
#define RGB_COLOR16_555(r, g, b)       ((unsigned long) (((r/RGB_555_RED_SCALE) << RGB_555_RED_SHIFT) | ((g/RGB_555_GREEN_SCALE) << RGB_555_GREEN_SHIFT) | ((b/RGB_555_BLUE_SCALE) << RGB_555_BLUE_SHIFT)))
#define RGB_FROM32_555(rgb)            ((unsigned long) ((((rgb&RGB_RED_MASK)/RGB_555_RED_SCALE) << RGB_555_RED_SHIFT) | (((rgb&RGB_GREEN_MASK)/RGB_555_GREEN_SCALE) << RGB_555_GREEN_SHIFT) | (((rgb&RGB_BLUE_MASK)/RGB_555_BLUE_SCALE) << RGB_555_BLUE_SHIFT)))
#define RGB_FROM16_555(rgb)            ((unsigned long) ((RGB_RED16_555(rgb) << RGB_RED_SHIFT) | (RGB_GREEN16_555(rgb) << RGB_GREEN_SHIFT) | (RGB_BLUE16_555(rgb) << RGB_BLUE_SHIFT)))

#endif//BYTE ORDER

#endif //PIXELFORMAT_H