// Lightray3D (C) 2002-2006 Tzivenis Sakis

#ifndef ARRAY_H
#define ARRAY_H

#pragma warning(disable: 4251)

#define ARRAY_INITSIZE 16// 16 * sizeof(element) initial heap size

//Array ONLY uses these
COREAPI void* array_new(unsigned long sz);
COREAPI void array_delete(void*);
COREAPI void array_assert(int);

template <class Type> class Array
{
public:
	Array(int s=0);//ctor will allocate 's' size
	~Array();//dtor will delete array

	void	SetSize(int s);	//Will grow,shrink(keeps elements if any) or delete array(if s==0).
	//Note:	If you are going to do a sequence of Addings, it's more efficient to 
	//		first call SetSize to make room for them.
	void	Add(Type);		//Will add element and grow array if needed.Multiple same elements allowed.
	void	AddUnique(Type);//Will add element if not already added and grow array if needed.
	//Note:	Array must be sized accordingly before insertions.
	void	InsertAt(int i,Type);//Will insert element at index.Multiple same elements allowed.
	void	InsertUniqueAt(int i,Type);//Will insert element if not already in array.
	int		FindFirst(Type);//Returns first index of element found.Returns -1 if not found.
	int 	FindAll(Type);	//Returns total count of same element found.Returns 0 if none found.
	//Note: When removing use Pack if you want to shrink array to number of elements.
	void	Remove(Type);	//Will remove first element found.Won't shrink array.
	void	RemoveAll(Type);//Will remove all same elements found.Won't shrink array.
	void	RemoveAt(int i);//Will remove element at index.Won't shrink array.
	void	Pack();			//Shrink array so that array_size==num_elem.
	int		Size() const {return array_size;}	//Returns array size.
	int		Count()const {return num_elem;}	//Returns elements number in array.
	//Access operators
	Type&	operator[](int i) const
	{
		array_assert((i>=0 && i<num_elem));
		return element[i];
	}
	Type*	Ptr(int i) const
	{
		array_assert((i>=0 && i<num_elem));
		return(&element[i]);
	}

	//(ver.>=1.3.5):Returns a pointer to elements
	const void* Elements() const{return (const void*)element;}

private:
	void	allocate(int s);
	Type*	element;
	int		num_elem;
	int		array_size;
};


template <class Type>
Array<Type>::Array(int s)
{
	num_elem=0;
	array_size = 0;
	element = NULL;
	if(s>0) allocate(s);
}

template <class Type>
Array<Type>::~Array<Type>()
{
	if(element) array_delete(element);//delete [] element;
}

template <class Type>
void Array<Type>::allocate(int s)
{
	if(s>0)
	{
		Type *old = element;
		array_size =s;
		element = (Type*)array_new(sizeof(Type)*array_size);//new Type[array_size];
		array_assert(element!=NULL);
		memset(element,0,sizeof(Type)*array_size);
		for(int i=0;i<num_elem&&i<s;i++)
			element[i]=old[i];
		if(old) array_delete(old);//delete [] old;
		if(num_elem>array_size) num_elem=array_size;
	}
	else
	{
		if(element) array_delete(element);//delete [] element;
		element=NULL;
		num_elem=0;
		array_size = 0;
	}
}

template <class Type>
void Array<Type>::SetSize(int s)
{
	allocate(s);
//	num_elem=s;
}

template <class Type>
void Array<Type>::Pack()
{
	allocate(num_elem);
}

template <class Type>
void Array<Type>::AddUnique(Type t)
{
	if(FindFirst(t)==-1) Add(t);
}

template <class Type>
void Array<Type>::Add(Type t)
{
	array_assert(num_elem<=array_size);

	if(num_elem==array_size)
		allocate((array_size)?array_size*2:ARRAY_INITSIZE);
//	int i;
//	for(i=0;i<num_elem;i++)// dissallow duplicates
//		array_assert(element[i] != t);
	element[num_elem++] = t;
}

template <class Type>
void Array<Type>::InsertUniqueAt(int i,Type t)
{
	if(FindFirst(t)==-1) InsertAt(i,t);
}

template <class Type>
void Array<Type>::InsertAt(int index,Type t)
{
	array_assert(index<array_size);

//	int i;
//	for(i=0;i<num_elem;i++)// dissallow duplicates
//		array_assert(element[i] != t);
	element[index] = t;
	if(index>=num_elem)
	{
		num_elem=index+1;
		array_assert(num_elem<=array_size);
	}
}

template <class Type>
int Array<Type>::FindFirst(Type t)
{
	int i;
	for(i=0;i<num_elem;i++)
	{
		if(element[i] == t) return i;
	}
	return -1;
}

template <class Type>
int Array<Type>::FindAll(Type t)
{
	int i;
	int count=0;
	for(i=0;i<num_elem;i++)
	{
		if(element[i] == t) count++;
	}
	return count;
}

template <class Type>
void Array<Type>::RemoveAt(int i)
{
	array_assert(i<num_elem);
	if(i>=num_elem) return;
	num_elem--;
	while(i<num_elem)
	{
		element[i] = element[i+1];
		i++;
	}
}

template <class Type>
void Array<Type>::Remove(Type t)
{
	int i;
	for(i=0;i<num_elem;i++)
	{
		if(element[i] == t)	break;
	}
	if(i<num_elem) RemoveAt(i);
}

template <class Type>
void Array<Type>::RemoveAll(Type t)
{
	int i;
next:
	for(i=0;i<num_elem;i++)
	{
		if(element[i] == t)	break;
	}
	if(i<num_elem)
	{
		RemoveAt(i);
		goto next;
	}
//	for(i=0;i<num_elem;i++) array_assert(element[i] != t);
}

#endif //ARRAY_H