// Lightray3D (C) 2002-2006 Tzivenis Sakis

#ifndef CANIM_H
#define CANIM_H

#include "anim.h"
#include "cmathlib.h"
#include "cio.h"


class COREAPI ckey
{
public:
	ckey()
	{
		time=0;position=MakePt3(0,0,0);rotation.Identity();scale=MakePt3(1,1,1);
	}
	ckey(int t,const cpoint3& p,const cquat& r,const cpoint3& s)
	{
		time=t;position=p;rotation=r;scale=s;
	}
	ckey(const ckey& k)
	{
		time=k.time;position=k.position;rotation=k.rotation;scale=k.scale;
	}
	void Set(int t,const cpoint3& p,const cquat& r,const cpoint3& s)
	{
		time=t;position=p;rotation=r;scale=s;
	}

	~ckey(){}//(ver.>=1.3.4):Needed by RAYscript

	int time;
	cpoint3 position;
	cquat	rotation;
	cpoint3 scale;
};

class COREAPI canim
{
public:
	canim();
	~canim();

	void Clear();

	void Copy(const canim& src);

	//(ver.>=1.3.6):Will size key array and clear it unless bClear=FALSE.
	//NOTE: Num Keys must be >= 1
	BOOL SetNumKeys(int nk,BOOL bClear=TRUE);

	//These will set values to key if it exists, or create new
	void SetKey(const ckey& key);
	void SetKey(int time,const cpoint3& pos,const cquat& rot,const cpoint3& scale);

	//If no key at time will return FALSE.
	BOOL GetKey(int time,ckey& key) const;
	//If no keys or failure will return FALSE
	//NOTE:time will be clamped to first,last key times
	BOOL GetInterpolateKey(int time,ckey& key) const;
	//If no key at time will return FALSE
	BOOL RemoveKey(int time);
	//(ver.>=1.3.6):Remove keys segment
	BOOL RemoveKeysSeg(int fromtime,int totime);

	//These use key indices in list not key time value
	BOOL SetKeyAt(int index,ckey& key,BOOL bMatchKeyTime=TRUE);//(ver.>=1.3.6):Will return FALSE if out of range OR if bMatch==TRUE and key time differs
	BOOL GetKeyAt(int index,ckey& key) const;
	BOOL RemoveKeyAt(int index);
	//(ver.>=1.3.6):Remove keys segment
	BOOL RemoveKeysSegAt(int fromindex,int toindex);

	//(ver.>=1.3.6):If no key at time will return FALSE.
	BOOL KeyExists(int time) const;

	int NumKeys() const {return nkeys;}
	int GetFirstKeyTime() const {return first;}
	int GetLastKeyTime() const {return last;}

	//IO methods
	BOOL Load(cio_load& iload,DWORD fileversion);//support for previous file versions
	BOOL Save(cio_save& isave);

protected:
	void Set(const ckey&);

//	ckey **keys;
	ckey *keys;//(ver.>=1.3.6)
	int nkeys;
	int nallockeys;//(ver.>=1.3.6)
	int first,last;
};

//(ver.>=1.3.6):Faster key search algo
COREAPI int FindKeyEntry(int time,int nkeys,ckey* keys);//finds closer to 'time' key entry in array
COREAPI int FindKeyByEntry(int time,int nkeys,ckey* keys);//finds exact key checking time and returns index in array or -1 if doesn't exist

#endif //CANIM_H