// Lightray3D (C) 2002-2006 Tzivenis Sakis

#ifndef CCAMANIM_H
#define CCAMANIM_H

#include "anim.h"
#include "cmathlib.h"
#include "cio.h"

class COREAPI ccamkey
{
public:
	ccamkey()
	{
		time=0;
		fov=53.0f;
	}
	ccamkey(int t,float f)
	{
		time=t;
		fov=f;
	}
	ccamkey(const ccamkey& k)
	{
		time=k.time;
		fov=k.fov;
	}
	void Set(int t,float f)
	{
		time=t;
		fov=f;
	}

	~ccamkey(){}//(ver.>=1.3.4):Used by RAYscript

	int time;
	float fov;
};

class COREAPI ccamanim
{
public:
	ccamanim();
	~ccamanim();

	void Clear();

	void Copy(const ccamanim& src);

	//(ver.>=1.3.6):Will size key array and clear it unless bClear=FALSE.
	//NOTE: Num Keys must be >= 1
	BOOL SetNumKeys(int nk,BOOL bClear=TRUE);

	//These will set values to key if it exists, or create new
	void SetKey(const ccamkey& key);
	void SetKey(int time,float f);

	//If no key at time will return FALSE.
	BOOL GetKey(int time,ccamkey& key) const;
	//If no keys or failure will return FALSE
	//NOTE:time will be clamped to first,last key times
	BOOL GetInterpolateKey(int time,ccamkey& key) const;
	//If no key at time will return FALSE
	BOOL RemoveKey(int time);
	//(ver.>=1.3.6):Remove keys segment
	BOOL RemoveKeysSeg(int fromtime,int totime);

	//These use key indices in list not key time value
	BOOL SetKeyAt(int index,ccamkey& key,BOOL bMatchKeyTime=TRUE);//(ver.>=1.3.6):Will return FALSE if out of range OR if bMatch==TRUE and key time differs
	BOOL GetKeyAt(int index,ccamkey& key) const;
	BOOL RemoveKeyAt(int index);
	//(ver.>=1.3.6):Remove keys segment
	BOOL RemoveKeysSegAt(int fromindex,int toindex);

	//(ver.>=1.3.6):If no key at time will return FALSE.
	BOOL KeyExists(int time) const;

	int NumKeys() const {return nkeys;}
	int GetFirstKeyTime() const {return first;}
	int GetLastKeyTime() const {return last;}

	//IO methods
	BOOL Load(cio_load& iload,DWORD fileversion);//support for previous file versions
	BOOL Save(cio_save& isave);

protected:
	void Set(const ccamkey&);

	int first,last;

//	ccamkey **keys;
	ccamkey *keys;//(ver.>=1.3.6)
	int nkeys;
	int nallockeys;//(ver.>=1.3.6)
};

//(ver.>=1.3.6):Faster key search algo
COREAPI int FindKeyEntry(int time,int nkeys,ccamkey* keys);//finds closer to 'time' key entry in array
COREAPI int FindKeyByEntry(int time,int nkeys,ccamkey* keys);//finds exact key checking time and returns index in array or -1 if doesn't exist

#endif //CCAMANIM_H