// Lightray3D (C) 2002-2006 Tzivenis Sakis

#ifndef CCOLOR_H
#define CCOLOR_H

#include "Pixelformat.h"

class COREAPI ccolor{
	friend COREAPI ccolor MakeColor(float R, float G, float B, float A);

public:
	ccolor(){r=g=b=a=1.0f;}
	ccolor(const ccolor& c) { r = c.r; g = c.g; b = c.b; a = c.a; }
	//Pass values 0...1 for these
	ccolor(float R, float G, float B)  { r = R; g = G; b = B;  a = 1.0f; }
	ccolor(float R, float G, float B, float A)  { r = R; g = G; b = B;  a = A; }
	ccolor(float c[3]) { r = c[0]; g = c[1]; b = c[2]; a = 1.0f; }
	//Pass values 0...255
	ccolor(int R, int G, int B)  { r = (float)R/255; g = (float)G/255; b = (float)B/255;  a = 1.0f; }
	ccolor(int R, int G, int B, int A)  { r = (float)R/255; g = (float)G/255; b = (float)B/255;  a = (float)A/255; }
	ccolor(int c[3]) { r = (float)c[0]/255; g = (float)c[1]/255; b = (float)c[2]/255; a = 1.0f; }

	~ccolor(){}//(ver.>=1.3.4):Needed for RAYscript

	inline ccolor& Set(float R, float G, float B);
	inline ccolor& Set(float R, float G, float B, float A);
	inline ccolor& Set(int R, int G, int B);
	inline ccolor& Set(int R, int G, int B, int A);

	inline ccolor& Identity(){r=g=b=a=1.0f;return *this;}
	inline ccolor& Zero(){r=g=b=a=0.0f;return *this;}
	inline ccolor& Clamp();
	inline int Redi(){return (int)(r*255);}
	inline int Greeni(){return (int)(g*255);}
	inline int Bluei(){return (int)(b*255);}
	inline int Alphai(){return (int)(a*255);}

	// Access operators
	float&  operator[](int i) { return (&r)[i]; }
	const float& operator[](int i) const { return (&r)[i]; }

	// Conversion function
	operator float*() { return(&r); }

	// Assignment operators
	inline ccolor& operator-=(const ccolor&);
	inline ccolor& operator+=(const ccolor&);
	inline ccolor& operator/=(const ccolor&);
	inline ccolor& operator*=(const ccolor&);
	inline ccolor& operator-=(float);
	inline ccolor& operator+=(float);
	inline ccolor& operator/=(float);
	inline ccolor& operator*=(float); 
	inline ccolor& operator-=(int);
	inline ccolor& operator+=(int);
	inline ccolor& operator/=(int);
	inline ccolor& operator*=(int); 

	// Binary operators
	inline ccolor operator-(const ccolor&) const;
	inline ccolor operator+(const ccolor&) const;
	inline ccolor operator/(const ccolor&) const;
	inline ccolor operator*(const ccolor&) const;
	inline ccolor operator-(float) const;
	inline ccolor operator+(float) const;
	inline ccolor operator/(float) const;
	inline ccolor operator*(float) const; 
	inline ccolor operator-(int) const;
	inline ccolor operator+(int) const;
	inline ccolor operator/(int) const;
	inline ccolor operator*(int) const; 

	// Comparison operators
	int operator==(const ccolor& c) const { return ((c.r==r)&&(c.g==g)&&(c.b==b)&&(c.a==a)); }
	int operator!=(const ccolor& c) const { return ((c.r!=r)||(c.g!=g)||(c.b!=b)||(c.a!=a)); }

	float r,g,b,a;
};

inline ccolor Clamp(const ccolor& c);

inline ccolor& ccolor::Set(float R, float G, float B)
{
    r = R;
    g = G;
    b = B;
    return *this;
}

inline ccolor& ccolor::Set(float R, float G, float B, float A)
{
    r = R;
    g = G;
    b = B;
	a = A;
    return *this;
}

inline ccolor& ccolor::Set(int R, int G, int B)
{
    r = (float)R/255;
    g = (float)G/255;
    b = (float)B/255;
    return *this;
}

inline ccolor& ccolor::Set(int R, int G, int B, int A)
{
    r = (float)R/255;
    g = (float)G/255;
    b = (float)B/255;
	a = (float)A/255;
    return *this;
}

inline ccolor& ccolor::Clamp()
{
	if(r < 0.0) r = 0.0;
	if(g < 0.0) g = 0.0;
	if(b < 0.0) b = 0.0;
	if(a < 0.0) a = 0.0;

	if(r > 1.0f) r = 1.0f;
	if(g > 1.0f) g = 1.0f;
	if(b > 1.0f) b = 1.0f;
	if(a > 1.0f) a = 1.0f;
	return *this;
}

inline ccolor& ccolor::operator-=(const ccolor& c)
{	
	r -= c.r;	g -= c.g;	b -= c.b; a -= c.a;
	return *this;
}

inline ccolor& ccolor::operator+=(const ccolor& c)
{
	r += c.r;	g += c.g;	b += c.b; a += c.a;
	return *this;
}

inline ccolor& ccolor::operator/=(const ccolor& c)
{ 
	r /= c.r;	g /= c.g;	b /= c.b; a /= c.a;
	return *this; 
}

inline ccolor& ccolor::operator*=(const ccolor& c)
{ 
	r *= c.r;	g *= c.g;	b *= c.b; a *= c.a;
	return *this; 
}

inline ccolor& ccolor::operator-=(float f)
{
	r -= f;   g -= f;	b -= f;
	return *this;
}

inline ccolor& ccolor::operator+=(float f)
{
	r += f;   g += f;	b += f;
	return *this;
}

inline ccolor& ccolor::operator/=(float f)
{
	r /= f;   g /= f;	b /= f;
	return *this;
}

inline ccolor& ccolor::operator*=(float f)
{
	r *= f;   g *= f;	b *= f;
	return *this;
}

inline ccolor& ccolor::operator-=(int f)
{
	r -= (float)f/255;   g -= (float)f/255;	b -= (float)f/255;
	return *this;
}

inline ccolor& ccolor::operator+=(int f)
{
	r += (float)f/255;   g += (float)f/255;	b += (float)f/255;
	return *this;
}

inline ccolor& ccolor::operator/=(int f)
{
	r /= (float)f/255;   g /= (float)f/255;	b /= (float)f/255;
	return *this;
}

inline ccolor& ccolor::operator*=(int f)
{
	r *= (float)f/255;   g *= (float)f/255;	b *= (float)f/255;
	return *this;
}

inline ccolor ccolor::operator-(const ccolor& c) const
{
	return MakeColor(r-c.r,g-c.g,b-c.b,a-c.a);
}

inline ccolor ccolor::operator+(const ccolor& c) const
{
	return MakeColor(r+c.r,g+c.g,b+c.b,a+c.a);
}

inline ccolor ccolor::operator/(const ccolor& c) const
{
	return MakeColor(r/c.r,g/c.g,b/c.b,a/c.a);
}

inline ccolor ccolor::operator*(const ccolor& c) const
{
	return MakeColor(r*c.r, g*c.g, b*c.b,a*c.a);	
}

inline ccolor ccolor::operator-(float f) const
{
	return MakeColor(r-f, g-f, b-f, a);
}

inline ccolor ccolor::operator+(float f) const
{
	return MakeColor(r+f, g+f, b+f, a);
}

inline ccolor ccolor::operator/(float f) const
{
	return MakeColor(r/f, g/f, b/f, a);
}

inline ccolor ccolor::operator*(float f) const
{
	return MakeColor(r*f, g*f, b*f, a);
}

inline ccolor ccolor::operator-(int f) const
{
	return MakeColor(r-(float)f/255, g-(float)f/255, b-(float)f/255, a);
}

inline ccolor ccolor::operator+(int f) const
{
	return MakeColor(r+(float)f/255, g+(float)f/255, b+(float)f/255, a);
}

inline ccolor ccolor::operator/(int f) const
{
	return MakeColor(r/(float)f/255, g/(float)f/255, b/(float)f/255, a);
}

inline ccolor ccolor::operator*(int f) const
{
	return MakeColor(r*(float)f/255, g*(float)f/255, b*(float)f/255, a);
}

inline ccolor Clamp(ccolor& c)
{
	if(c.r < 0.0) c.r = 0.0;
	if(c.g < 0.0) c.g = 0.0;
	if(c.b < 0.0) c.b = 0.0;
	if(c.a < 0.0) c.a = 0.0;

	if(c.r > 1.0f) c.r = 1.0f;
	if(c.g > 1.0f) c.g = 1.0f;
	if(c.b > 1.0f) c.b = 1.0f;
	if(c.a > 1.0f) c.a = 1.0f;

	return MakeColor(c.r,c.g,c.b,c.a);
}

#endif //CCOLOR_H