// Lightray3D (C) 2002-2006 Tzivenis Sakis

#ifndef CGIZMO_H
#define CGIZMO_H

//Handy class for visual representation of modifiers and others

#include "cnode.h"

//---------------------------------------------------------------------------------------------
class COREAPI cgizmo : public cnode
{
public:
	cgizmo();//ctor will set default plane gizmo
	~cgizmo();
	void Delete();

	void SetGizmoType(UINT type);//Initializes points for this gizmo type, will also call SetSubTypeID()
	UINT GetGizmoType();//Equivalent to GetSubTypeID()
	
	//For now TRUE will set color to yellow if gizmo is to be
	//transformed(pos,rot,scale) else color is set to orange(default is FALSE-orange).
	//Just to visually notify user that this gizmo can be transformed.
	void SetGizmoTransformable(BOOL);
	BOOL IsGizmoTransformable() const {return bTransformable;}

	//(ver.>=1.3.3):Sets the graph axis so that user is aware of it
	//Should be one of AXIS_X,AXIS_Y,AXIS_Z or 0.
	//Default is 0, no graph
	void SetGizmoAxis(int axis);
	int GetGizmoAxis() const {return axis;}
	void SetGizmoAxisNeg(BOOL bNegative);
	int IsGizmoAxisNeg() const {return bNegAxis;}

	void Reset();
	void SetPoint(int i,const cpoint3& p);
	cpoint3 GetPoint(int i) const;

	int NumPoints() const {return numpts;}

	//From cnode
	//(ver.>=1.3.5):Added render flags parameter(see cnode.h)
	void Render(int attime,CGLWindow*,UINT rflags);

protected:
	void ResetPlane();
	void ResetBox();
	void ResetCylinder();
	void ResetSphere();
	void RenderPlane(int attime,CGLWindow*);
	void RenderBox(int attime,CGLWindow*);
	void RenderCylinder(int attime,CGLWindow*);
	void RenderSphere(int attime,CGLWindow*);

	cpoint3 *points;
	int numpts;

	BOOL bTransformable;
	//(ver.>=1.3.3):graph axis
	int axis;
	BOOL bNegAxis;
};

//(ver.>=1.3.2)://Use this instead of the new operator
COREAPI cgizmo* CreateNewGizmo();

#endif //CGIZMO_H