// Lightray3D (C) 2002-2006 Tzivenis Sakis

#ifndef CIMAGE_H
#define CIMAGE_H


//(ver.>=1.3.6):Increased to 512
#define MAX_IMAGE_FILENAME		512//256

#define IMAGE_EMPTY_STRING		"<empty>"//(ver.>=1.3.3):This will be returned if no path

//Versions >= 1.2.5 support animated image frames,AVIs.Frames are loaded in real time for
//both opengl rendering and raytracing.
//Loading of avi,ifl is handled in cimage.cpp(core.dll)

//Playback flags
#define CIMAGE_LOOP		0
#define CIMAGE_STOP		1
#define CIMAGE_BOUNCE	2

//(ver.>=1.3.6):OpenGL
//{
#define TEX_64			64
#define TEX_128			128
#define TEX_256			256
#define TEX_512			512
#define TEX_1024		1024
#define TEX_MAX_SIZE	TEX_512//TEX_1024

#define TEX_NONE		0
#define TEX_NEAREST		1
#define TEX_LINEAR		2

//defaults
#define TEX_DFLT_SIZE	TEX_256
#define TEX_DFLT_TEXEL	TEX_NEAREST
#define TEX_DFLT_MIP	TEX_NONE
//}

#include "ccolor.h" //(ver.>=1.3.5)

//Pixel bits are a RGBA array accessed via GetBits functions 
class COREAPI cimage
{
	friend COREAPI BOOL CopyImage(cimage& dest,const cimage& src);
public:
	cimage();
	~cimage();

	void Clear();

	BOOL Load(const char *filename);
	BOOL Reload();//if already loaded
	BOOL IsEmpty() const;//Returns TRUE if no bits
	//(ver.>=1.3.5):returns TRUE if bits alpha component < 255
	BOOL IsTransparent() const;

	//(ver.>=1.3.2):Added generic loading support.Bits are copied, so user must deallocate mem after.
	//Having your own image loader use this to set bits,size etc.
	//If no palette set pal to NULL.bpp parameter must be 8 or 16 or 24 or 32
	//If no filename set NULL.You must set pathname with SetPathName though.
	//If bpp==8 then pal must exist else returns FALSE
	//(ver.>=1.3.5):Fixed a bug and added b565 param.
	//Set b565 to TRUE OR FALSE according to 16bpp image format:565 OR 555
	BOOL SetBits(char *filename,BYTE* bits,BYTE*pal,int sx,int sy,int bpp,BOOL b565=TRUE);

	//(ver.>=1.3.5):Alpha Channel,Transparency Helpers(opengl colorkey workaround)
	void MakeAlphaChannelOutOfRGB();//copies RGB sum to alpha
	void MakeRGBOutOfAlphaChannel();//copies alpha to RGB(grayscale)
	void SetColorKey(BYTE r,BYTE g,BYTE b);//replaces alpha channel
	void SetColorKey(COLORREF);//replaces alpha channel
	void SetColorKey(ccolor col);//replaces alpha channel
	void GetColorKey(BYTE& red,BYTE& green,BYTE& blue) const;
	void EnableColorKey(BOOL);
	BOOL IsColorKeyEnabled() const;
	void EnableAlpha(BOOL);
	BOOL IsAlphaEnabled() const;
	void RestoreAlpha();


	void SetPathName(char *str);//path+name+extension

	BOOL GetPath(char *str) const;//path
	BOOL GetName(char *str) const;//name
	BOOL GetExt(char *str) const;//extension
	BOOL GetTitle(char *str) const;//name+extension
	BOOL GetPathName(char *str) const;//path+name+extension
	//(ver.>=1.3.4):Added
	const char* GetPathNamePtr() const;//path+name+extension

	//Playback
	void SetFramesFlag(UINT flag);
	UINT GetFramesFlag() const;
	void SetStartFrame(int frame);
	int GetStartFrame() const;
	void SetFrameRate(float nth);//controls the playback speed
	float GetFrameRate() const;
	//If freeze flag set only start frame will be used(useful when needing one single frame out of a video file)
	BOOL SetFreezeFlag(BOOL);//returns previous flag
	BOOL GetFreezeFlag() const;
	int CalcFrameIndex(int attime); //used for calculating the frame index
									//for the given animation time 

	void GetSize(int *w,int *h) const;
	int GetWidth() const;
	int GetHeight() const;
	int GetNumFrames() const;

	const BYTE* GetBitsAt(int index);//returns indexed frame bits
	const BYTE* GetBits(int attime);//returns frame bits for the given animation time value
	const BYTE* GetBitsPtr() const;//returns the current bits pointer

	BYTE& operator[](int ndx)   {return bits[ndx];}

	//For internal use
	UINT GetIDAt(int nframe);
	UINT GetID(int attime);

protected:
	void Bind();
	void UnBind();
	BYTE* ConvertTo32(BYTE*,BYTE*,int,int);//paletted - 8bpp
	//(ver.>=1.3.5):Added helper func for converting bits to 32bpp.
	//Set b565 to TRUE OR FALSE according to 16bpp image format:565 OR 555
	BYTE* ConvertTo32(BYTE* data, int w,int h,int bpp,BOOL b565=TRUE);//16,24 to 32
	BOOL LoadAVI(const char *filename);
	BOOL LoadAVIFrame(int,BOOL);
	BOOL LoadIFL(const char *filename);
	BOOL LoadIFLFrame(int,BOOL);
	void CleanUpAVI();

	char Path[MAX_IMAGE_FILENAME];
	int width,height,numframes;
	BYTE *bits;
	UINT id;
	int lastindex;

	//AVI
	BOOL initializedavi;
	void *pavifile;
	void *pavistream;
	void *getaviframe;

	//Playback
	int startframe;
	float nthframe;
	UINT playflag;
	BOOL bFreeze;

	//(ver.>=1.3.5):Transparent alpha bits flag, colorkey
	BOOL bTrans;
	BYTE *alphabits;
	BOOL bColorKey,bAlpha;
	BYTE colkeyR,colkeyG,colkeyB;
	void UpdateAlphaColorKey(BOOL bBind);
	void StoreAlphaBits();
};

//(ver.>=1.3.5):Added quality param for JPG support
COREAPI BOOL SaveImage(const char *filename,BYTE *data,BYTE *pal,int w,int h,int bpp,int quality=100);
//(ver.>=1.3.6):Added OpenGL tex options(Max Size,Texel lookup,MipMap lookup)
COREAPI void Tex_SetMaxSize(int size);//64,128,256,512 - default is 256
COREAPI int Tex_GetMaxSize();
COREAPI void Tex_SetTexelFilter(int type);//TEX_NEAREST,TEX_LINEAR - default is TEX_NEAREST
COREAPI int Tex_GetTexelFilter();
COREAPI void Tex_SetMipMapFilter(int type);//TEX_NONE,TEX_NEAREST,TEX_LINEAR - default is TEX_NONE
COREAPI int Tex_GetMipMapFilter();

//(ver.>=1.3.6 update1):Helper - Flips The Red And Blue Bytes
COREAPI void ISwapRedBlue(void* buffer,int width,int height,int bpp);//bpp: 32 OR 24

#endif //CIMAGE_H