// Lightray3D (C) 2002-2006 Tzivenis Sakis

#ifndef CLIGHT_H
#define CLIGHT_H

//(ver>=1.3):added target
//(ver>=1.3.1):added attenuation range,enable flag
//(ver>=1.3.2):added shadow mapping,volumetric atmo,changed attenuation vars


#include "cnode.h"
#include "clightanim.h"

#define ATT_TYPE_NONE		0
#define ATT_TYPE_LINEAR		1
#define ATT_TYPE_QUAD		2


class COREAPI clight : public cnode
{
public:
	clight();
	~clight();
	void Delete();

	void Create(UINT ltype,const cpoint3&,const cpoint3& dir,const cpoint3& up);
	void Create(UINT ltype,
				float x,float y,float z,
				float dirx,float diry,float dirz,
				float upx,float upy,float upz);

	void SetLightType(UINT ltype);//will set class sub type to ltype
	void SetLightColor(int attime,const ccolor&);
	void SetDistance(float);
	void SetHotSpot(int attime,float);
	void SetFallOff(int attime,float);
	void SetAttenuationFlag(BOOL);//(ver.>=1.3.1):Enable attenuation
	void SetRange(int attime,float);//(ver.>=1.3.1):Attenuation fall-off with distance
//	void SetConstantAttenuation(int attime,float);
//	void SetLinearAttenuation(int attime,float);
//	void SetQuadraticAttenuation(int attime,float);
	void SetAttenuationType(int);
	void SetStrength(int attime,float);
	void SetLightCastShadowFlag(BOOL);
	void SetTarget(cnode*);
	//(ver.>=1.3.2):Added shadow mapping,volumetric atmo
	void SetShadowMapFlag(BOOL);//Enable shadow mapping,else use raytraced shadows
	void SetShadowMapSize(int);//Shadow map size(8 to 2048)
	void SetShadowMapBias(float);//Shadow map bias.Set 0.0 for absolute bias.(0 to 1000)
	void SetVolumeFlag(BOOL);

	UINT GetLightType() const;//equivalent to GetSubTypeID()
	ccolor GetLightColor(int attime) const;
	float GetDistance() const;
	float GetHotSpot(int attime) const;
	float GetFallOff(int attime) const;
	BOOL GetAttenuationFlag() const;//(ver.>=1.3.1):Enable attenuation
	float GetRange(int attime) const;//(ver.>=1.3.1):Attenuation fall-off with distance
//	float GetConstantAttenuation(int attime) const;
//	float GetLinearAttenuation(int attime) const;
//	float GetQuadraticAttenuation(int attime) const;
	int GetAttenuationType() const;
	float GetStrength(int attime) const;
	BOOL GetLightCastShadowFlag() const;
	cnode* GetTarget();
	//(ver.>=1.3.2):Added shadow mapping,volumetric atmo
	BOOL GetShadowMapFlag() const;//Shadow mapping or raytraced shadows
	int GetShadowMapSize() const;//Shadow map size
	float GetShadowMapBias() const;//Shadow map bias.0.0 for absolute bias
	BOOL GetVolumeFlag() const;

	//GL call to enable this light
	void PushLight(int attime,unsigned int i);

	//Light animation
	clightanim* GetLightAnimPtr(){return &lanim;}

	//From cnode
	BOOL Clone(int attime,cnode&) const;
	//(ver.>=1.3.5):allowed after users request
//	void SetColor(const ccolor& clr){}//Changing color is not allowed
	//If target exists must filter orientation calls bypassing normal light's rotation
	//{
	void SetRotation(int attime,const cquat&);
	cquat GetRotation(int attime) const;
//	void Rotate(int attime,const cquat&,const cpoint3& center);
	//}
	void UpdateTimeIdle();
	void SnapKey();
	void DeleteKey(int attime);
	BOOL HasKey(int attime);
//	void MoveKeys(int pos,int from,int to);//(ver.>=1.3.1):Let app handle this
	//(ver.>=1.3.6 update1):Added HitTest to handle delta
	BOOL HitTest(int attime,CGLWindow*,int x,int y,float *zvalue,float delta);
	void Flash(int attime,CGLWindow*);
	//(ver.>=1.3.5):Added render flags parameter(see cnode.h)
	void Render(int attime,CGLWindow*,UINT rflags);

	//IO methods
	BOOL Load(cio_load& iload,DWORD fileversion);//support for previous file versions
	BOOL Save(cio_save& isave);

	BOOL HandleMessage(DWORD msg,void *Param1,void *Param2);
//	void NodeIsBeingDeleted(cnode*);//now using HandleMessage

protected:
	//Color
	ccolor lcolor;
	//Spot
	float distance;
	float hotspot,coneangle;
	//Attenuation
	BOOL bEnableAtt;
	//(ver.>=1.3.2):Changed attenuation vars
	float att_range;//,att_const,att_linear,att_quad;
	int att_type;//none,linear,quadratic
	float strength;
	//Cast shadows
	BOOL bLCastShadows;
	//Target
	cnode *target;
	//(ver.>=1.3.2):Added shadow mapping,volumetric atmo
	BOOL bShadowMap;//if false shadows will be raytraced
	int smapsize;//8 to 2048
	float smapbias;//0 to 1000
	BOOL bVolumetric;

	clightanim lanim;
};

//(ver.>=1.3.2)://Use this instead of the new operator
COREAPI clight* CreateNewLight();

#endif //CLIGHT_H