// Lightray3D (C) 2002-2006 Tzivenis Sakis

#ifndef CLIGHTANIM_H
#define CLIGHTANIM_H

#include "anim.h"
#include "cmathlib.h"
#include "ccolor.h"
#include "cio.h"

//(ver.>=1.3.1):Added attenuation fall-off range value
//(ver.>=1.3.2):Removed attenuation vars(const,linear,quad)

class COREAPI clightkey
{
public:
	clightkey()
	{
		time=0;
		color.Set(1.0f,1.0f,1.0f,1.0f);
		coneangle=45.0f;
		hotspot=43.0f;
		att_range=256.0f;
	//	att_const=1.0f;
	//	att_linear=0.0f;
	//	att_quad=0.0f;
		strength=1.0f;
	}
	clightkey(int t,const ccolor& c,
			  float angle,float hangle,
			  float att_r,
			  float s)
		//	  float att_c,float att_l,float att_q)
	{
		time=t;
		color=c;
		coneangle=angle;
		hotspot=hangle;
		att_range=att_r;
	//	att_const=att_c;
	//	att_linear=att_l;
	//	att_quad=att_q;
		strength=s;
	}
	clightkey(const clightkey& k)
	{
		time=k.time;
		color=k.color;
		coneangle=k.coneangle;
		hotspot=k.hotspot;
		att_range=k.att_range;
	//	att_const=k.att_const;
	//	att_linear=k.att_linear;
	//	att_quad=k.att_quad;
		strength=k.strength;
	}
	void Set(int t,const ccolor& c,
			  float angle,float hangle,
			  float att_r,
			  float s)
		//	  float att_c,float att_l,float att_q)
	{
		time=t;
		color=c;
		coneangle=angle;
		hotspot=hangle;
		att_range=att_r;
	//	att_const=att_c;
	//	att_linear=att_l;
	//	att_quad=att_q;
		strength=s;
	}

	~clightkey(){}//(ver.>=1.3.4):Used by RAYscript

	int time;
	ccolor color;
	float hotspot,coneangle;
	float att_range;//,att_const,att_linear,att_quad;
	float strength;
};

class COREAPI clightanim
{
public:
	clightanim();
	~clightanim();

	void Clear();

	void Copy(const clightanim& src);

	//(ver.>=1.3.6):Will size key array and clear it unless bClear=FALSE.
	//NOTE: Num Keys must be >= 1
	BOOL SetNumKeys(int nk,BOOL bClear=TRUE);

	//These will set values to key if it exists, or create new
	void SetKey(const clightkey& key);
	void SetKey(int time,const ccolor& c,
				float angle,float hangle,
				float att_r,
				float strength);
			//	float att_c,float att_l,float att_q);

	//If no key at time will return FALSE.
	BOOL GetKey(int time,clightkey& key) const;
	//If no keys or failure will return FALSE
	//NOTE:time will be clamped to first,last key times
	BOOL GetInterpolateKey(int time,clightkey& key) const;
	//If no key at time will return FALSE
	BOOL RemoveKey(int time);
	//(ver.>=1.3.6):Remove keys segment
	BOOL RemoveKeysSeg(int fromtime,int totime);

	//These use key indices in list not key time value
	BOOL SetKeyAt(int index,clightkey& key,BOOL bMatchKeyTime=TRUE);//(ver.>=1.3.6):Will return FALSE if out of range OR if bMatch==TRUE and key time differs
	BOOL GetKeyAt(int index,clightkey& key) const;
	BOOL RemoveKeyAt(int index);
	//(ver.>=1.3.6):Remove keys segment
	BOOL RemoveKeysSegAt(int fromindex,int toindex);

	//(ver.>=1.3.6):If no key at time will return FALSE.
	BOOL KeyExists(int time) const;

	int NumKeys() const {return nkeys;}
	int GetFirstKeyTime() const {return first;}
	int GetLastKeyTime() const {return last;}

	//IO methods
	BOOL Load(cio_load& iload,DWORD fileversion);//support for previous file versions
	BOOL Save(cio_save& isave);

protected:
	void Set(const clightkey&);

	int first,last;

//	clightkey **keys;
	clightkey *keys;//(ver.>=1.3.6)
	int nkeys;
	int nallockeys;//(ver.>=1.3.6)
};

//(ver.>=1.3.6):Faster key search algo
COREAPI int FindKeyEntry(int time,int nkeys,clightkey* keys);//finds closer to 'time' key entry in array
COREAPI int FindKeyByEntry(int time,int nkeys,clightkey* keys);//finds exact key checking time and returns index in array or -1 if doesn't exist

#endif //CLIGHTANIM_H