// Lightray3D (C) 2002-2006 Tzivenis Sakis

#ifndef CMATERIAL_H
#define CMATERIAL_H

#include "cimage.h"
#include "ccolor.h"
#include "cio.h"


//(ver.>=1.3.6):Increased to 64
#define MAX_MATERIAL_NAME		64//32

//(ver.>=1.3.5):Increased to 1024
#define MAX_NUM_MATS				1024//100


enum
{
	MAT_IMG_DIFFUSE = 0,
	MAT_IMG_OPACITY,
	MAT_IMG_BUMP,
	MAT_IMG_REFLECTION,
	MAT_IMG_REFRACTION,
	MAT_IMG_EMISSIVE,
	MAT_IMG_6,
	MAT_IMG_7,
	MAT_IMG_8,
	MAT_IMG_9,
	MAT_MAX_IMAGES
};

//(ver.>=1.3.5)
enum
{
	MAT_COL_AMBIENT = 0,
	MAT_COL_DIFFUSE,
	MAT_COL_SPECULAR,
	MAT_COL_EMISSIVE,
	MAT_COL_TRANS,
	MAT_MAX_COLORS
};

//(ver.>=1.3.2):Added env mapping types for reflection image
#define MAT_ENV_MAP_TEX			0//use object tex coords
#define MAT_ENV_MAP_SPHERICAL	1//spherical world xyz mapping(z axis is up)
#define MAT_ENV_MAP_CYLINDRICAL	2//cylindrical world xyz mapping(z axis is up)
#define MAT_ENV_MAP_SCREEN		3//screen xy mapping

#define SHINE_DEFAULT_STRENGTH	100.0f
#define SHINE_MAX_STRENGTH		128.0f


//(ver.>=1.3.5):OpenGL multi-tex units used
//(ver.>=1.3.6):Decreased to 2
#define MAT_TEX_UNITS_USED		2//4




class COREAPI cmaterial
{
public:
	cmaterial();
	cmaterial(const char *name);
	~cmaterial();

	void Delete();

	void Clear(BOOL bOnlyImgs=FALSE);//TRUE to keep values and clear only images
	void Copy(cmaterial*src);

	void SetName(const char* name);
	void GetName(char* name) const;
	char* GetName();

	void SetAmbient(const ccolor& c){ambient=c;}
	ccolor GetAmbient(){return ambient;}
	void SetAmbient(const float& r,const float& g,const float& b,const float& a=1.0)
	{ambient.Set(r,g,b,a);}
	void GetAmbient(float& r,float& g,float& b,float& a)
	{r=ambient.r;g=ambient.g;b=ambient.b;a=ambient.a;}
	void SetAmbient(const int& r,const int& g,const int& b,const int& a=255)
	{ambient.Set(r,g,b,a);}
	void GetAmbient(int& r,int& g,int& b,int& a)
	{r=ambient.Redi();g=ambient.Greeni();b=ambient.Bluei();a=ambient.Alphai();}

	void SetDiffuse(const ccolor& c){diffuse=c;}
	ccolor GetDiffuse(){return diffuse;}
	void SetDiffuse(const float& r,const float& g,const float& b,const float& a=1.0)
	{diffuse.Set(r,g,b,a);}
	void GetDiffuse(float& r,float& g,float& b,float& a)
	{r=diffuse.r;g=diffuse.g;b=diffuse.b;a=diffuse.a;}
	void SetDiffuse(const int& r,const int& g,const int& b,const int& a=255)
	{diffuse.Set(r,g,b,a);}
	void GetDiffuse(int& r,int& g,int& b,int& a)
	{r=diffuse.Redi();g=diffuse.Greeni();b=diffuse.Bluei();a=diffuse.Alphai();}

	void SetSpecular(const ccolor& c){specular=c;}
	ccolor GetSpecular(){return specular;}
	void SetSpecular(const float& r,const float& g,const float& b,const float& a=1.0)
	{specular.Set(r,g,b,a);}
	void GetSpecular(float& r,float& g,float& b,float& a)
	{r=specular.r;g=specular.g;b=specular.b;a=specular.a;}
	void SetSpecular(const int& r,const int& g,const int& b,const int& a=255)
	{specular.Set(r,g,b,a);}
	void GetSpecular(int& r,int& g,int& b,int& a)
	{r=specular.Redi();g=specular.Greeni();b=specular.Bluei();a=specular.Alphai();}

	void SetEmissive(const ccolor& c){emissive=c;}
	ccolor GetEmissive(){return emissive;}
	void SetEmissive(const float& r,const float& g,const float& b,const float& a=1.0)
	{emissive.Set(r,g,b,a);}
	void GetEmissive(float& r,float& g,float& b,float& a)
	{r=emissive.r;g=emissive.g;b=emissive.b;a=emissive.a;}
	void SetEmissive(const int& r,const int& g,const int& b,const int& a=255)
	{emissive.Set(r,g,b,a);}
	void GetEmissive(int& r,int& g,int& b,int& a)
	{r=emissive.Redi();g=emissive.Greeni();b=emissive.Bluei();a=emissive.Alphai();}

	void SetShininess(int s);
	int GetShininessi(){return (int)shininess;}
	void SetShininess(float s);
	float GetShininess(){return shininess;}

	//(ver.>=1.3):Added transparent color
	void EnableTransColor(BOOL);
	BOOL IsTransColorEnabled();
	void SetTransColor(const ccolor& c);
	ccolor GetTransColor();
	void SetTransColor(const float& r,const float& g,const float& b,const float& a=1.0);
	void GetTransColor(float& r,float& g,float& b,float& a);
	void SetTransColor(const int& r,const int& g,const int& b,const int& a=255);
	void GetTransColor(int& r,int& g,int& b,int& a);

	//(ver.>=1.3.5):Added diffuse image alpha channel enable flag
	void EnableAlpha(BOOL);
	BOOL IsAlphaEnabled();

	//(ver.>=1.3.5):Enable/Disable image fx in OpenGL viewport rendering
	//(i.e transparent faces,bump mapping etc.)
	//Used by application only
	void EnableOGLfx(BOOL);
	BOOL IsOGLfxEnabled();

	void SetImage(const cimage&,UINT type);
	cimage* GetImage(UINT type) const;
	void ClearImage(UINT type);
	BOOL IsValidImage(UINT type) const;//TRUE if image bits!=NULL
	int NumImages() const;//that is, valid images

	void SetDiffuseAmount(int amount);
	int GetDiffuseAmount() const;
	void SetOpacityAmount(int amount);
	int GetOpacityAmount() const;
	void SetBumpAmount(int amount);
	int GetBumpAmount() const;
	void SetReflectionAmount(int amount);
	int GetReflectionAmount() const;
	void SetRefractionAmount(int amount);
	int GetRefractionAmount() const;
	void SetEmissiveAmount(int amount);
	int GetEmissiveAmount() const;

	//(ver.>=1.3.2):Added env mapping types for reflection image
	void SetEnvMapType(int type);
	int GetEnvMapType() const;

	//(ver.>=1.3.5):returns TRUE if:
	// bEnableAlpha OR bEnableTrans
	// &&
	// an alpha diffuse image OR an opacity image OR opacity amount is less than 100%
	BOOL IsTransparent() const;

	void Render(int attime,BOOL btex=TRUE,BOOL bmat=TRUE);

	//IO methods
	BOOL Load(cio_load& iload,DWORD version);//support for previous file versions
	BOOL Save(cio_save& isave);

protected:
	char Name[MAX_MATERIAL_NAME];
	ccolor ambient,diffuse,specular,emissive,trans;
	BOOL bEnableTrans,bEnableAlpha,bEnableOGLfx;
	float shininess;
	cimage image_diffuse,image_opacity,image_bump,image_refl,image_refr,image_emiss;
	int diffuse_amount,opacity_amount,bump_amount,refl_amount,refr_amount,emiss_amount;
	//(ver.>=1.3.2):Added env mapping types for reflection image
	int envmaptype;//default is spherical
};

//(ver.>=1.3.2)://Use this instead of the new operator
COREAPI cmaterial* CreateNewMaterial();

#endif //CMATERIAL_H