// Lightray3D (C) 2002-2006 Tzivenis Sakis

#ifndef CMESH_H
#define CMESH_H

//(ver.>=1.3.1):Added hide sets(undo use them too),fixed some misc interanal calcs and other.

#include "cnode.h"


#define INDEX_NOTVALID	-1


//(ver.>=1.3.5):Smoothing Groups Normal
class COREAPI csmgroup
{
public:
	csmgroup();
	~csmgroup();
	cpoint3		norm;
	DWORD		group;
	int			nfaces;
	csmgroup	*next;
};
class COREAPI csmnormal
{
public:
	csmnormal();
	~csmnormal();
	void Clear();

	int AddNormal(const cpoint3& n, DWORD bits);
	BOOL GetNormal(cpoint3& n,DWORD bits);
	int FindSM(DWORD bits);
	void Normalize();

	//Assignment operator
	csmnormal& operator=(const csmnormal& b);

//private:
	csmgroup	*m_rGroup;
	int nsg;
};
//Additional to assignment operator.Also used by RAYscript.
COREAPI void CopySMnormal(csmnormal& dest,csmnormal& src);


//Vertex
class COREAPI cvertex : public celement
{
public:
	cpoint3 v;
	cpoint3 n;
	csmnormal smn;//(ver.>=1.3.5):Smoothing groups normal
	ccolor color;//not used yet
	DWORD dwSV;//(ver.>=1.3.5):Used for editing skin objects.An index to a skin verts array.
				//Used ONLY when converting a skin to a mesh and when replacing a skin's mesh data.
				//If you modify mesh verts you should handle this var and set to:
				//-1(new vertex, unassigned)
				//or a skin vertex index.

	cvertex()
	{
		type=ELEMENT_POINT;
		wFlags=NO_FLAGS;
		v=MakePt3(0,0,0);
		n=MakePt3(0,0,0);
		color=MakeColor(1,1,1,1);
		dwSV=INDEX_NOTVALID;//-1
	}

	~cvertex(){}//(ver.>=1.3.4):Used by RAYscript
};


//Edge
class COREAPI cedge : public celement
{
public:
	int e1,e2;

	cedge()
	{
		type=ELEMENT_LINE;
		wFlags=NO_FLAGS;
		e1=e2=INDEX_NOTVALID;
	}
	
	cedge(int E1,int E2)
	{
		wFlags=NO_FLAGS;
		e1=E1;
		e2=E2;
	}

	~cedge(){}//(ver.>=1.3.4):Used by RAYscript

	// Conversion function
	operator int*() { return(&e1); }

	// Access operators
	int&  operator[](int i) { return (&e1)[i]; }
	const int& operator[](int i) const { return (&e1)[i]; }

	// Comparison operators
	inline BOOL operator==(const cedge& e) const { return ((e.e1==e1)&&(e.e2==e2)); }
	inline BOOL operator!=(const cedge& e) const { return ((e.e1!=e1)||(e.e2!=e2)); }

	//Swap indices
	inline cedge& Swap() { int tmp=e1;e1=e2;e2=tmp;return *this; }

	BOOL CheckIndices() const
	{
		if(e1<=INDEX_NOTVALID || e2<=INDEX_NOTVALID
			|| e1==e2) return FALSE;
		return TRUE;
	}
};

inline COREAPI BOOL CompareEdge(const cedge& E1,const cedge& E2)
{
	return ((E1.e1==E2.e1)&&(E1.e2==E2.e2));
}

inline COREAPI BOOL CompareSwap(const cedge& E1,const cedge& E2)
{
	return ((E1.e1==E2.e2)&&(E1.e2==E2.e1));
}

inline COREAPI BOOL CompareBoth(const cedge& E1,const cedge& E2)
{
	return (((E1.e1==E2.e1)&&(E1.e2==E2.e2))||((E1.e1==E2.e2)&&(E1.e2==E2.e1)));
}

inline COREAPI cedge Swap(const cedge& e)
{
	cedge edge(e.e2,e.e1);
	return edge;
}



//Face
//Flags continue from last cnode flag
#define EDGE_1			(1<<3)//vis flags
#define EDGE_2			(1<<4)//>>
#define EDGE_3			(1<<5)//>>
#define EDGE_ALL		(EDGE_1|EDGE_2|EDGE_3)//>>
#define FACE_DSIDED		(1<<6)//currently not used


class COREAPI cface :public celement
{
public:
	int v[3];
	int tv[3];
	DWORD smgroup;//(ver.>=1.3.5):Smoothing group
	cmaterial* mat;

	cface()
	{
		type=ELEMENT_POLYGON;
		wFlags=EDGE_ALL;
		smgroup=0;
		mat=NULL;
		for(int i=0;i<3;i++)
		{
			v[i]=INDEX_NOTVALID;
			tv[i]=INDEX_NOTVALID;
		}
	}

	~cface(){}//(ver.>=1.3.4):Used by RAYscript

	BOOL CheckVIndices() const
	{
		if(v[0]<=INDEX_NOTVALID || v[1]<=INDEX_NOTVALID || v[2]<=INDEX_NOTVALID
			|| v[0]==v[1] || v[0]==v[2] || v[1]==v[2])
			return FALSE;
		return TRUE;
	}
	BOOL CheckTVIndices() const
	{
		if(tv[0]<=INDEX_NOTVALID || tv[1]<=INDEX_NOTVALID || tv[2]<=INDEX_NOTVALID)
		//	|| tv[0]==tv[1] || tv[0]==tv[2] || tv[1]==tv[2])
			return FALSE;
		return TRUE;
	}
};


//Sub object
#define SUB_LEVEL_VERT		SUB_LEVEL_POINT
#define SUB_LEVEL_FACE		SUB_LEVEL_POLYGON
#define SUB_LEVEL_EDGE		SUB_LEVEL_LINE


//Mesh
//IMPORTANT:
//	*Constructing a mesh - Allocate,add/set verts,normals,tverts and last faces.Call Invalidate() to make sure
//	that some internal variables are calculated correctly.
//	*FLAG_SELECTED must not be set if FLAG_HIDDEN.Code will try to deal with it BUT if both exist user should
//	check for these flags and give priority to FLAG_HIDDEN.
//	*Hiding elements won't hide others sharing them(e.g faces,edges share vertices.If verts are hidden faces,edges should be too.
//	This is taken care of by some application specific funcs(e.g HideSelected(),UnhideAllVerts() etc.) but when user calls funcs
//	like HideVertSet,AddVert etc. this must be performed manually.

class COREAPI cmesh : public cnode
{
public:
	cmesh();
	~cmesh();
	void Delete();

	//Clears geometry only and not other flags or variables
	void Clear();

	//Clears arrays and allocates space for setting data('Set' funcs below)
	BOOL Allocate(int nv,int ntv,int nf);
	//Will size arrays and clear 'em unless bClear=FALSE.
	//Must always call CalcBBox(),Invalidate() after using them for internal calculations
	//else calling NumSelFaces(),GetSelFacesCenter() and others will return wrong values.
	//NOTE:Don't use 'em unless you know what you're doing
	BOOL SetNumVerts(int nv,BOOL bClear=TRUE);
	BOOL SetNumTVerts(int ntv,BOOL bClear=TRUE);
	BOOL SetNumFaces(int nf,BOOL bClear=TRUE);

	//Check faces' element indices.Return FALSE if indices out of range.
	BOOL CheckFaceVIndices(int i) const;
	BOOL CheckFaceVIndices(const cface&) const;
	BOOL CheckFaceTVIndices(int i) const;
	BOOL CheckFaceTVIndices(const cface&) const;
	BOOL CheckEdgeVIndices(int i) const;
	BOOL CheckEdgeVIndices(const cedge&) const;

	//Sets faces' element indices out of range to INDEX_NOTVALID and calls Invalidate(),CalcBBox().
	void CorrectFacesIndices(int which);//'which' should be ELEMENT_POINT,ELEMENT_TVPOINT

	//Degenerate faces won't be accepted(vertex indices <0 || >=num verts)
	//(ver.>=1.3.6):Added 'attime' param(needed for misc calcs)
	BOOL AddMesh(int attime,const cmesh&,BOOL bKeepMats=FALSE);//attach mesh(adds only geometry,set bKeepMats to TRUE to keep material(s))
	BOOL SetMesh(const cmesh&);//will replace old arrays(copies only geometry)
	BOOL SetMesh(const cpoint3 *v,const cpoint3 *n,
				 const cpoint3 *tv,const int *f,//const cface *f,//(ver.>=1.3.1):int indices instead of cface(num faces * 3 sized)
				 int nv,int ntv,int nf);//will replace old arrays and call Invalidate()

	//Append elements
	BOOL AddVert(const cvertex&);
	BOOL AddVert(const cpoint3&);
	BOOL AddTVert(const cpoint3&);
	BOOL AddFace(const cface&);//must set edges' flags for this face manually with SetEdgeFlags
							   //index for each edge should be 3*faceIndex + edgeIndex

	//If array index exists will replace old values and return TRUE else will return FALSE.
	//NOTE:If mesh is already constructed(faces,edges exist) should call Invalidate() after
	//last SetVert() to calculate internal vars else selected elems centers and others
	//will be wrong.
	BOOL SetVert(int i,const cvertex&);//You can change flags for selecting,hidding elements.
	BOOL SetVert(int i,const cpoint3&);
	BOOL SetNormal(int i,const cpoint3&);
	BOOL SetTVert(int i,const cpoint3&);
	BOOL SetFace(int i,const cface&);//You can change flags for selecting,hidding elements.
									 //If verts indices changed must either call Invalidate() or correct edges flags manually.
	BOOL SetEdgeFlags(int i,WORD flags);//You can change flags for selecting,hidding elements.
										//Index should be 3*faceIndex + edgeIndex
	//(ver.>=1.3.5):skinvertex index
	BOOL SetSkinVertIndex(int i,DWORD index);
	//(ver.>=1.3.6 update1):NOTE - Vertex color not used yet
	BOOL SetVertColor(int i,ccolor&);

	//If array index exists will return 'valid' values
//	cvertex GetVert(int i) const;
	BOOL GetVert(int i,cvertex&) const;//(ver.>=1.3.5):Changed
	cpoint3 GetVertPoint(int i) const;
	cpoint3 GetNormal(int i) const;
	cpoint3 GetTVert(int i) const;
	cface GetFace(int i) const;
	cpoint3 GetFaceNormal(int i) const;
	cedge GetEdge(int i) const;//index should be 3*faceIndex + edgeIndex
	//(ver.>=1.3.5):skinvertex index
	DWORD GetSkinVertIndex(int i) const;
	//(ver.>=1.3.6 update1):NOTE - Vertex color not used yet
	ccolor GetVertColor(int i) const;

	//(ver.>=1.3.6):Flags
	WORD GetVertFlags(int i) const;
	WORD GetFaceFlags(int i) const;
	WORD GetEdgeFlags(int i) const;

	//These are time consuming for large arrays.Use bitarray sets(see funcs below).
	void DeleteVert(int i);//deletes faces,edges too
	void DeleteTVert(int i);//will also set faces' tv indices to INDEX_NOTVALID
	void DeleteFace(int i);//deletes edges too
	void DeleteEdge(int i);//index should be 3*faceIndex + edgeIndex.Deletes face too.

	//Some modifying funcs
	BOOL DivideFace(int i,cpoint3 pt,cpoint3 tv,ccolor color);//(ver.>=1.3.6 update1):Modified
	BOOL ExtrudeFace(int i,float val,cpoint3 normal,WORD wEdge=EDGE_ALL);//(ver.>=1.3.1):Added wEdge flag option for side quads
																		 //(ver.>=1.3.4):Added normal parameter.Use GetFaceNormal() to calc own dir.
	//(ver.>=1.3.6 update1):Added fDist and bAdjacent flags to control division point and adjacent face division
	BOOL DivideEdge(int i,float fDist=0.5f,BOOL bAdjacent=TRUE);//index should be 3*faceIndex + edgeIndex
	//(ver.>=1.3.5):Added normal parameter, corrected extrude direction
	BOOL ExtrudeEdge(int i,float val,cpoint3 norm);//index should be 3*faceIndex + edgeIndex
	//(ver.>=1.3.6 update1):Rotate edge
	BOOL RotateEdge(int i);//index should be 3*faceIndex + edgeIndex
	//(ver.>=1.3.6 update1):Better use cbitarray sets for large arrays(see Break funcs below)
	void BreakVert(int i);
	void BreakFace(int i);
	void BreakEdge(int i);

	//(ver.>=1.3.5):will also calc smgroups normals if bCalcSmNormals==TRUE.
	void Smooth(BOOL bCalcSmNormals=TRUE);//Calculates vertex normals
	//(ver.>=1.3.5):Smoothing groups
	//{
	BOOL AutoSmooth(float smoothangle,BOOL bSelOnly);//calcs smgroups depending on smoothangle(in degrees:0-180)
	void CalcSmNormals();//Is called in Smooth() if bCalcSmNormals==TRUE for calculating faces sm normals.
						//Use it after vertex/face modification if not calling Smooth.
	BOOL GetSmNormal(cpoint3& n,int faceindex,int facevertexindex);//,cpoint3 *fnorm=NULL);//returns face vertex normal
														// facevertexindex: one of face's v[0],v[1],v[3] NOT actual vertex list index.
	//}


	//Will clear isolated/degenerate elements and return number found
	int ClearUnusedVerts();//not referenced by any face
	int ClearUnusedTVerts();//not referenced by any face
	int ClearInvalidFaces();//faces with vert indices out of range

	//If bResetFaces==FALSE, won't affect faces else will set faces to NULL material
	void SetMeshMaterial(cmaterial*,BOOL bResetFaces=FALSE);

	//Face material
	void SetFaceMaterial(int index,cmaterial*);
	cmaterial* GetFaceMaterial(int index) const;


	//'set' arrays must be num of elements to be affected sized.
	//All return number of elements affected.
	int SelectVertSet(cbitarray& set);//Bit values are the indices to the real array
	int SelectFaceSet(cbitarray& set);//Bit values are the indices to the real array
	int SelectEdgeSet(cbitarray& set);//Bit values defined as 3*faceIndex + edgeIndex
	//(ver.>=1.3.1):Added hide funcs(NOTE:Will set FLAG_HIDDEN but won't remove FLAG_SELECTED.
	//This must be done manually(e.g SelectVertSet or GetVert-wFlags&=~FLAG_SELECTED-SetVert)
	int HideVertSet(cbitarray& set);//Bit values are the indices to the real array
	int HideFaceSet(cbitarray& set);//Bit values are the indices to the real array
	int HideEdgeSet(cbitarray& set);//Bit values defined as 3*faceIndex + edgeIndex
	//If there are more than one calls set Invalidate to TRUE at the last one,
	//or Invalidate manually to avoid repeated internal calculations.
	int DeleteVertSet(cbitarray& set,BOOL Invalidate=TRUE);//Deletes faces,edges
	int DeleteTVertSet(cbitarray& set,BOOL Invalidate=TRUE);//NOTE: Invalidate boolean is ignored
	int DeleteFaceSet(cbitarray& set,BOOL Invalidate=TRUE);//Deletes edges
	int DeleteEdgeSet(cbitarray& set,BOOL Invalidate=TRUE);//Deletes faces
	int WeldVertSet(cbitarray& set,BOOL Invalidate=TRUE);
	//(ver.>=1.3.6 update1);Added Break funcs for verts,faces,edges
	int BreakVertSet(cbitarray& set,BOOL Invalidate=TRUE);
	int BreakFaceSet(cbitarray& set,BOOL Invalidate=TRUE);
	int BreakEdgeSet(cbitarray& set,BOOL Invalidate=TRUE);


	//(ver.>=1.3.1):Weld mesh vertices.
	//Useful for removing double verts
	int WeldMesh(float thresh=0.0f);//Be careful, if wrong threshold value is set whole mesh will collapse.

	//(ver.>=1.3.3):Flip all faces
	void FlipAllFaces();

	//Useful for sub-element calculations(local coords-system)
	void GetSelVertsBBox(cpoint3& bmin,cpoint3& bmax);//in LCS
	void GetSelFacesBBox(cpoint3& bmin,cpoint3& bmax);//in LCS
	void GetSelEdgesBBox(cpoint3& bmin,cpoint3& bmax);//in LCS
	cpoint3 GetSelVertsCenter();//in LCS
	cpoint3 GetSelFacesCenter();//in LCS
	cpoint3 GetSelEdgesCenter();//in LCS

	//All internal calculations are done automatically when geometry changes(except SetVert)
	//but if somehow recalculation is needed use these:
	void CalcBBox();
	void Update();//Calculates:selected elems centers,num of selected elems,hidden elems etc.
				//(ver.>=1.3.5):Changed naming used to be Invalidate()

	//(ver.>=1.3.1):Added musthaveflag.Useful for checking only faces that have a specific flag(e.g FLAG_SELECTED)
	//Both will return face index or INDEX_NOTVALID if not found.Indices order doesn't matter.
	//Set prevface to previous face found to search for next.
	int FindFaceFromEdge(int v1,int v2,int prevface=-1,WORD musthaveflag=0);//next search is prevface+1
	//(ver.>=1.3.1):Added.Useful for finding adjacent faces.Set skip to pass the known one .
	int FindAdjacentFaceFromEdge(int v1,int v2,int skip=-1,WORD musthaveflag=0);//use skip for a known

	//No comments..
	int NumVerts() const {return nverts;}
	int NumNormals() const {return nverts;}
	int NumTVerts() const {return ntverts;}
	int NumFaces() const {return nfaces;}
	int NumEdges() const {return edges.GetSize();}
	int NumSelVerts() const {return nselverts;}
	int NumSelFaces() const {return nselfaces;}
	int NumSelEdges() const {return nseledges;}
	//(ver.>=1.3.1)
	int NumHiddenVerts() const {return nhidverts;}
	int NumHiddenFaces() const {return nhidfaces;}
	int NumHiddenEdges() const {return nhidedges;}

	//Application specific
	int SelectAllVerts();//Hidden are omitted
	int SelectAllFaces();//Hidden are omitted
	int SelectAllEdges();//Hidden are omitted
	int DeselectAllVerts();
	int DeselectAllFaces();
	int DeselectAllEdges();
	int InvertSelVerts();
	int InvertSelFaces();
	int InvertSelEdges();
	void UnhideAllVerts();
	void UnhideAllFaces();
	void UnhideAllEdges();
	void DeleteSelVerts();
	void DeleteSelFaces();
	void DeleteSelEdges();
//	void DivideSelFaces();//(ver.>=1.3.6 update1):No more used.
	void ExtrudeSelFaces(float val);
	void FlipSelFaces();
	cmesh* DetachSelFaces();//BOOL bErase);
//	void DivideSelEdges();//(ver.>=1.3.6 update1):No more used.
	void ExtrudeSelEdges(float val);//(ver.>=1.3.5):Corrected extrude direction
	int WeldSelVerts(float threshval);
	BOOL Mirror(int attime,int axis,cmatrix4* tm);//(ver.>=1.3.5):changed Mirror func-see cnode(Geometry only)
	int ClearAllUnused();
	void ShowNormals(BOOL);
	BOOL GetShowNormalsFlag() const;
	void IgnoreBack(BOOL);
	BOOL GetIgnoreBackFlag() const;
	//(ver.>=1.3.2):Removed these
//	void PivotOnly(BOOL);
//	BOOL GetPivotOnlyFlag() const;
	//(ver.>=1.3.2):Replaced these with cnode's virtual ones
//	void ResetTransform();
//	void CenterMeshToPivot();
//	void CenterPivotToMesh();

	//TODO:Should these be private?
	//These return hit element's index
	int PickVertex(int attime,CGLWindow*,int x,int y,int selflags=SEL_SEL,float delta=HITDELTA);
	int PickFace(int attime,CGLWindow*,int x,int y,int selflags=SEL_SEL);
	int PickEdge(int attime,CGLWindow*,int x,int y,int selflags=SEL_SEL,float delta=HITDELTA);
	//These return (min z value)hit element's index if selflags==SEL_ISHIT or number of hit elements
	int PickVertexRect(int attime,CGLWindow*,int x,int y,int x2,int y2,int selflags=SEL_SEL);
	int PickFaceRect(int attime,CGLWindow*,int x,int y,int x2,int y2,int selflags=SEL_SEL);
	int PickEdgeRect(int attime,CGLWindow*,int x,int y,int x2,int y2,int selflags=SEL_SEL);
	int PickVertexCircle(int attime,CGLWindow*,int x,int y,int radius,int selflags=SEL_SEL);
	int PickFaceCircle(int attime,CGLWindow*,int x,int y,int radius,int selflags=SEL_SEL);
	int PickEdgeCircle(int attime,CGLWindow*,int x,int y,int radius,int selflags=SEL_SEL);
	int PickVertexContour(int attime,CGLWindow*,cpoint3*,int,int selflags=SEL_SEL);
	int PickFaceContour(int attime,CGLWindow*,cpoint3*,int,int selflags=SEL_SEL);
	int PickEdgeContour(int attime,CGLWindow*,cpoint3*,int,int selflags=SEL_SEL);

	//From cnode
	BOOL Clone(int attime,cnode&) const;
	void SetMaterial(cmaterial*);//Will set material according to subobject level
	cmaterial* GetMaterial() const;//Returns mesh material
	//(ver.>=1.3.6):Added GetElement()
	BOOL GetElement(celement& elem,int index,UINT elemtype);//Returns FALSE if index out of range OR not such element type
	int GetNumElements(UINT elemtype);
	int GetNumSelElements(UINT elemtype);
	int GetNumHiddenElements(UINT elemtype);//(ver.>=1.3.1)
	BOOL GetSelElementsCenter(UINT elemtype,int attime,cpoint3&);//in LCS, attime is ignored
	//UV
	BOOL HandlesUVMapping() const {return TRUE;}
	//type: 0=plane 1=box 2=cylinder 3=sphere
	//(ver.>=1.3.5):Added capping flag(cylindrical mapping)
	BOOL ApplyUV(int type,BOOL bCap,BOOL bFlipU,BOOL bFlipV,float uscale,float vscale,const cmatrix4& tm);
	//Render
	BOOL IsRenderable() const {return TRUE;}
	RenderMesh* GetRenderMesh(int attime);
	//Sub-level specific
	int SelectAll();
	int DeselectAll();
	int InvertSelection();
	void HideSelected();//Other elements will be hidden too(e.g faces,edges sharing verts etc.)
	void UnhideAll();
	void DeleteSelected();
	//Returns hit element's index
	int PickElement(int attime,CGLWindow*,int x,int y,int selflags,float delta);
	//These return (min z value)hit element's index if selflags==SEL_ISHIT or number of hit elements
	int PickElementRect(int attime,CGLWindow*,int x,int y,int x2,int y2,int selflags);
	int PickElementCircle(int attime,CGLWindow*,int x,int y,int radius,int selflags);
	int PickElementContour(int attime,CGLWindow*,cpoint3*,int,int selflags);
	//Mouse drag handlers.
	//(ver.>=1.3.2):Added pivot flag
	//(ver.>=1.3.6):Added bUseViewIKThresh,ikthresh(see cnode.h)
	void Move(int attime,const cpoint3&,BOOL bPivotOnly=FALSE,BOOL bIK=FALSE,
		BOOL bUseViewIKThresh=TRUE,float ikthresh=1.0f);//WCS
	void Rotate(int attime,const cquat&,const cpoint3& center,BOOL bPivotOnly=FALSE);//WCS
	void Scale(int attime,const cpoint3&,const cpoint3& center,BOOL bPivotOnly=FALSE);//LCS
	//(ver.>=1.3.2):Replaced pivot functions, with new cnode's virtual ones
	//(ver.>=1.3.6):Used TIME_IDLE, now using specified frame('attime' param)
	void ResetTransform(int attime,int tmflags=(TM_ROT|TM_SCALE));//(ver.>=1.3.6 update1):Added tmflags, choose rotation,scale or both
	void CenterGeomToPivot(int attime);//(old:CenterMeshToPivot)
	void CenterPivotToGeom(int attime);//(old:CenterPivotToMesh)
	//Hit test
	BOOL HitTest(int attime,CGLWindow*glwnd,int x,int y,float *zvalue,float delta)
	;//{return cnode::HitTestBBox(attime,glwnd,x,y,zvalue,delta);}
	BOOL HitTestRect(int attime,CGLWindow*glwnd,int x,int y,int x2,int y2)
	;//{return cnode::HitTestRectBBox(attime,glwnd,x,y,x2,y2);}
	BOOL HitTestCircle(int attime,CGLWindow*glwnd,int x,int y,int radius)
	;//{return cnode::HitTestCircleBBox(attime,glwnd,x,y,radius);}
	BOOL HitTestContour(int attime,CGLWindow*glwnd,cpoint3*contour,int npts)
	;//{return cnode::HitTestContourBBox(attime,glwnd,contour,npts);}
	//Display
	//(ver.>=1.3.5):Added render flags parameter(see cnode.h)
	void Render(int attime,CGLWindow*,UINT rflags);

	//IO methods
	BOOL Load(cio_load& iload,DWORD fileversion);//support for previous file versions
	BOOL Save(cio_save& isave);

	BOOL HandleMessage(DWORD msg,void *Param1,void *Param2);
//	BOOL MaterialIsBeingRemoved(cmaterial*);//now using HandleMessage
	BOOL IsMaterialUsed(cmaterial*);


protected:
	void ClearVerts();
	void ClearTVerts();
	void ClearFaces();
	void ClearEdges();

//	BOOL FixAdjacentFaceFlags(int i,int edge);//(ver.>=1.3.6 update1):Removed

	void AddToBBox(const cvertex&);
	void RemoveFromBBox(const cvertex&);

	cpoint3 CalcSelVertsCenter();//in LCS
	cpoint3 CalcSelFacesCenter();//in LCS
	cpoint3 CalcSelEdgesCenter();//in LCS

	//(ver.>=1.3.1)
	void CalcHiddenVerts();
	void CalcHiddenFaces();
	void CalcHiddenEdges();

	void Select(const cvertex&);
	void Deselect(const cvertex&);
	void Select(const cface&);
	void Deselect(const cface&);
	void Select(int i);
	void Deselect(int i);

	//(ver.>=1.3.1)
	void Hide(const cvertex&);
	void Unhide(const cvertex&);
	void Hide(const cface&);
	void Unhide(const cface&);
	void Hide(int i);
	void Unhide(int i);

	void RenderObject(int,CGLWindow*,UINT);
	void RenderVerts(int,CGLWindow*);
	void RenderFaces(int,CGLWindow*);
	void RenderEdges(int,CGLWindow*);
	void RenderNormals(int,CGLWindow*);
	int RenderByMaterial(cmaterial*,int,BOOL,cbitarray&);
	//(ver.>=1.3.5):Rendering Helpers
	cbitarray farray;

	cvertex		*verts;
	cpoint3		*tverts;
	cface		*faces;
	int			nverts,ntverts,nfaces;
	cbitarray	edges;

	cpoint3		selvpos,selfpos,selepos;
	cpoint3		selvmin,selvmax,selfmin,selfmax,selemin,selemax;//(ver.>=1.3.3)
	int			nselverts,nselfaces,nseledges;
	int			nhidverts,nhidfaces,nhidedges;//(ver.>=1.3.1)

	cmaterial* material;

	//TODO:If these are increased in number they should be bitflags
	BOOL		bShowNormals;
	BOOL		bIgnoreBack;
//	BOOL		bPivotOnly;//(ver.>=1.3.2):Removed
};

//(ver.>=1.3.2)://Use this instead of the new operator and delete using cnode's virtual Delete() func
COREAPI cmesh* CreateNewMesh();

//(ver.>=1.3.5):Used by application ONLY!
COREAPI void Mesh_CreateSubMode(BOOL);

#endif //CMESH_H