// Lightray3D (C) 2002-2006 Tzivenis Sakis

#ifndef CSHAPE_H
#define CSHAPE_H

#include "cnode.h"
#include "cmesh.h"


#define SHAPE_TYPE_CORNER		0
#define SHAPE_TYPE_CURVE		1

#define SHAPE_DEFAULT_NUM_CURVE_POINTS	10
#define SHAPE_MAX_CURVE_POINTS	100

#define SHAPE_LATHE_ALIGN_MIN		0
#define SHAPE_LATHE_ALIGN_CENTER	1
#define SHAPE_LATHE_ALIGN_MAX		2
#define SHAPE_LATHE_MIN_SEGS		3
#define SHAPE_LATHE_MAX_SEGS		100
#define SHAPE_LATHE_X_AXIS			0
#define SHAPE_LATHE_Y_AXIS			1
#define SHAPE_LATHE_Z_AXIS			2


//Shape point
class COREAPI cshpoint : public celement
{
public:
	cpoint3 p;
	cshpoint()
	{
		type=ELEMENT_POINT;
		wFlags=NO_FLAGS;
		p=MakePt3(0,0,0);
	}

	~cshpoint(){}//(ver.>=1.3.4):Used by RAYscript
};

//	*Shape points must be ordered, ccw or cw doesn't matter.
//	*FLAG_SELECTED must not be set if FLAG_HIDDEN.Code will try to deal with it BUT if both exist user should
//	check for these flags and give priority to FLAG_HIDDEN.

class COREAPI cshape : public cnode
{
public:
	cshape();
	~cshape();

	void Delete();

	//Set 'extrudeval' to 0 to get a flat mesh('sections' is ignored)
	cmesh* GetMesh(int attime,float extrudeval,int sections);
	//(ver.>=1.3.3):Added capping,degrees
	cmesh* GetLatheMesh(int attime,int segments,int align,int axis,
		float degs,BOOL bWeldCore,BOOL bCapStart,BOOL bCapEnd);
	//(ver.>=1.3.5):Extrude along path(shape)
	cmesh* GetMeshEx(int attime,cshape* path);

	void Clear();
	
	void Create(cpoint3*contour,int npts);
	void Create(float x,float y,float z,cpoint3*contour,int npts);
	void Create(const cpoint3& pos,cpoint3*contour,int npts);

	//Will affect only geometry not other values(bClosed,shapetype etc.)
	//These must be set manually.
	BOOL SetShape(const cshape&);//will replace old arrays

	BOOL AddPoint(const cshpoint&);
	BOOL AddPoint(const cpoint3&);
	//NOTE:If shape is already constructed should call Update() after
	//last SetPoint() to calculate internal vars else selected elems centers and others will be wrong.
	BOOL SetPoint(int index,const cshpoint&);//You can change flags for selecting,hidding elements.
	BOOL SetPoint(int index,const cpoint3&);
	BOOL GetPoint(int index,cshpoint&) const;
	cshpoint GetPoint(int index) const;
	BOOL GetPoint(int index,cpoint3&) const;
	cpoint3 GetPoint3(int index) const;
	BOOL SetEdgeFlags(int i,WORD flags);//You can change flags for selecting,hidding elements.
	WORD GetEdgeFlags(int i);
	int GetEdgeIndex1(int index) const;//returns -1 if invalid edge index
	int GetEdgeIndex2(int index) const;//returns -1 if invalid edge index

	//(ver.>=1.3.6):Point Flags
	WORD GetPointFlags(int i) const;

	//(ver.>=1.3.6):Clears arrays and allocates space for setting data('Set' funcs above)
	BOOL Allocate(int npts);
	//(ver.>=1.3.6):Will size arrays and clear 'em unless bClear=FALSE.
	//NOTE:Don't use 'em unless you know what you're doing
	BOOL SetNumPoints(int npts,BOOL bClear=TRUE);

	void SetClosedFlag(BOOL);
	BOOL IsClosed() const;

	void SetShapeType(UINT type);
	UINT GetShapeType(){return shapetype;}
	void SetNumCurvePts(int num);//curve interpolated points(accepted values:0-SHAPE_MAX_CURVE_POINTS)
	int NumCurvePts() const {return ncurvepts;}

	//Will work for shape points>=3 and output edge's curve points in 'pts' of size 'npts'
	//(not counting edge start-end points).
	BOOL GetEdgeCurvePts(int index,cpoint3 *pts,int npts);

	void DeletePoint(int index);//This is time consuming for large arrays
	void DeleteEdge(int index);//This is time consuming for large arrays

	//'set' arrays must be num of elements to be affected sized.
	//All return number of elements affected.
	int SelectPointSet(cbitarray& set);//Bit values are the indices to the real array
	int SelectEdgeSet(cbitarray& set);//Bit values are the indices to the real array
	//If there are more than one calls set Invalidate to TRUE at the last one.
	int DeletePointSet(cbitarray& set,BOOL Invalidate=TRUE);
	int DeleteEdgeSet(cbitarray& set,BOOL Invalidate=TRUE);

	cpoint3 GetSelPointsCenter();//in LCS
	cpoint3 GetSelEdgesCenter();//in LCS

	//No comments..
	int NumPoints() const {return npoints;}
	int NumEdges() const {return nedges;}
	int NumSelPoints() const {return nselpoints;}
	int NumSelEdges() const {return nseledges;}

	//Application specific
	void DivideSelEdges();
	int SelectAllPoints();
	int DeselectAllPoints();
	int InvertSelPoints();
	int SelectAllEdges();
	int DeselectAllEdges();
	int InvertSelEdges();
	void DeleteSelPoints();
	void DeleteSelEdges();
	BOOL Mirror(int attime,int axis,cmatrix4* tm);//(ver.>=1.3.5):changed Mirror func-see cnode

	//All internal calculations are done automatically when geometry changes but
	//if somehow recalculation is needed use these
	void CalcBBox();
	void Update(BOOL doBBox=TRUE);//(ver.>=1.3.5):Changed naming used to be Invalidate()

	//TODO:Should these be private?
	//These return hit element's index
	int PickPoint(int attime,CGLWindow*,int x,int y,int selflags=SEL_SEL,float delta=HITDELTA);
	int PickEdge(int attime,CGLWindow*,int x,int y,int selflags=SEL_SEL,float delta=HITDELTA);
	//These return (min z value)hit element's index if selflags==SEL_ISHIT or number of hit elements
	int PickPointRect(int attime,CGLWindow*,int x,int y,int x2,int y2,int selflags=SEL_SEL);
	int PickEdgeRect(int attime,CGLWindow*,int x,int y,int x2,int y2,int selflags=SEL_SEL);
	int PickPointCircle(int attime,CGLWindow*,int x,int y,int radius,int selflags=SEL_SEL);
	int PickEdgeCircle(int attime,CGLWindow*,int x,int y,int radius,int selflags=SEL_SEL);
	int PickPointContour(int attime,CGLWindow*,cpoint3*,int,int selflags=SEL_SEL);
	int PickEdgeContour(int attime,CGLWindow*,cpoint3*,int,int selflags=SEL_SEL);

	//From cnode
	BOOL Clone(int attime,cnode&) const;
	//(ver.>=1.3.6):Added GetElement()
	BOOL GetElement(celement& elem,int index,UINT elemtype);//Returns FALSE if index out of range OR not such element type
	int GetNumElements(UINT elemtype);
	int GetNumSelElements(UINT elemtype);
	BOOL GetSelElementsCenter(UINT elemtype,int attime,cpoint3&);//in LCS, attime is ignored
	//Sub-level specific
	int SelectAll();
	int DeselectAll();
	int InvertSelection();
	void DeleteSelected();
	//Returns hit element's index
	int PickElement(int attime,CGLWindow*,int x,int y,int selflags,float delta);
	//These return (min z value)hit element's index if selflags==SEL_ISHIT or number of hit elements
	int PickElementRect(int attime,CGLWindow*,int x,int y,int x2,int y2,int selflags);
	int PickElementCircle(int attime,CGLWindow*,int x,int y,int radius,int selflags);
	int PickElementContour(int attime,CGLWindow*,cpoint3*,int,int selflags);
	//Mouse drag handlers.
	//(ver.>=1.3.2):Added pivot flag
	//(ver.>=1.3.6):Added bUseViewIKThresh,ikthresh(see cnode.h)
	void Move(int attime,const cpoint3&,BOOL bPivotOnly=FALSE,BOOL bIK=FALSE,
		BOOL bUseViewIKThresh=TRUE,float ikthresh=1.0f);//WCS
	void Rotate(int attime,const cquat&,const cpoint3& center,BOOL bPivotOnly=FALSE);//WCS
	void Scale(int attime,const cpoint3&,const cpoint3& center,BOOL bPivotOnly=FALSE);//LCS
	//(ver.>=1.3.2):Added cnode's pivot virtual functions
	//(ver.>=1.3.6):Used TIME_IDLE, now using specified frame('attime' param)
	void ResetTransform(int attime,int tmflags=(TM_ROT|TM_SCALE));//(ver.>=1.3.6 update1):Added tmflags, choose rotation,scale or both
	void CenterGeomToPivot(int attime);
	void CenterPivotToGeom(int attime);

	BOOL HitTest(int attime,CGLWindow*glwnd,int x,int y,float *zvalue,float delta);
//	{return cnode::HitTestBBox(attime,glwnd,x,y,zvalue,delta);}
	BOOL HitTestRect(int attime,CGLWindow*glwnd,int x,int y,int x2,int y2);
//	{return cnode::HitTestRectBBox(attime,glwnd,x,y,x2,y2);}
	BOOL HitTestCircle(int attime,CGLWindow*glwnd,int x,int y,int radius);
//	{return cnode::HitTestCircleBBox(attime,glwnd,x,y,radius);}
	BOOL HitTestContour(int attime,CGLWindow*glwnd,cpoint3*contour,int npts);
//	{return cnode::HitTestContourBBox(attime,glwnd,contour,npts);}
	//Display
	void Flash(int attime,CGLWindow*);
	//(ver.>=1.3.5):Added render flags parameter(see cnode.h)
	void Render(int attime,CGLWindow*,UINT rflags);

	//IO methods
	BOOL Load(cio_load& iload,DWORD fileversion);//support for previous file versions
	BOOL Save(cio_save& isave);

protected:
	void Select(const cshpoint&);
	void Deselect(const cshpoint&);
	void Select(int);
	void Deselect(int);

	cpoint3 CalcSelPointsCenter();//LCS
	cpoint3 CalcSelEdgesCenter();//LCS

	BOOL bClosed;
	UINT shapetype;
	int ncurvepts;
	cshpoint *points;
	cedge	*edges;
	int npoints;
	int nedges;
	int nseledges;
	int nselpoints;
	cpoint3 selppos;
	cpoint3 selepos;
	cpoint3	selpmin,selpmax,selemin,selemax;//(ver.>=1.3.3)
};

//(ver.>=1.3.2)://Use this instead of the new operator and delete using cnode's virtual Delete() func
COREAPI cshape* CreateNewShape();

#endif //CSHAPE_H