// Lightray3D (C) 2002-2006 Tzivenis Sakis

#ifndef CSKIN_H
#define CSKIN_H

#include "cnode.h"
#include "cmesh.h"
#include "array.h"


//(ver.>=1.3.3):Multi-weighting
#define SKIN_MAX_WEIGHTS	10

class COREAPI cskinweight
{
public:
	float weight;
	cpoint3 voffset;
	cpoint3 noffset;
	cnode* deformer;
	
	cskinweight()
	{
		Reset();
	}

	~cskinweight(){}//(ver.>=1.3.4):Used by RAYscript

	void Reset()
	{
		weight=0.0f;
		voffset=noffset=MakePt3(0,0,0);
		deformer=NULL;
	}
};

//Vertex
class COREAPI cskinvertex : public cvertex
{
public:
	cskinweight weights[SKIN_MAX_WEIGHTS];
	int numWeights;
	csmnormal skinsmn;//(ver.>=1.3.5):Smoothing groups deformed normal
						//Valid ONLY after a CalcSmNormals(int attime) call

	cskinvertex()
	{
		numWeights=0;
	}

	~cskinvertex(){}//(ver.>=1.3.4):Used by RAYscript
};


//NOTE: Skin contains both simple mesh geom data and deformed geom data (verts,norms,faces).
//Funcs below are used for calculating both mesh & skin(deformed geom) data.
//Make a distinction between funcs that have an 'attime' param and those that don't.
//'attime' funcs used for parsing,calculating deformed geometry(skin) data.

class COREAPI cskin : public cnode//cmesh
{
public:
	cskin();
	~cskin();

	void Delete();


	BOOL Init(cmesh*);//at least verts must exist
	
	//(ver.>=1.3.6):Added 'attime' param(needed by CalcBBox)
	BOOL Replace(int attime,cmesh*);//Must have a mesh converted first.
									//New mesh geometry num must be equal to existing not counting tverts.
	cmesh* GetMesh(int attime);
	void Clear();

	//Will copy geometry from src replacing arrays
	void SetSkin(const cskin& src);//used to be GetGeometry()

	//(ver.>=1.3.4):Added AddSkin func.
	//Degenerate faces won't be accepted(vertex indices <0 || >=num verts)
	//(ver.>=1.3.6):Added 'attime' param(needed by CalcBBox)
	BOOL AddSkin(int attime,const cskin& src,BOOL bKeepMats=FALSE);//(adds only geometry,set bKeepMats to TRUE to keep material(s))
	//(ver.>=1.3.6):Added
	BOOL AddMesh(int attime,const cmesh& src,BOOL bKeepMats=FALSE);//(adds only geometry,set bKeepMats to TRUE to keep material(s))
	//Append elements
	BOOL AddVert(const cskinvertex&);//should call CalcBBox after all verts added
	BOOL AddVert(const cpoint3&);//should call CalcBBox after all verts added
	BOOL AddTVert(const cpoint3&);
	BOOL AddFace(const cface&);//must set edges' flags for this face manually with SetEdgeFlags
							   //index for each edge should be 3*faceIndex + edgeIndex

	//(ver.>=1.3.2):Added these for undoing purposes.
	//Can be used for other purposes if you know what you're doing!
	BOOL SetVert(int i,cskinvertex& v);
	BOOL SetTVert(int i,cpoint3& tv);
	BOOL SetFace(int i,cface& face);
	BOOL SetEdgeFlags(int i,WORD flags);
	//(ver.>=1.3.4)
	//Clears arrays and allocates space for setting data('Set' funcs above)
	BOOL Allocate(int nv,int ntv,int nf);
	//Will size arrays and clear 'em unless bClear=FALSE.
	//NOTE:Don't use 'em unless you know what you're doing
	BOOL SetNumVerts(int nv,BOOL bClear=TRUE);
	BOOL SetNumTVerts(int ntv,BOOL bClear=TRUE);
	BOOL SetNumFaces(int nf,BOOL bClear=TRUE);

	//(ver.>=1.3.5):will also calc smgroups normals if bCalcSmNormals==TRUE.
	void Smooth(BOOL bCalcSmNormals=TRUE);//Calculates vertex normals
	//(ver.>=1.3.5):Smoothing Groups
	//{
	BOOL AutoSmooth(float smoothangle,BOOL bSelOnly);//calcs smgroups depending on smoothangle(in degrees:0-180)
	void CalcSmNormals();//Is called in Smooth() if bCalcSmNormals==TRUE for calculating faces' sm normals.
						//Use it after vertex/face modification if not calling Smooth.
	BOOL GetSmNormal(cpoint3& n,int faceindex,int facevertexindex);//returns face's vertex sm normal
														// facevertexindex: one of face's v[0],v[1],v[3] NOT actual vertex list index.

	//Deformed sm normals.
	//These func calculate the deformed skin normals opposite to the funcs above that calculate mesh's normals.
	void CalcSmNormals(int attime);//call this before using/parsing skin faces' smnormals.
	BOOL GetSmNormal(int attime,cpoint3& n,int faceindex,int facevertexindex);//returns face vertex normal
														// facevertexindex: one of face's v[0],v[1],v[3] NOT actual vertex index.
	//Helper:Using an array to store ALL faces' deformed normals.
	//Array is sized to (num faces *3).
	//To retrieve a face vertex normal: norm=array[faceindex*3+facevertexindex].-vertexindex == 0 to 2-
	BOOL GetSmNormalArray(int attime,Array<cpoint3>& array);
	//}

	//(ver.>=1.3.3):Check faces' element indices.Return FALSE if indices out of range.
	BOOL CheckFaceVIndices(int i) const;
	BOOL CheckFaceVIndices(const cface&) const;
	BOOL CheckFaceTVIndices(int i) const;
	BOOL CheckFaceTVIndices(const cface&) const;
	BOOL CheckEdgeVIndices(int i) const;
	BOOL CheckEdgeVIndices(const cedge&) const;

//	cskinvertex GetVert(int i) const;
	BOOL GetVert(int i,cskinvertex&) const;//(ver.>=1.3.5):Changed
	cpoint3 GetVertPoint(int i) const;
	cpoint3 GetNormal(int i) const;
	cpoint3 GetTVert(int i) const;
	cface GetFace(int i) const;
	cpoint3 GetFaceNormal(int i) const;
	cedge GetEdge(int i) const;//index should be 3*faceIndex + edgeIndex

	//(ver.>=1.3.6):Flags
	WORD GetVertFlags(int i) const;
	WORD GetFaceFlags(int i) const;
	WORD GetEdgeFlags(int i) const;

	void SelectVert(int i);

	//(ver.>=1.3.4):Added
	//These are time consuming if deleting many elements at a time.
	//Use bitarray sets(see funcs below)
	//(ver.>=1.3.6):Added 'attime' param(needed by CalcBBox)
	void DeleteVert(int attime,int i);//deletes faces,edges too
	void DeleteTVert(int i);//will also set faces' tv indices to INDEX_NOTVALID
	void DeleteFace(int i);//deletes edges too
	void DeleteEdge(int i);//index should be 3*faceIndex + edgeIndex.Deletes face too.
	//Will clear isolated/degenerate elements and return number found
	//(ver.>=1.3.6):Added 'attime' param(needed by CalcBBox)
	int ClearUnusedVerts(int attime);//not referenced by any face
	int ClearUnusedTVerts();//not referenced by any face
	int ClearInvalidFaces();//faces with vert indices out of range
	int ClearAllUnused(int attime);//(ver.>=1.3.6):Added 'attime' param(needed by CalcBBox)

	//Will create new weight(TRUE) if cnode not already set(FALSE).
	BOOL SetVertDeformer(int i,cnode*);//This may be slow for large array, better use bitarrays
	void SetVertWeight(int i,int w,const cskinweight& weight);
	void SetVertWeightVal(int i,int w,float weight);
	void SetVertNumWeights(int i,int nweights);
	//Always check this flag for weight's deformer node:(verts[i].deformer->GetFlags()&FLAG_HOLD)
	//If present dont use.Treat it as if deformer pointer is NULL.This flag is used when an object 
	//is deleted but still exists in Undo mem until undo items exceed limit and then actually deleted.
	cnode* GetVertDeformer(int i,int w) const;
	BOOL GetVertWeight(int i,int w,cskinweight& weight) const;
	float GetVertWeightVal(int i,int w) const;
	int GetVertNumWeights(int i);
	void ResetVert(int i);
	//(ver.>=1.3.3):
	//Normalize vert's weights.Sum must be 1.(i.e sum=w1+w2+wN - w1/=sum,w2/=sum,wN/=sum)
	void NormalizeVertWeights(int i);
	int NormalizeVertsWeights();//All.Returns num verts normalized

	//Returns vert's weights sum clamped to 0-1.
	float GetVertWeightSumClamped(int i) const;
	//Returns vert's weights sum
	float GetVertWeightSum(int i) const;

	//(ver.>=1.3.2):Added to recalculate vertex offsets and other vars if linkage is changed or something...
	BOOL UpdateDeformers(int attime);//if no deformers returns FALSE
	//(ver.>=1.3.2):Added(copy/store deformed verts/normals to mesh arrays and update deformers)
	//(ver.>=1.3.6):(Renamed to SetPoseAndUpdate from SetPose)
	void SetPoseAndUpdate(int attime);
	//(ver.>=1.3.6):Copy/store deformed verts/normals to mesh arrays
	void SetPose(int attime);

	//(ver.>=1.3.6):Editing bones without meshing skin vertices
	//Mostly used by the app when user wants to adjust bones(move,rotate,scale)
	//without messing the geometry
	void LockGeom(int attime,BOOL bLock);
	BOOL IsGeomLocked();


	//-Get deformed verts in skin local space
	//These are too slow for retrieving large numbers of points/normals/faces, 
	//better use arrays below.
	cpoint3 GetVert(int attime,int i) const;
	cpoint3 GetNormal(int attime,int i) const;
	cpoint3 GetFaceNormal(int attime,int i) const;
	//All will return FALSE if verts or faces number is 0.Arrays will get sized accordingly
	BOOL GetVertArray(int attime,Array<cpoint3>& array) const;
	BOOL GetNormalArray(int attime,Array<cpoint3>& array) const;
	BOOL GetFaceNormalArray(int attime,Array<cpoint3>& array) const;

	//'set' arrays must be num of elements to be affected sized.
	//Returns number of elements affected.
	int SelectVertSet(cbitarray& set);//Bit values are the indices to the real array
	//Set deformer node(usually a bone) to verts.
	//Will create new weights(ret value==num weights created) if cnode not already set(ret value==0).
	int SetVertSetDeformer(cnode*,cbitarray& set);//Bit values are the indices to the real array
	//Set weight value(0.0 to 1.0) to verts assigned to deformer.
	int SetVertSetWeight(cnode* deformer,float val,cbitarray& set);
	//Get indices of verts being deformed by deformer node.Node must exist else set values are undefined
	void GetVertSetByDeformer(cnode*,cbitarray& set);
	//Reset verts
	void ResetVertSet(cbitarray& set);

	//(ver.>=1.3.4):
	//If there are more than one calls set Invalidate to TRUE at the last one,
	//or Invalidate manually to avoid repeated internal calculations.
	//(ver.>=1.3.6):Added 'attime' param(needed by CalcBBox)
	int DeleteVertSet(int attime,cbitarray& set,BOOL bInvalidate=TRUE);//Deletes faces,edges
	int DeleteTVertSet(cbitarray& set,BOOL bInvalidate=TRUE);//NOTE: Invalidate boolean is ignored
	int DeleteFaceSet(cbitarray& set,BOOL bInvalidate=TRUE);//Deletes edges
	int DeleteEdgeSet(cbitarray& set,BOOL bInvalidate=TRUE);//Deletes faces

	//Face material
	void SetFaceMaterial(int index,cmaterial*);
	cmaterial* GetFaceMaterial(int index) const;

	//Resets all verts that own this deformer
	BOOL RemoveDeformer(cnode*);//returns TRUE if any vert(s) reset

	//(ver.>=1.3.3):Bind skin verts to bones.
	//NOTE: Bones that have no parent(skeletal structure) won't be used in calculations.
	//(ver.>=1.3.6):Added <attime>,<bOnlySelVerts> parameters, better point to bone proximity calcs
	BOOL BindRoot(int attime,cnode *rootbone,int nbonespervert=1,BOOL bOnlySelVerts=FALSE);
	//(ver.>=1.3.6):This one handles user defined bone array
	BOOL Bind(int attime,cnode **bones,int numbones,int nbonespervert=1,BOOL bOnlySelVerts=FALSE);

	//(ver.>=1.3.4):Added
	//Useful for sub-element calculations(local coords-system)
	void GetSelVertsBBox(int attime,cpoint3& bmin,cpoint3& bmax);//in LCS
	cpoint3 GetSelVertsCenter(int attime);//in LCS

	void CalcBBox(int attime);//(ver.>=1.3.4):Made it public

	//No comments..
	int NumVerts() const {return nverts;}
	int NumNormals() const {return nverts;}
	int NumTVerts() const {return ntverts;}
	int NumFaces() const {return nfaces;}
	int NumEdges() const {return edges.GetSize();}
	int NumSelVerts() const;//(ver.>=1.3.3)

	//Application specific
	int SetSelVertsDeformer(cnode*);//(ver.>=1.3.2):Changed return value from void to int to reflect num verts affected
	int SelectByDeformer(cnode*);//(ver.>=1.3.2):Changed return value from void to int to reflect num verts affected
	int SelectUnassigned();//(ver.>=1.3.5):Select all unassigned vertices.Returns number of unassigned verts found & selected.
	int ResetSelected();//(ver.>=1.3.2):Changed return value from void to int to reflect num verts affected
	int SetSelWeight(cnode* deformer,float,BOOL bNormOthers);//(ver.>=1.3.2):Changed return value from void to int to return num verts affected
															 //(ver.>=1.3.6 update1):Added option for normalizing other weights
	BOOL GetSelWeight(cnode* deformer,float*);//Returns FALSE if none selected or if selected don't share same value
	BOOL RemoveSelDeformer(cnode*);//(ver.>=1.3.3)
	//(ver.>=1.3.3):Normalize vert's weights.Sum must be 1.(i.e sum=w1+w2+wN - w1/=sum,w2/=sum,wN/=sum)
	int NormalizeSelVertsWeights();
	//(ver.>=1.3.4)
	//(ver.>=1.3.6):Added 'attime' param(needed by CalcBBox)
	void DeleteSelVerts(int attime);
	//(ver.>=1.3.4)
	cskin* DetachSelVerts();

	//(ver.>=1.3.5):Mirroring
	BOOL Mirror(int attime,int axis,cmatrix4* tm);//changed Mirror func-see cnode(Geometry only)

	//From cnode
	BOOL Clone(int attime,cnode&) const;
	void SetMaterial(cmaterial*);
	cmaterial* GetMaterial() const;
	BOOL GetBBox(int attime,cpoint3& bmin,cpoint3& bmax);//will set deformed verts bbox
	//(ver.>=1.3.6):Added GetElement()
	BOOL GetElement(celement& elem,int index,UINT elemtype);//Returns FALSE if index out of range OR not such element type
	int GetNumElements(UINT elemtype);
	int GetNumSelElements(UINT elemtype);
	//(ver.>=1.3.4):Added
	BOOL GetSelElementsCenter(UINT elemtype,int attime,cpoint3&);//in LCS, attime is used
	//Render
	BOOL IsRenderable() const {return TRUE;}
	RenderMesh* GetRenderMesh(int attime);
	//Sub-level specific
	int SelectAll();
	int DeselectAll();
	int InvertSelection();
	void DeleteSelected();//(ver.>=1.3.4):Added
	//Returns hit element's index
	int PickElement(int attime,CGLWindow*,int x,int y,int selflags,float delta);
	//These return (min z value)hit element's index if selflags==SEL_ISHIT or number of hit elements
	int PickElementRect(int attime,CGLWindow*,int x,int y,int x2,int y2,int selflags);
	int PickElementCircle(int attime,CGLWindow*,int x,int y,int radius,int selflags);
	int PickElementContour(int attime,CGLWindow*,cpoint3*,int,int selflags);
	//(ver.>=1.3.2):Added cnode's pivot virtual functions
	//(ver.>=1.3.6):Added 'attime' param to the folowing 3 funcs(needed by CalcBBox)
	void ResetTransform(int attime,int tmflags=(TM_ROT|TM_SCALE));//(ver.>=1.3.6 update1):Added tmflags, choose rotation,scale or both
	void CenterGeomToPivot(int attime);
	void CenterPivotToGeom(int attime);
	//(ver.>=1.3.3):Mouse drag handlers(pivoting)
	//(ver.>=1.3.6):Added bUseViewIKThresh,ikthresh(see cnode.h)
	void Move(int attime,const cpoint3&,BOOL bPivotOnly=FALSE,BOOL bIK=FALSE,
		BOOL bUseViewIKThresh=TRUE,float ikthresh=1.0f);//WCS
	void Rotate(int attime,const cquat&,const cpoint3& center,BOOL bPivotOnly=FALSE);//WCS
	void Scale(int attime,const cpoint3&,const cpoint3& center,BOOL bPivotOnly=FALSE);//LCS
	//Hit test
	BOOL HitTest(int attime,CGLWindow*glwnd,int x,int y,float *zvalue,float delta)
	;//{CalcBBox(attime);return cnode::HitTestBBox(attime,glwnd,x,y,zvalue,delta);}
	BOOL HitTestRect(int attime,CGLWindow*glwnd,int x,int y,int x2,int y2)
	;//{CalcBBox(attime);return cnode::HitTestRectBBox(attime,glwnd,x,y,x2,y2);}
	BOOL HitTestCircle(int attime,CGLWindow*glwnd,int x,int y,int radius)
	;//{CalcBBox(attime);return cnode::HitTestCircleBBox(attime,glwnd,x,y,radius);}
	BOOL HitTestContour(int attime,CGLWindow*glwnd,cpoint3*contour,int npts)
	;//{CalcBBox(attime);return cnode::HitTestContourBBox(attime,glwnd,contour,npts);}

	//(ver.>=1.3.5):Added render flags parameter(see cnode.h)
	void Render(int attime,CGLWindow*,UINT rflags);

	//IO methods
	BOOL Load(cio_load& iload,DWORD fileversion);//support for previous file versions
	BOOL Save(cio_save& isave);

	BOOL HandleMessage(DWORD msg,void *Param1,void *Param2);
//	void NodeIsBeingDeleted(cnode*);//now using HandleMessage
//	BOOL MaterialIsBeingRemoved(cmaterial*);//now using HandleMessage
	BOOL IsMaterialUsed(cmaterial*);


protected:

	cpoint3 CalcSelVertsCenter(int attime);//(ver.>=1.3.4):LCS

	int RenderByMaterial(int attime,cmaterial*,int,BOOL,
		cbitarray&,Array<cpoint3>&,Array<cpoint3>&,Array<cpoint3>&);

	//(ver.>=1.3.5):Helper for faster calcs
	void _CalcSmNormals(int attime,Array<cpoint3>& varray);

	//(ver.>=1.3.6):Helper used in rendering
	void _GetFaceNormalArray(int attime,Array<cpoint3>& varray,Array<cpoint3>& array);

	//(ver.>=1.3.5):Rendering arrays for faster calcs
	Array<cpoint3> r_varray;
	Array<cpoint3> r_vnarray;
	Array<cpoint3> r_fnarray;
	cbitarray r_farray;

	//(ver.>=1.3.6):Editing bones without meshing skin vertices
	BOOL bLockGeom;

	cskinvertex	*verts;
	cpoint3		*tverts;
	cface		*faces;
	cbitarray	edges;
	int			nverts,ntverts,nfaces;

	cmaterial* material;
};

//(ver.>=1.3.2)://Use this instead of the new operator and delete using cnode's virtual Delete() func
COREAPI cskin* CreateNewSkin();

#endif //CSKIN_H