// Lightray3D (C) 2002-2006 Tzivenis Sakis

#ifndef FIXED_H
#define FIXED_H

// 16.16 Fixed point class
#define DOUBLE_TO_INT(x) (int)(((long)(x * 65536.0)) >> 16)

inline long fixed_mul(long a, long b) {return MulDiv(a, b, 65536);}
inline long fixed_div(long a, long b) {return MulDiv(a, 65536, b);}
class COREAPI cfixed
{
    private:
        long fx;
    public:
        cfixed()             {}
        ~cfixed()            {}

        cfixed(long l)       {fx = l<<16;}
        cfixed(int i)        {fx = (long)i<<16;}
        cfixed(float f)		 {fx = (long)(f * 65536.0f);}
        cfixed(double d)     {fx = (long)(d * 65536.0);}

        int Int()           {return (int)(fx >> 16);}
        int Frac()          {return (int)(fx & 0xFFFF);}

        operator int()      {return (int)(fx >> 16);}
        operator float()	{return (float)fx / 65536.0f;}
        operator double()   {return (double)fx / 65536.0;}

        cfixed operator +(cfixed a) {cfixed c; c.fx = fx + a.fx; return c;}
        cfixed operator -(cfixed a) {cfixed c; c.fx = fx - a.fx; return c;}
        cfixed operator *(cfixed a) {cfixed c; c.fx = fixed_mul(fx,a.fx); return c;}
        cfixed operator /(cfixed a) {cfixed c; c.fx = fixed_div(fx,a.fx); return c;}

        int operator <(cfixed a)  {return fx < a.fx;}
        int operator >(cfixed a)  {return fx > a.fx;}
        int operator ==(cfixed a) {return fx == a.fx;}
        int operator !=(cfixed a) {return fx != a.fx;}

        cfixed& operator +=(cfixed a) {fx += a.fx; return *this;}
        cfixed& operator -=(cfixed a) {fx -= a.fx; return *this;}
        cfixed& operator *=(cfixed a) {fx  = fixed_mul(fx,a.fx); return *this;}
        cfixed& operator /=(cfixed a) {fx  = fixed_div(fx,a.fx); return *this;}
};


#endif //FIXED_H