// Lightray3D (C) 2002-2006 Tzivenis Sakis

#ifndef GLWND_H
#define GLWND_H

extern unsigned s_stipple[32];//use: glPolygonStipple((GLubyte*)s_stipple)

#include "cmathlib.h"
#include "ccolor.h"
#include "cimage.h"
//#include "ccamera.h"
class ccamera;

#define GLWND_PROJ_ORTHOGRAPHIC		0
#define GLWND_PROJ_PERSPECTIVE		1

#define GLWND_VIEW_XZY				0	//XZY front-back
#define GLWND_VIEW_XYZ				1	//XYZ top-bottom
#define GLWND_VIEW_YZX				2	//YZX left-right

#define GLWND_VIEW_FRONT			0
#define GLWND_VIEW_BACK				1
#define GLWND_VIEW_TOP				2
#define GLWND_VIEW_BOTTOM			3
#define GLWND_VIEW_LEFT				4
#define GLWND_VIEW_RIGHT			5

#define GLWND_RENDER_WIRE			0
#define GLWND_RENDER_FLAT			1
#define GLWND_RENDER_SMOOTH			2
#define GLWND_RENDER_FLAT_TEX		3
#define GLWND_RENDER_SMOOTH_TEX		4

//Some stock colors
#define GLWND_COLOR_BLACK			0
#define GLWND_COLOR_WHITE			1
#define GLWND_COLOR_LTGRAY			2
#define GLWND_COLOR_GRAY			3
#define GLWND_COLOR_DKGRAY			4
#define GLWND_COLOR_RED				5
#define GLWND_COLOR_GREEN			6
#define GLWND_COLOR_BLUE			7
#define GLWND_COLOR_YELLOW			8
#define GLWND_COLOR_CYAN			9
#define GLWND_COLOR_MAGENTA			10
#define GLWND_COLOR_ORANGE			11
#define GLWND_COLOR_LR3D			12//default Lr3D color
//(ver.>=1.3.5)
#define GLWND_COLOR_BKGND			13
#define GLWND_COLOR_GRID			14
#define GLWND_NUM_COLORS			15

//Font sizes
#define GLWND_FONT_12		0
#define GLWND_FONT_14		1
#define GLWND_FONT_18		2
#define GLWND_FONT_RES		3//according to current resolution(up to 1024x768 - 14, higher 18)
//gl font list base
#define GLWND_FONT_BASE 1//1000
//end=GLWND_FONT_BASE + 3 fonts of 256


//internal
#define GLWND_FLAG_USER				1
#define GLWND_FLAG_HASFOCUS			2

#define ISVIEWXZY(type)		(type==GLWND_VIEW_FRONT||type==GLWND_VIEW_BACK)
#define ISVIEWXYZ(type)		(type==GLWND_VIEW_TOP||type==GLWND_VIEW_BOTTOM)
#define ISVIEWYZX(type)		(type==GLWND_VIEW_LEFT||type==GLWND_VIEW_RIGHT)

//defaults
#define GLWND_DEFAULT_PROJ						GLWND_PROJ_ORTHOGRAPHIC
#define GLWND_DEFAULT_VIEW						GLWND_VIEW_FRONT
#define GLWND_DEFAULT_RENDERING					GLWND_RENDER_WIRE
#define GLWND_DEFAULT_DEPTH_NEAR				1.0f//0.1f
#define GLWND_DEFAULT_DEPTH_FAR					4096.0f//1024.0f
#define GLWND_DEFAULT_DRAWWAXIS_ENABLED			TRUE
#define GLWND_DEFAULT_GRID_SIZE					16.0f
//#define GLWND_DEFAULT_BIG_BLOCKS_GRID_SIZE		64.0f
#define GLWND_DEFAULT_GRID_ENABLED				TRUE
#define GLWND_DEFAULT_GRID_DEPTH_ENABLED		TRUE//(ver.>=1.3.6 update1)
#define GLWND_DEFAULT_PRINTINFO_ENABLED			TRUE
#define GLWND_DEFAULT_ZOOM_FACTOR				1.0f
#define GLWND_DEFAULT_FOV						53.0f
#define GLWND_DEFAULT_BKND_COLOR				GLWND_COLOR_BKGND//GLWND_COLOR_LTGRAY
#define GLWND_DEFAULT_FRAME_COLOR				GLWND_COLOR_LR3D//GLWND_COLOR_ORANGE
#define GLWND_DEFAULT_GRID_COLOR				GLWND_COLOR_GRID//GLWND_COLOR_DKGRAY
//#define GLWND_DEFAULT_BIG_GRID_COLOR			GLWND_COLOR_GRAY
#define GLWND_DEFAULT_AMBIENT_COLOR				0.2f //each component value e.g {0.2,0.2,0.2,1.0}


class COREAPI CGLWindow{

public:
	CGLWindow();
	~CGLWindow();

	void Create(HWND);
	void Destroy();

	BOOL SetupPixelFormat();

	void SetFocus(BOOL);
	BOOL HasFocus();

	void Reset();

	HWND GetHWND();
	HDC GetDC();
	HGLRC GetGLRC();

	void SetSize(int w,int h);
//	void SetWidth(int w);
	int GetWidth();
//	void SetHeight(int h);
	int GetHeight();

	void GetNameDesc(char *str);

	ccolor GetStockColor(int stockcol);

	void SetColor(int stockcol);//stock color value
	void SetColor(const ccolor&);
	ccolor GetColor();

	void SetFrameColor(int stockcol);//stock color value
	void SetFrameColor(const ccolor&);
	ccolor GetFrameColor();
	
	void SetGridColor(int stockcol);//stock color value
	void SetGridColor(const ccolor&);
	ccolor GetGridColor();
	ccolor GetBigGridColor();

	void SetAmbient(float r,float g,float b);
	void SetAmbient(const ccolor&);
	ccolor GetAmbient();

	void SetDepthNear(float d);
	float GetDepthNear();
	void SetDepthFar(float d);
	float GetDepthFar();

	void SetFOV(float f);
	float GetFOV();

	void SetZoomFactor(float z);
	float GetZoomFactor();

	void SetGridSize(float g);
	float GetGridSize();

	void SetProjection(int proj);
	int GetProjection();

	void SetRender(int rend);
	int GetRender();

	//draw world axis?
	void EnableWorldAxis(BOOL b);
	BOOL IsWorldAxisEnabled();

	//draw grid?
	void EnableGrid(BOOL b);
	BOOL IsGridEnabled();

	//(ver.>=1.3.6 update1):Enable z buffer write/test?
	void EnableGridDepth(BOOL b);
	BOOL IsGridDepthEnabled();

	//draw view info on screen?
	void EnableInfo(BOOL b);
	BOOL IsInfoEnabled();

	void SetViewType(int v);
	int GetViewType();

	void Translate(float x,float y,float z);
	void Translate(const cpoint3&);
	void SetTranslation(float x,float y,float z);
	void SetTranslation(const cpoint3&);
	void Rotate(float x,float y,float z);
	void Rotate(const cquat&);
	void SetRotation(float x,float y,float z);
	void SetRotation(const cquat&);
	void GetTranslation(float xyz[3]);
	cpoint3 GetTranslation();
	cquat GetRotation();

	float MeasureScrToView(const cpoint3& winpt1,const cpoint3& winpt2);
	float MeasureViewToScr(const cpoint3& pt1,const cpoint3& pt2);

	//(ver.>=1.3.3):Added cpos,mul helper vars.(Default center=0,0,0 - mul=1).
	void MousePan(float dx,float dy,float cx=0.0f,float cy=0.0f,float cz=0.0f,float mul=1.0f);
	void MouseRotate(float dx,float dy);
	void MouseZoom(float dx,float dy,float cx=0.0f,float cy=0.0f,float cz=0.0f,float mul=1.0f);

	void GetCameraXform(cmatrix4&);
	void GetCameraGLXform(float m[16]);
	void GetCameraGLXform(double m[16]);

	void GetViewXform(cmatrix4&);
	void GetViewGLXform(float m[16]);
	void GetViewGLXform(double m[16]);

	void GetProjectionXform(cmatrix4&);
	void GetProjectionGLXform(float m[16]);
	void GetProjectionGLXform(double m[16]);

	void MakeProjXform(cmatrix4&);

	cpoint3 GetViewXdir();
	cpoint3 GetViewYdir();
	cpoint3 GetViewZdir();

	cpoint3 GetCameraXdir();
	cpoint3 GetCameraYdir();
	cpoint3 GetCameraZdir();

	BOOL Project(cpoint3& winpt,float x,float y,float z);
	BOOL UnProject(cpoint3& pt,float scrx,float scry,float scrz=0.5f);
	//(ver.>=1.3.5 update1):Added point arrays
	BOOL Project(cpoint3* winptout,cpoint3* ptin,int npts);
	BOOL UnProject(cpoint3* ptout,cpoint3* winptin,int npts);

	//For now these two are used internally
	void PushTM(const cmatrix4&);
	void PopTM();

	void Begin2D();
	void Begin2D(int orgx,int orgy,int width,int height);
	void End2D();

	void BeginOverlay();
	void EndOverlay();

	void BeginPaintOverlay();
	void EndPaintOverlay();

	void BeginPaint(BOOL bClearColor=TRUE,BOOL bClearZ=TRUE);
	void EndPaint();

	void DrawString(int fsz,char *string,float scrx,float scry,float color[4]);
	void DrawStringOutline(int fsz,char *string,float scrx,float scry,
						   float color[4],float outline[4]);

	void SetCamera(ccamera *cam);
	ccamera* GetCamera() {return camera;}

	//(ver.>=1.3.2):Added bkgnd image.If image is not loaded
	//scene's env bkgnd will be used(if any)
	void EnableBkImage(BOOL b){bEnableBk=b;}
	BOOL IsBkImageEnabled(){return bEnableBk;}
	void SetBkImage(const cimage&);
	cimage* GetBkImage() const;
	void ClearBkImage();//Equivalent to cimage::Clear()

	//(ver.>=1.3.6):Flags used for rendering
	//TODO: Move these funcs to some other proper place
	void EnableLights(BOOL);
	BOOL GetEnableLightsFlag();

protected:
	void DrawViewMisc();
	void DrawFrame();
	void DrawGrid();
	void DrawWorldAxis();
	void RotateVector(float *vec);
	void InvRotateVector(float *vec);


private:
	HWND	hwnd;
	HDC		hdc;
	HGLRC	hGLRC;
	int		width;
	int		height;
	int		view_type;
	int		projection;
	int		flags;//internal
	int		render_mode;//rendering
//	int		color,framecolor,gridcolor,bigridcolor;
	ccolor	color,framecolor,gridcolor;//,bigridcolor;
//	float	ambient[4];
	ccolor	ambient;
	float	depth_clip_near,depth_clip_far;
	BOOL	bworldaxis;
	float	grid_size;
	float	big_grid_size;//make this global?
	BOOL	bgrid;
	BOOL	bgriddepthtest;//(ver.>=1.3.6 update1)
	BOOL	bprintinfo;
	float	translation[3];//world space
	cquat	rotation;//world space
	//orthographic specific
	float	zoomfactor;
	//projection specific
	float	fov;
	//camera
	ccamera *camera;
	//overlay rendering
	HDC over_olddc;
	HGLRC over_oldctx;
	int over_drawbuf;
	//(ver.>=1.3.2):Added bkgnd image
	BOOL bEnableBk;
	cimage bkimage;
	//(ver.>=1.3.6)
	BOOL bEnableLights;
};


#endif //GLWND_H
