// Lightray3D (C) 2002-2006 Tzivenis Sakis

#ifndef INTERSECT_H
#define INTERSECT_H

#define INTRS_EPSILON 1.0e-7// A small positive value

//NOTE: Ray origin,direction must be first transformed in object space.

COREAPI double Intersect_Plane(const cpoint3& pos, const cpoint3& dir,
							   const cpoint3& planenormal, float planeD,
							   cpoint3& ip, cpoint3& in);
COREAPI double Intersect_Box(const cpoint3& pos, const cpoint3& dir,
							 const cpoint3& boxmin, const cpoint3& boxmax,
							 cpoint3& ip, cpoint3& in);
COREAPI double Intersect_Sphere(const cpoint3& pos, const cpoint3& dir,
								const cpoint3& spherecenter, float radius,
								cpoint3& ip, cpoint3& inormal);
COREAPI double Intersect_Cylinder(const cpoint3& pos, const cpoint3& dir,
								  float radius, float height,
								  cpoint3& ip, cpoint3& in);
COREAPI double Intersect_Cone(const cpoint3& pos, const cpoint3& dir,
							  float radius1, float radius2, float height,
							  cpoint3& ip, cpoint3& inormal);

//Second version funcs
COREAPI BOOL Intersect_Box2(const cpoint3& pos,const cpoint3& dir,
							float &t0,float &t1,cpoint3& bmin,cpoint3& bmax);
COREAPI BOOL Intersect_Sphere2(const cpoint3& pos,const cpoint3& dir,
							   float &t0,float &t1,float radius);
COREAPI BOOL Intersect_Cylinder2(const cpoint3& pos,const cpoint3& dir,
								 float &t0,float &t1,float Rsq,float height);


//(ver.>=1.3.5 update1):Added some more ray helper funcs

//Intersect plane, doesn't check for intersection point inside plane area.
// p0: plane point 0
// pnormal: plane normal
// ro&rd: ray origin, ray direction(normalized)
// ip: intersection point( ro+rd*dist)
// dist: distance from ray origin to intersection point
// bCull: backface culling
COREAPI BOOL Ray_IntersectPlane(const cpoint3& p0,const cpoint3& pnormal,
								const cpoint3& ro,const cpoint3& rd,
								cpoint3& ip,float& dist,
								BOOL bCull);
//Intersect triangle, checks for intersection point inside triangle area(depends on bCheckInside parameter).
// p0,p1,p2: triangle points
// normal: triangle normal
// ro&rd: ray origin, ray direction(normalized)
// ip: intersection point( ro+rd*dist)
// dist: distance from ray origin to intersection point
// bCull: backface culling
// bCheckInside: intersection is valid if point inside triangle area
COREAPI BOOL Ray_IntersectTriangle(const cpoint3& p0,const cpoint3& p1,const cpoint3& p2,const cpoint3& normal,
								   const cpoint3& ro,const cpoint3& rd,
								   cpoint3& ip,float& dist,
								   BOOL bCull,
								   BOOL bCheckInside);
//Same but normal(calculated in func)
COREAPI BOOL Ray_IntersectTriangle(const cpoint3& p0,const cpoint3& p1,const cpoint3& p2,
								   const cpoint3& ro,const cpoint3& rd,
								   cpoint3& ip,float& dist,
								   BOOL bCull,
								   BOOL bCheckInside);

#endif //INTERSECT_H
