// Lightray3D (C) 2002-2006 Tzivenis Sakis


#ifndef MESHADJ_H
#define MESHADJ_H



#define UNDEFINED	-1 //0xffffffff

//------------------------------------------------------------------------------------------------------------------------
// AdjFaceList code adapted from:
//---------------------------------
// Source code for "Creating Efficient Triangle Strips"
// (C) 2000, Pierre Terdiman (p.terdiman@wanadoo.fr)
//

//Macros
#define		MAKE_ADJ_TRI(x)			(x&0x3fffffff)
#define		GET_EDGE_NB(x)			(x>>30)
#define		IS_BOUNDARY(x)			(x==0xffffffff)

struct AdjTriangle{
	DWORD	VRef[3];		// Vertex-references
	DWORD	ATri[3];		// Links/References of adjacent triangles. The 2 most significant bits contain
							// the counterpart edge in the adjacent triangle.
	COREAPI BYTE	FindEdge(DWORD vref0, DWORD vref1);
	COREAPI DWORD	OppositeVertex(DWORD vref0, DWORD vref1);
};

struct AdjTriEdge{
	DWORD	Ref0;			// Vertex reference
	DWORD	Ref1;			// Vertex reference
	DWORD	FaceNum;		// Owner face
};

class cface;
class cmesh;
class cskin;

class COREAPI AdjFaceList
{
public:
	AdjFaceList();
	~AdjFaceList();

	//First call Init and pass mesh or faces array.
	//Then call Create to create the lists.
	BOOL					Init(cmesh& m);
	BOOL					Init(cskin& s);
	BOOL					Init(cface *facearray,int nfaces);//this version needs a cface[nfaces*3] array
	BOOL					Init(int *facearray,int nfaces);//this version needs a int[nfaces*3] array
	BOOL					Create();

	AdjTriangle				*mFaces;
	DWORD					mNumFaces;

private:
	AdjTriEdge				*mEdges;
	DWORD					mNumEdges;
	DWORD					mCurrentNumFaces;
	BOOL					AddTriangle(DWORD ref0, DWORD ref1, DWORD ref2);
	BOOL					AddEdge(DWORD ref0, DWORD ref1, DWORD face);
	BOOL					UpdateLink(DWORD firsttri, DWORD secondtri, DWORD ref0, DWORD ref1);
	BYTE					FindEdge(AdjTriangle* tri, DWORD vref0, DWORD vref1);
};


//------------------------------------------------------------------------------------------------------------------------
//
//From here on we do it ourselves
//(c) 2003, Tzivenis Sakis
//
//------------------------------------------------------------------------------------------------------------------------

struct AdjEdge{
	DWORD	Ref0;			// Vertex reference
	DWORD	Ref1;			// Vertex reference
	DWORD	FaceNum;		// Owner face
	DWORD	AFaceNum;		// Adjacent face 
};

class COREAPI AdjEdgeList
{
public:
	AdjEdgeList();
	~AdjEdgeList();

	BOOL					Init(cmesh& m);
	BOOL					Init(cskin& s);
	BOOL					Create();

	// 1 cbitarray per vertex. Each cbitarray is a list of indices into the edge list, 1 for each edge adjacent to the vertex.
	cbitarray				*verts;
	int nverts;

	//Edges list
	AdjEdge					*mEdges;
	DWORD					mNumEdges;

private:
	BOOL					AddEdge(DWORD ref0, DWORD ref1, DWORD face);
	BOOL					UpdateLink(DWORD firsttri, DWORD secondtri, DWORD ref0, DWORD ref1);
};

class COREAPI FaceClusterList
{
public:
	FaceClusterList();
	~FaceClusterList();

	BOOL					Init(cmesh& m,AdjFaceList& adj,BOOL bUseSel,float smoothangle=45.0f,BOOL bUseSmAngle=FALSE);
	BOOL					Init(cskin& m,AdjFaceList& adj,BOOL bUseSel,float smoothangle=45.0f,BOOL bUseSmAngle=FALSE);

	DWORD					*cluster;//clusters array, one for each face
	DWORD					nclusters;//num of mesh faces

private:
	AdjFaceList				*adj;
	DWORD					Recurse(DWORD i,DWORD icluster);
	//helpers
	BOOL					bSelOnly;
	float					smoothangle;
	BOOL					bUseSmAngle;
	cpoint3					*fnarray;
	DWORD					*visited;
};

class COREAPI EdgeClusterList
{
public:
	EdgeClusterList();
	~EdgeClusterList();

	BOOL					Init(cmesh& m,AdjEdgeList& adj,BOOL bUseSel);
	BOOL					Init(cskin& s,AdjEdgeList& adj,BOOL bUseSel);

	DWORD					*cluster;//clusters array, one for each edge
	DWORD					nclusters;//num of mesh edges(num faces * 3)

private:
	AdjEdgeList				*adj;
	DWORD					Recurse(DWORD i,DWORD icluster);
	//helpers
	BOOL					bSelOnly;
	DWORD					*visited;
};



//(ver.>=1.3.6):Error checking
struct ChkTriangle{
	DWORD	VRef[3];		// Vertex-references
	DWORD	Edge[3];		// Edges-Number Of Triangles Sharing The Edges(including self)
							// 1=BOUNDARY, 2=NORMAL, >2=Non-Manifold
							// TODO: cbitarray Edge[3], containing adjacent face indices
	COREAPI BYTE	FindEdge(DWORD vref0, DWORD vref1);
	COREAPI DWORD	OppositeVertex(DWORD vref0, DWORD vref1);
};

struct ChkTriEdge{
	DWORD	Ref0;			// Vertex reference
	DWORD	Ref1;			// Vertex reference
	DWORD	FaceNum;		// Owner face
	DWORD	SharedNum;		// Num triangles sharing this edge
							// 1=BOUNDARY, 2=NORMAL, >2=Non-Manifold
};

class COREAPI CheckFaceList
{
public:
	CheckFaceList();
	~CheckFaceList();

	//First call Init and pass mesh or faces array.
	//Then call Check to create and check the lists.
	BOOL					Init(cmesh& m);
	BOOL					Init(cskin& s);
	BOOL					Init(cface *facearray,int nfaces);//this version needs a cface[nfaces*3] array
	BOOL					Init(int *facearray,int nfaces);//this version needs a int[nfaces*3] array
	BOOL					Check();

	ChkTriangle				*mFaces;
	DWORD					mNumFaces;

private:
	ChkTriEdge				*mEdges;
	DWORD					mNumEdges;
	DWORD					mCurrentNumFaces;
	BOOL					AddTriangle(DWORD ref0, DWORD ref1, DWORD ref2);
	BOOL					AddEdge(DWORD ref0, DWORD ref1, DWORD face);
};

#endif //MESHADJ_H