#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "noise.h"


// Generate turbulent noise across a number of octaves.
// Returns values between -1.0 and 1.0.
MYFLOATVAR turbulence(MYFLOATVAR v[3], MYFLOATVAR freq)
{
	MYFLOATVAR t, vec[3];

	for (t = 0.0 ; freq >= 1.0 ; freq /= 2.0) {
		vec[0] = freq * v[0];
		vec[1] = freq * v[1];
		vec[2] = freq * v[2];
		t += fabs(noise3(vec)) / freq;
	}
	return t;
}


#define MAX_OCTAVES		50
static MYFLOATVAR  exponent_array[MAX_OCTAVES];
// Fractal brownian motion.
// H range fractal dimension 1.0 - 0.0 where 0.0 is total chaos.
// lacunarity gap between successive frequencies,  use 2.0
// Octaves is the number of frequencies in the fBm. 3-7
// Returns values between -1.0 and 1.0.
MYFLOATVAR fBm( MYFLOATVAR point[3], MYFLOATVAR H, MYFLOATVAR lacunarity, MYFLOATVAR octaves )
{
	MYFLOATVAR value = 0.0, frequency = 1.0, remainder = octaves - (int)octaves;
	int     i;
	MYFLOATVAR p[3];
	static MYFLOATVAR oldH = 0.0, oldL = 0.0;
	if (H != oldH || lacunarity != oldL) {
		/* precompute and store special weights */
		for (i=0; i <= octaves; i ++)
		  {
		  // compute weight for each frequency.
		  exponent_array[i] = pow( frequency, -H );
		  frequency *= lacunarity;
		  }
		oldH = H;
		oldL = lacunarity;
	}

	p[0] = point[0];
	p[1] = point[1];
	p[2] = point[2];
	// Inner loop of spectral construction.
	for (i=0; i < octaves; i++)
	{
		value += noise3(p) * exponent_array[i];
		p[0] *= lacunarity;
		p[1] *= lacunarity;
		p[2] *= lacunarity;
	}

	if (remainder)
	{
		value += remainder *noise3(p) * exponent_array[i];
	}
	return(value);
}

