// sxImgLib (C) 2002-2006 Tzivenis Sakis

#ifndef SXIMGLIB_H
#define SXIMGLIB_H

//NOTES:
//Names are case-insensitive
//All funcs will return 1 for success, 0 for failure
//Use free() to deallocate memory
//Check version compatibility
//Check image formats supported

//(ver.>=1.1):Added JPG saving(24bpp), DDS loading
//(ver.>=1.2):Added RAW,PNG loading
//(ver.>=1.3):Added SwapRedBlue func, bSwapRedBlue option for BMP saving

#ifdef SXIMGLIB_EXPORTS
#define SXIMGLIB_API __declspec(dllexport)
#else
#define SXIMGLIB_API __declspec(dllimport)
#endif

//Library Version
#define VERSION_MAJOR	1
#define VERSION_MINOR	3

// Flips The Red And Blue Bytes
void SwapRedBlue(void* buffer,int width,int height,int bpp);//bpp: 32 OR 24

//Supported formats query
SXIMGLIB_API int sxImgLib_SupportsImgType(const char *extension);

//Dialog filters(i.e "BMP (*.bmp)\0*.bmp\0TGA (*.tga)\0*.tga\0 ... \0All files (*.*)\0*.*\0")
SXIMGLIB_API const char* sxImgLib_DlgFilters(int *numFilters);

//Load from file
SXIMGLIB_API int sxImgLib_LoadLBM (const char *filename, unsigned char **data, unsigned char **palette, int *width, int *height);
SXIMGLIB_API int sxImgLib_LoadWAL (const char *filename, unsigned char **data, unsigned char **palette, int *width, int *height);
SXIMGLIB_API int sxImgLib_LoadPCX (const char *filename, unsigned char **data, unsigned char **palette, int *width, int *height);
SXIMGLIB_API int sxImgLib_LoadBMP (const char *filename, unsigned char **data, unsigned char **palette, int *width, int *height);
SXIMGLIB_API int sxImgLib_LoadTGA (const char *filename, unsigned char **data, int *width, int *height);
SXIMGLIB_API int sxImgLib_LoadJPG (const char *filename, unsigned char **data, int *width, int *height);
SXIMGLIB_API int sxImgLib_LoadTIFF(const char *filename, unsigned char **data, int *width, int *height);
//SXIMGLIB_API int sxImgLib_LoadGIF (const char *filename, unsigned char **data, unsigned char **palette, int *width, int *height);
SXIMGLIB_API int sxImgLib_LoadDDS (const char *filename, unsigned char **data, int *width, int *height);
SXIMGLIB_API int sxImgLib_LoadRAW (const char *filename, unsigned char **data, int *width, int *height, int headersize=0);
SXIMGLIB_API int sxImgLib_LoadPNG(const char *filename, unsigned char **data, int *width, int *height);

//Load from buffer
SXIMGLIB_API int sxImgLib_LoadLBMBuffer (const unsigned char *buffer, int bufsize, unsigned char **data, unsigned char **palette, int *width, int *height);
SXIMGLIB_API int sxImgLib_LoadWALBuffer (const unsigned char *buffer, int bufsize, unsigned char **data, unsigned char **palette, int *width, int *height);
SXIMGLIB_API int sxImgLib_LoadPCXBuffer (const unsigned char *buffer, int bufsize, unsigned char **data, unsigned char **palette, int *width, int *height);
SXIMGLIB_API int sxImgLib_LoadBMPBuffer (const unsigned char *buffer, int bufsize, unsigned char **data, unsigned char **palette, int *width, int *height);
SXIMGLIB_API int sxImgLib_LoadTGABuffer (const unsigned char *buffer, int bufsize, unsigned char **data, int *width, int *height);
SXIMGLIB_API int sxImgLib_LoadJPGBuffer (const unsigned char *buffer, int bufsize, unsigned char **data, int *width, int *height);
//SXIMGLIB_API int sxImgLib_LoadTIFFBuffer(const unsigned char *buffer, int bufsize, unsigned char **data, int *width, int *height);
//SXIMGLIB_API int sxImgLib_LoadGIFBuffer (const unsigned char *buffer, int bufsize, unsigned char **data, unsigned char **palette, int *width, int *height);
SXIMGLIB_API int sxImgLib_LoadDDSBuffer (const unsigned char *buffer, int bufsize, unsigned char **data, int *width, int *height);
SXIMGLIB_API int sxImgLib_LoadRAWBuffer (const unsigned char *buffer, int bufsize, unsigned char **data, int *width, int *height, int headersize=0);
//SXIMGLIB_API int sxImgLib_LoadPNGBuffer(const unsigned char *buffer, int bufsize, unsigned char **data, int *width, int *height);

//Load by extension from file
SXIMGLIB_API int sxImgLib_Load (const char *filename, unsigned char **data, unsigned char **palette, int *width, int *height);

//Save to file(if palette exists bpp is ignored else should be 16,24,32)
//NOTE:data should be dword aligned!!!
SXIMGLIB_API int sxImgLib_SaveBMP (const char *filename, unsigned char *data, unsigned char *palette, int width, int height, int bpp, int bSwapRedBlue);
SXIMGLIB_API int sxImgLib_SaveJPG (const char *filename, unsigned char *data,unsigned char* palette, int width, int height, int bpp, int nQuality);

//Save by extension from file(BMP,JPG only for now)
//NOTE:data should be dword aligned!!!
//NOTE:Quality is used for JPG's
//NOTE: bSwapRedBlue is used for BMP
SXIMGLIB_API int sxImgLib_Save (const char *filename, unsigned char *data, unsigned char *palette, int width, int height, int bpp, int quality=100, int bSwapRedBlue=1);

//Free memory
SXIMGLIB_API void sxImgLib_Free (void*);

#endif //SXIMGLIB_H