// Lightray3D (C) 2002-2006 Tzivenis Sakis

// Simple,efficient 2D triangulation code for any contour/polygon(no holes)
// How to use it:
// * If the polygon is 2dimensional set x,y components and pass XY to proj parameter
// * If the polygon is 3dimensional:
//   a) If planar calculate normal and find projection plane yourself
//   b) If not planar try all possible projection axes: XY,YZ,XZ
// NOTE: Triangulator will fail if polygon edges intersect

#ifndef TRIANGULATE_H
#define TRIANGULATE_H

#include "Array.h"

//Polygon points 2D projection flags
#define XY	1
#define YZ	2
#define XZ	3

class COREAPI TriVertex
{
public:
	TriVertex(){mX=0.0f;mY=0.0f;mY=0.0f;}
	TriVertex(float x,float y,float z){Set(x,y,z);}
	void  Set(float x,float y,float z){mX = x;mY = y;mZ = z;}
	float GetX() const { return mX; }
	float GetY() const { return mY; }
	float GetZ() const { return mZ; }
private:
	float mX;
	float mY;
	float mZ;
};

class COREAPI TriTriangle
{
public:
	TriTriangle(){i1=0;i2=0;i3=0;}
	TriTriangle(int index1,int index2,int index3)
	{
		Set(index1,index2,index3);
	}
	void Set(int index1,int index2,int index3){i1=index1;i2=index2;i3=index3;}
	int Get1() const { return i1; }
	int Get2() const { return i2; }
	int Get3() const { return i3; }
private:
	int i1;
	int i2;
	int i3;
};

typedef Array<TriVertex> TriVertexArray;
typedef Array<TriTriangle> TriTriangleArray;

// Order of input vertices is not significant(CW,CCW)
// Order of output triangles is counter-clockwise.
// This is the static triangulator class
class COREAPI Triangulate
{
public:

	// Triangulate a contour/polygon, places results in result as series of triangled vertices.
	// Result number of triangles is nverts-2 & number of triangled vertices is (nverts-2)*3.
	static bool Process(int proj,const TriVertexArray &contour,TriVertexArray &result);
	
	// Triangulate a contour/polygon, places results in result as series of triangle indices
	// to passed contour/polygon vertices.Result number of triangles is nverts-2.
	static bool Process(int proj,const TriVertexArray &contour,TriTriangleArray &result);

	// compute area of a contour/polygon
	static float Area(int proj,const TriVertexArray &contour);
	
	// Check if point Px/Py is inside triangle defined by (Ax,Ay) (Bx,By) (Cx,Cy)
	static bool InsideTriangle(float Ax, float Ay,
							   float Bx, float By,
							   float Cx, float Cy,
							   float Px, float Py);
	
private:
	static bool Cut(int proj,const TriVertexArray &contour,int u,int v,int w,int n,int *V);
};


#endif //TRIANGULATE_H

