//LightRay3D SDK (C) 2002-2006 Tzivenis Sakis

#include <stdio.h>
#include "myassert.h"
#include <windows.h>
#include "resource.h"

#include "plugapi.h"
#include "api.h"
#include "Imodskew.h"
#include "interface.h"
#include "misc.h"//for bbox calcs



static BOOL bWorldCS=FALSE;
static float amount=0.0f;
static int axis=AXIS_Z;
static BOOL smnorms=TRUE;
static cgizmo modgizmo;


cskewmodifier::cskewmodifier()
{
	hInst=NULL;
	IDD=IDD_MODSKEW;//If we have some parameters we must have a dialog, else this should be 0
}

void cskewmodifier::Init()
{
	modgizmo.SetGizmoType(GIZMO_TYPE_BOX);
	Update();//After Init(), Display() will be called so do calculations here
				//for deform gizmo preview to work
}

//This is called every time selection state changes.The app must know if this modifier
//can handle selected nodes.
BOOL cskewmodifier::IsOkToModify(cnode *node)
{
	if(!node) return FALSE;//this isn't the case but just in case

	if(node->GetTypeID()!=TYPE_MESH) return FALSE;//we handle only meshes
	if(node->GetSubLevel()!=SUB_LEVEL_OBJECT
		&& node->GetSubLevel()!=SUB_LEVEL_VERT
		&& node->GetSubLevel()!=SUB_LEVEL_FACE)
		return FALSE;//edge sub level is not the case (could be though)
	if(node->GetSubLevel()==SUB_LEVEL_VERT && ((cmesh*)node)->NumSelVerts()<2) return FALSE;
	if(node->GetSubLevel()==SUB_LEVEL_FACE && ((cmesh*)node)->NumSelFaces()==0) return FALSE;

	return TRUE;
}

//Here we just render the gizmo(preview)
void cskewmodifier::Display(int attime,CGLWindow*glwnd,UINT renderflags)//(ver.>=1.3.5):Added renderflags param
{
	modgizmo.Render(attime,glwnd,renderflags);
}

//Update only the gizmo.Nodes get modified on Execute()
void cskewmodifier::Update()
{
	modgizmo.SetGizmoAxis(axis);
	Calculate();
	DeformGizmo(&modgizmo);
}

BOOL cskewmodifier::Execute()
{
	// Nothing to do
	if(fabs(amount) <0.0001) return FALSE;

	//Do calculations
	Calculate();

	// Nothing to do
	if(fabs(len) <0.0001) return FALSE;

	//Deform
	for(int i=0;i<Interface::NumNodes();i++)
	{
		cnode*node=Interface::GetNodeAt(i);
		//We perform checking ourselves
		if(!(node->GetFlags()&FLAG_SELECTED)) continue;
		if(node->GetTypeID()!=TYPE_MESH) continue;
		
		Deform((cmesh*)node);
	}

	return TRUE;
}

void cskewmodifier::Calculate()
{
	len=0.0f;
	bmin=bmax=MakePt3(0,0,0);
	
	cnode*node=Interface::GetSelNode();

	if(node)//If single selection check for sub level modifying
	{
		assert(node->GetTypeID()==TYPE_MESH);

		if(node->GetSubLevel()==SUB_LEVEL_VERT)
		{
			assert(((cmesh*)node)->NumSelVerts()>=2);
			((cmesh*)node)->GetSelVertsBBox(bmin,bmax);
		}
		else if(node->GetSubLevel()==SUB_LEVEL_FACE)
		{
			assert(((cmesh*)node)->NumSelFaces()!=0);
			((cmesh*)node)->GetSelFacesBBox(bmin,bmax);
		}
		else
		{
			node->GetBBox(TIME_IDLE,bmin,bmax);
		}

		if(bWorldCS)
		{
			//Make WCS bbox
			cmatrix4 tm=node->GetNodeTM(TIME_IDLE);
			MakeWCSBBox(bmin,bmax,tm);
		}
	}
	else//(multi-selection)
	{
		for(int i=0;i<Interface::NumNodes();i++)
		{
			cnode*node=Interface::GetNodeAt(i);
			if(!(node->GetFlags()&FLAG_SELECTED)) continue;
			if(node->GetTypeID()!=TYPE_MESH) continue;
			
			//Get each node's bbox,transform it to world space and make a bbox for all
			cpoint3 min,max;
			node->GetBBox(TIME_IDLE,min,max);
			cmatrix4 tm=node->GetNodeTM(TIME_IDLE);
			//Make WCS bbox
			MakeWCSBBox(min,max,tm);
			if(i==0)
			{
				bmin=min;
				bmax=max;
			}
			else
			{
				if(min.x<bmin.x) bmin.x=min.x;
				if(min.y<bmin.y) bmin.y=min.y;
				if(min.z<bmin.z) bmin.z=min.z;
				if(max.x>bmax.x) bmax.x=max.x;
				if(max.y>bmax.y) bmax.y=max.y;
				if(max.z>bmax.z) bmax.z=max.z;
			}
		}
	}

	//Get axis length for further calculations
	switch (axis)
	{
	case AXIS_X:  len = bmax.x - bmin.x; break;
	case AXIS_Y:  len = bmax.y - bmin.y; break;
	case AXIS_Z:  len = bmax.z - bmin.z; break;
	}

	bbcenter=(bmin+bmax)/2;

	//(ver.>=1.3.5 update1):Setup gizmo

	if(node==NULL || bWorldCS)//world space
	{
		//center to object(s) selection
		modgizmo.SetPosition(0,bbcenter);
		//rotation
		modgizmo.SetRotation(0,MakeQuat(0,0,0,1));
		//scale
		modgizmo.SetScale(0,MakePt3(1,1,1));

		bmax=bmax-bbcenter;
		bmin=bmin-bbcenter;
		bbcenter=MakePt3(0,0,0);
	}
	else if(node!=NULL)//local space
	{
		//Set position
		if(node->GetSubLevel()==SUB_LEVEL_OBJECT)
		{
			modgizmo.SetPosition(0,node->GetPosition(TIME_IDLE));
		}
		else
		{
			//pivot may be off 0,0,0 and we need gizmo centered to bbox center
			cmatrix4 tm=node->GetNodeTM(TIME_IDLE);
			cpoint3 center=tm*bbcenter;
			modgizmo.SetPosition(0,center);

			bmax=bmax-bbcenter;
			bmin=bmin-bbcenter;
			bbcenter=MakePt3(0,0,0);
		}
		//Set rotation
		modgizmo.SetRotation(0,node->GetRotation(TIME_IDLE));
		//Set scale
		modgizmo.SetScale(0,node->GetScale(TIME_IDLE));
	}
}

void cskewmodifier::DeformGizmo(cgizmo*gizmo)
{
	BOOL bSkew=FALSE;

	float amountOverLength;

	if(fabs(amount) >= 0.0001 && fabs(len) >= 0.0001)
	{
		bSkew=TRUE;
		amountOverLength=amount/len;
	}

	gizmo->Reset();//Reset points(unit points unmodified)

	for(int i=0;i<gizmo->NumPoints();i++)
	{
		cpoint3 p=gizmo->GetPoint(i);

		p.x*=(bmax.x - bmin.x);
		p.y*=(bmax.y - bmin.y);
		p.z*=(bmax.z - bmin.z);

		p=p+bbcenter;
		
		if(bSkew)
		{
			if(axis==AXIS_X) p.x += (p.z-bmin.z) * amountOverLength;
			else if(axis==AXIS_Y) p.y += (p.z-bmin.z) * amountOverLength;
			else// if(axis==AXIS_Z)
				p.z += (p.y-bmin.y) * amountOverLength;
		}

		//Set it back
		gizmo->SetPoint(i,p);
	}
}

void cskewmodifier::Deform(cmesh*mesh)
{
	if(fabs(len) <0.0001) return;

	int i;
	float amountOverLength=amount/len;

	cmatrix4 ntm=mesh->GetNodeTM(TIME_IDLE);
	cmatrix4 gtm=modgizmo.GetNodeTM(0);
	cmatrix4 tmin=ntm*Inverse(gtm);//to gizmo space
	cmatrix4 tmout=gtm*Inverse(ntm);//back to node space

	//See which verts will be modified
	cbitarray verts;
	verts.SetSize(mesh->NumVerts(),1);
	if(mesh->GetSubLevel()==SUB_LEVEL_OBJECT)
	{
		verts.SetAll(1);
	}
	else if(mesh->GetSubLevel()==SUB_LEVEL_VERT)
	{
		for(i=0;i<mesh->NumVerts();i++)
		{
			cvertex vert;
			mesh->GetVert(i,vert);
			if(!(vert.wFlags&FLAG_SELECTED)) continue;
			verts.Set(i,1);
		}
	}
	else if(mesh->GetSubLevel()==SUB_LEVEL_FACE)
	{
		for(i=0;i<mesh->NumFaces();i++)
		{
			if(!mesh->CheckFaceVIndices(i)) continue;
			cface face=mesh->GetFace(i);
			if(!(face.wFlags&FLAG_SELECTED)) continue;
			verts.Set(face.v[0],1);
			verts.Set(face.v[1],1);
			verts.Set(face.v[2],1);
		}
	}

	//Skew
	for(i=0;i<mesh->NumVerts();i++)
	{
		if(!verts[i]) continue;

		cpoint3 p=mesh->GetVertPoint(i);
		
		//Transform to gizmo space
		p=tmin*p;

		if(axis==AXIS_X) p.x += (p.z-bmin.z) * amountOverLength;
		else if(axis==AXIS_Y) p.y += (p.z-bmin.z) * amountOverLength;
		else// if(axis==AXIS_Z)
			p.z += (p.y-bmin.y) * amountOverLength;

		//Transform back to node space
		p=tmout*p;

		//Set it back
		mesh->SetVert(i,p);
	}
	mesh->Update();//(ver.>=1.3.1):To calc internal vars
	if(smnorms) mesh->Smooth();//(ver.>=1.3.3)
}



//---------------------------------------------------------------------------------------------
//Dialog procedure(If we have any parameters we must have a dialog)
//---------------------------------------------------------------------------------------------
static HWND hDialog=NULL;
static char txt[32];

static float GetEditFloat(HWND dlg,UINT item)
{
	*(short *)txt=sizeof(txt)-1;
	int l=SendDlgItemMessage(dlg,item,EM_GETLINE,0,(LPARAM)txt);
	txt[l]=0;
	return atof(txt);
}

static void UpdateParams()
{
	if(!hDialog) return;

	if(Interface::GetSelNode()==NULL) ::EnableWindow(::GetDlgItem(hDialog,IDC_SKEW_WCS),FALSE);
	else ::EnableWindow(::GetDlgItem(hDialog,IDC_SKEW_WCS),TRUE);
	SendDlgItemMessage(hDialog,IDC_SKEW_WCS,BM_SETCHECK,(bWorldCS==TRUE)?BST_CHECKED:BST_UNCHECKED,0);

	SendDlgItemMessage(hDialog,IDC_SKEW_X,BM_SETCHECK,(axis==AXIS_X)?BST_CHECKED:BST_UNCHECKED,0);
	SendDlgItemMessage(hDialog,IDC_SKEW_Y,BM_SETCHECK,(axis==AXIS_Y)?BST_CHECKED:BST_UNCHECKED,0);
	SendDlgItemMessage(hDialog,IDC_SKEW_Z,BM_SETCHECK,(axis==AXIS_Z)?BST_CHECKED:BST_UNCHECKED,0);
	SendDlgItemMessage(hDialog,IDC_SKEW_SMOOTH_NORMALS,BM_SETCHECK,(smnorms==TRUE)?BST_CHECKED:BST_UNCHECKED,0);

	sprintf(txt,"%.1f",amount);
	SendDlgItemMessage(hDialog,IDC_SKEW_AMOUNT,WM_SETTEXT,0,(LPARAM)txt);
}

BOOL cskewmodifier::DlgProc(HWND hDlg,UINT uMsg,WPARAM wParam,LPARAM lParam)
{
	switch (uMsg)
	{
	case WM_INITDIALOG:
		{
			hDialog=hDlg;
			UpdateParams();
		}
		return 1;

	case WM_COMMAND:
		{
			switch(LOWORD(wParam))
			{
			case IDC_SKEW_WCS:
				bWorldCS=!bWorldCS;
				UpdateParams();
				Update();
				Interface::UpdateViews();
				break;
			case IDC_SKEW_X:
				axis=AXIS_X;
				UpdateParams();
				Update();
				Interface::UpdateViews();
				break;
			case IDC_SKEW_Y:
				axis=AXIS_Y;
				UpdateParams();
				Update();
				Interface::UpdateViews();
				break;
			case IDC_SKEW_Z:
				axis=AXIS_Z;
				UpdateParams();
				Update();
				Interface::UpdateViews();
				break;
			case IDC_SKEW_SMOOTH_NORMALS:
				smnorms=!smnorms;
				UpdateParams();
				break;

			case IDC_SKEW_AMOUNT:
				if(HIWORD(wParam)!=EN_KILLFOCUS) break;
			case IDOK:
				amount=GetEditFloat(hDlg,IDC_SKEW_AMOUNT);
				if(amount<-999999.0f) amount=-999999.0f;
				else if(amount>999999.0f) amount=999999.0f;
				UpdateParams();
				Update();
				Interface::UpdateViews();
				break;
			}
		}
		break;
	}

	return 0;
}
