//LightRay3D SDK (C) 2002-2006 Tzivenis Sakis

// Sphere.cpp : Defines the entry point for the DLL application.
//
// Project settings:
// Link to core.lib, additional include paths: sdk/include
// additional lib paths: sdk/libs.
// Define PLUGINLIB_EXPORTS.
// Put dll to Plugs folder not StdPlugs.

#include <stdio.h>
#include <windows.h>
#include "resource.h"

#include "plugapi.h"
#include "api.h"
#include "primitives.h"
#include "interface.h"
#include "version.h"
#include "Sphere.h"



#define ZERO_MOUSE_DELTA	1

#define CLAMP_MIN_VALUE	0.0001f
//#define CLAMP_MAX_VALUE	99999999.0f
#define CLAMP_MIN_SIZE(fval)	(fval<CLAMP_MIN_VALUE?fval=CLAMP_MIN_VALUE:fval)
//#define CLAMP_MAX_SIZE(fval)	(fval>CLAMP_MAX_VALUE?fval=CLAMP_MAX_VALUE:fval)

static BOOL _bCorner=FALSE,_bCornerAfterBeginDrag=FALSE;
static cpoint3 _CornerStartPos;



PLUGINAPI ccreatesphere spherelib;


BOOL APIENTRY DllMain( HANDLE hModule, 
                       DWORD  ul_reason_for_call, 
                       LPVOID lpReserved
					 )
{
	spherelib.hInst=(HINSTANCE)hModule;

    switch (ul_reason_for_call)
	{
		case DLL_PROCESS_ATTACH:
		case DLL_THREAD_ATTACH:
		case DLL_THREAD_DETACH:
		case DLL_PROCESS_DETACH:
			break;
    }
    return TRUE;
}

//These must exist.Make a *.def file.
//We can have multiple classes in one dll(i.e More primitives)
PLUGINAPI ULONG LibType(int i)
{
	return (LIB_CREATE);
}

PLUGINAPI int LibNumberClasses(){return 1;}

PLUGINAPI void *LibClassDesc(int i)
{
	return (void*)&spherelib;
}

PLUGINAPI ULONG LibVersion(int i)
{
	return (APP_VERSION);
}



//---------------------------------------------------------------------------------------------
//Dialog procedure(If we have some parameters we must have a dialog)

static char txt[32];

//Helpers
static float GetEditFloat(HWND dlg,UINT item)
{
	*(short *)txt=sizeof(txt)-1;
	int l=SendDlgItemMessage(dlg,item,EM_GETLINE,0,(LPARAM)txt);
	txt[l]=0;
	return atof(txt);
}
static int GetEditInt(HWND dlg,UINT item)
{
	*(short *)txt=sizeof(txt)-1;
	int l=SendDlgItemMessage(dlg,item,EM_GETLINE,0,(LPARAM)txt);
	txt[l]=0;
	return atoi(txt);
}


static HWND sphere_hDialog=NULL;
static csphere sphere;
static cmesh *spheremesh=NULL;

static float sphere_radius=0;//32;
static int sphere_rings=12,sphere_sections=12;
static float sphere_szx=0,sphere_szy=0;

static void UpdateSphereParams()
{
	if(!sphere_hDialog) return;
	sprintf(txt,"%.4f",sphere_radius);
	SendDlgItemMessage(sphere_hDialog,IDC_SPHERE_RADIUS,WM_SETTEXT,0,(LPARAM)txt);
	sprintf(txt,"%d",sphere_rings);
	SendDlgItemMessage(sphere_hDialog,IDC_SPHERE_RINGS,WM_SETTEXT,0,(LPARAM)txt);
	sprintf(txt,"%d",sphere_sections);
	SendDlgItemMessage(sphere_hDialog,IDC_SPHERE_SECTIONS,WM_SETTEXT,0,(LPARAM)txt);
	//(ver.>=1.3.6 update1)
	if(_bCorner)
		CheckRadioButton(sphere_hDialog,IDC_SPHERE_CENTER,IDC_SPHERE_CORNER,IDC_SPHERE_CORNER);
	else
		CheckRadioButton(sphere_hDialog,IDC_SPHERE_CENTER,IDC_SPHERE_CORNER,IDC_SPHERE_CENTER);
}

BOOL ccreatesphere::DlgProc(HWND hDlg,UINT uMsg,WPARAM wParam,LPARAM lParam)
{
	int i;
	float f;

	switch (uMsg)
	{
	case WM_INITDIALOG:
		{
			sphere_hDialog=hDlg;
			UpdateSphereParams();
		}
		return 1;

	case WM_COMMAND:
		{
			//(ver.>=1.3.6 update1)
			switch(LOWORD(wParam))
			{
			case IDC_SPHERE_CENTER:
				CheckRadioButton(hDlg,IDC_SPHERE_CENTER,IDC_SPHERE_CORNER,IDC_SPHERE_CENTER);
				_bCorner=FALSE;
				break;
			case IDC_SPHERE_CORNER:
				CheckRadioButton(hDlg,IDC_SPHERE_CENTER,IDC_SPHERE_CORNER,IDC_SPHERE_CORNER);
				_bCorner=TRUE;
				break;
			}

			int id=0;
			if(LOWORD(wParam)==IDOK)
			{
				HWND fhwnd=::GetFocus();
				if(fhwnd) id=::GetDlgCtrlID(fhwnd);
			}
			else if(HIWORD(wParam)==EN_KILLFOCUS) id=LOWORD(wParam);

			if(id)
			{
				switch(id)
				{
				case IDC_SPHERE_RADIUS:
					f=GetEditFloat(hDlg,IDC_SPHERE_RADIUS);
					CLAMP_MIN_SIZE(f);
					sphere_radius=f;
					break;
				case IDC_SPHERE_RINGS:
					i=GetEditInt(hDlg,IDC_SPHERE_RINGS);
					if(i>0) sphere_rings=i;
					break;
				case IDC_SPHERE_SECTIONS:
					i=GetEditInt(hDlg,IDC_SPHERE_SECTIONS);
					if(i>=3) sphere_sections=i;
					break;
				}
				UpdateSphereParams();
				sphere.SetRadius(sphere_radius);
				sphere.SetRings(sphere_rings);
				sphere.SetSections(sphere_sections);
				//After creation
				if(spheremesh)
				{
					sphere.BuildMesh(*spheremesh);
					Interface::UpdateViews();
				}
			}
		}
		break;
	}

	return 0;
}


//-----------------------------------------------------------------------------------------------------------------------
ccreatesphere::ccreatesphere()
{
	hInst=NULL;
	IDD=IDD_PRIM_SPHERE;//If we have some parameters we must have a dialog, else this should be 0
}

void ccreatesphere::Init()
{
	spheremesh=NULL;
	sphere_radius=0;
}

void ccreatesphere::Done()
{
	spheremesh=NULL;
}


//User has just clicked in a view
BOOL ccreatesphere::BeginDrag(CGLWindow *glwnd,int scrx,int scry,
							  float x,float y,float z,
							  float dirx,float diry,float dirz,
							  float upx,float upy,float upz)
{
	sphere_szx=sphere_szy=0;
	sphere_radius=0;UpdateSphereParams();//reset & update

	cmesh *s=CreateNewMesh();
	s->SetName("Sphere1");
	s->SetPosition(TIME_IDLE,MakePt3(x,y,z));
	cmatrix4 dirtm;
	MatrixLookAt(dirtm,MakePt3(dirx,diry,dirz),MakePt3(upx,upy,upz));
	s->SetRotation(TIME_IDLE,QuatFromMatrix(dirtm));
	sphere.Create(*s,sphere_radius,sphere_rings,sphere_sections,1.0f,1.0f,1.0f);
	spheremesh=s;
	Interface::AddToScene(s);
	_bCornerAfterBeginDrag=_bCorner;
	_CornerStartPos=MakePt3(x,y,z);
	return TRUE;
}


//User drags mouse(left button is down)
BOOL ccreatesphere::Drag(CGLWindow *glwnd,int scrx,int scry,int scrdx,int scrdy,
						 float x,float y,float z,float sx,float sy)
{
	if(!spheremesh) return FALSE;//(ver.>=1.3.6 update1)

	if(_bCornerAfterBeginDrag)//(ver.>=1.3.6 update1)
	{
		sphere_szx=sx;
		sphere_szy=sy;
		float rad=sphere_radius=(float)(sqrt(sphere_szx*sphere_szx + sphere_szy*sphere_szy))/2;
		sphere.SetRadius(rad);
		sphere.BuildMesh(*spheremesh);

		//Center pivot
		cpoint3 dist=MakePt3(x,y,z)-_CornerStartPos;
		cpoint3 center=_CornerStartPos+(dist/2);
		spheremesh->SetPosition(TIME_IDLE,center);
		Interface::UpdateSelPos();
	}
	else
	{
	sphere_szx=sx;
	sphere_szy=sy;
	float rad=sphere_radius=(float)(sqrt(sphere_szx*sphere_szx + sphere_szy*sphere_szy));
	sphere.SetRadius(rad);
	sphere.BuildMesh(*spheremesh);
	}

	UpdateSphereParams();

	return TRUE;
}

//User releases mouse button
BOOL ccreatesphere::EndDrag(CGLWindow *glwnd,int scrx,int scry,int scrdx,int scrdy,
							float x,float y,float z,float sx,float sy)
{
	if(!spheremesh) return FALSE;//(ver.>=1.3.6 update1)

	//(ver.>=1.3.6 update1):Create nothing if no dragging
	//{
	if(abs(scrdx)<=ZERO_MOUSE_DELTA && abs(scrdy)<=ZERO_MOUSE_DELTA)
	{
		Interface::RemoveFromScene(spheremesh,FALSE);
		spheremesh->Delete();
		spheremesh=NULL;
		return TRUE;
	}
	//}

	if(sphere_szx==0 && sphere_szy==0)
	{
		sphere.SetRadius(sphere_radius);
	}
	else
	{
		sphere_radius=(float)(sqrt(sphere_szx*sphere_szx + sphere_szy*sphere_szy));//sphere_szx+sphere_szy;
	}
	UpdateSphereParams();
	sphere.BuildMesh(*spheremesh);
	return TRUE;
}

//User pressed right mouse button
BOOL ccreatesphere::Stop(CGLWindow *glwnd,int scrx,int scry,int scrdx,int scrdy,
					   float x,float y,float z,float szx,float szy)
{
	if(spheremesh && (GetAsyncKeyState(VK_LBUTTON) & 0x8000))
	{
		Interface::RemoveFromScene(spheremesh,FALSE);
		spheremesh->Delete();
		spheremesh=NULL;
		return TRUE;
	}
	return FALSE;
}

//HBITMAP icon for listbox
void* ccreatesphere::GetHBitmap()
{
	HBITMAP hbmp=LoadBitmap(hInst,MAKEINTRESOURCE(IDB_PRIM_SPHERE));
	return hbmp;
}

void ccreatesphere::DeleteHBitmap(void*hbmp)
{
	if(hbmp)
	{
		DeleteObject((HBITMAP)hbmp);
	}
}
