//LightRay3D SDK (C) 2002-2006 Tzivenis Sakis

// stdutil.cpp : Defines the entry point for the DLL application.
//
// Project settings:
// Link to core.lib, additional include paths: sdk/include
// additional lib paths: sdk/libs.
// Define PLUGINLIB_EXPORTS.
// Put dll to Plugs folder not StdPlugs.


#include <stdio.h>
#include <windows.h>
#include <commctrl.h>
#include "resource.h"
#include "myassert.h"

#include "plugapi.h"
#include "api.h"
#include "cnode.h"
#include "interface.h"
#include "version.h"
#include "stdutil.h"
#include "ogl.h"


static void ReadINI();
static void SaveINI();

PLUGINAPI cobjnames objnames;
PLUGINAPI cscrcapture scrcap;


BOOL APIENTRY DllMain( HANDLE hModule, 
                       DWORD  ul_reason_for_call, 
                       LPVOID lpReserved
					 )
{
	objnames.hInst=(HINSTANCE)hModule;
	scrcap.hInst=(HINSTANCE)hModule;

    switch (ul_reason_for_call)
	{
		case DLL_PROCESS_ATTACH:
		case DLL_THREAD_ATTACH:
			ReadINI();
			break;
		case DLL_THREAD_DETACH:
		case DLL_PROCESS_DETACH:
			SaveINI();
			break;
    }
    return TRUE;
}


PLUGINAPI ULONG LibType(int i)
{
	return (LIB_UTILITY);
}

PLUGINAPI int LibNumberClasses(){return 2;}

PLUGINAPI void *LibClassDesc(int i)
{
	if(i==0) return (void*)&objnames;
	else if(i==1) return (void*)&scrcap;
	else return NULL;
}

PLUGINAPI ULONG LibVersion(int i)
{
	return (APP_VERSION);
}


//---------------------------------------------------------------------------------------------
//Dialog helpers
static char txt[32];
static float GetEditFloat(HWND dlg,UINT item)
{
	*(short *)txt=sizeof(txt)-1;
	int l=SendDlgItemMessage(dlg,item,EM_GETLINE,0,(LPARAM)txt);
	txt[l]=0;
	return atof(txt);
}
static int GetEditInt(HWND dlg,UINT item)
{
	*(short *)txt=sizeof(txt)-1;
	int l=SendDlgItemMessage(dlg,item,EM_GETLINE,0,(LPARAM)txt);
	txt[l]=0;
	return atoi(txt);
}


//---------------------------------------------------------------------------------------------
//Object Names
//---------------------------------------------------------------------------------------------

static HWND hDialogObjNames=NULL;
static BOOL bMeshes=TRUE,bShapes=TRUE,bSkins=TRUE,bBones=TRUE,bLights=TRUE,bCameras=TRUE;
static BOOL bOutline=TRUE,bSelected=FALSE,bLines=TRUE;
static COLORREF txtcol=RGB(0,255,0),txtoutcol=RGB(0,0,0);
static int xoffset=10,yoffset=10;

static void UpdateObjParams()
{
	if(!hDialogObjNames) return;

	SendDlgItemMessage(hDialogObjNames,IDC_OBJ_TXT_OUT,BM_SETCHECK,bOutline?BST_CHECKED:BST_UNCHECKED,0);
	SendDlgItemMessage(hDialogObjNames,IDC_OBJ_SEL_ONLY,BM_SETCHECK,bSelected?BST_CHECKED:BST_UNCHECKED,0);
	SendDlgItemMessage(hDialogObjNames,IDC_OBJ_PIVOT_LINES,BM_SETCHECK,bLines?BST_CHECKED:BST_UNCHECKED,0);

	SendDlgItemMessage(hDialogObjNames,IDC_OBJ_MESHES,BM_SETCHECK,bMeshes?BST_CHECKED:BST_UNCHECKED,0);
	SendDlgItemMessage(hDialogObjNames,IDC_OBJ_SKINS,BM_SETCHECK,bSkins?BST_CHECKED:BST_UNCHECKED,0);
	SendDlgItemMessage(hDialogObjNames,IDC_OBJ_SHAPES,BM_SETCHECK,bShapes?BST_CHECKED:BST_UNCHECKED,0);
	SendDlgItemMessage(hDialogObjNames,IDC_OBJ_BONES,BM_SETCHECK,bBones?BST_CHECKED:BST_UNCHECKED,0);
	SendDlgItemMessage(hDialogObjNames,IDC_OBJ_LIGHTS,BM_SETCHECK,bLights?BST_CHECKED:BST_UNCHECKED,0);
	SendDlgItemMessage(hDialogObjNames,IDC_OBJ_CAMERAS,BM_SETCHECK,bCameras?BST_CHECKED:BST_UNCHECKED,0);
	char txt[32];
	sprintf(txt,"%d",xoffset);
	SendDlgItemMessage(hDialogObjNames,IDC_OBJ_TXT_XOFFSET,WM_SETTEXT,0,(LPARAM)txt);
	sprintf(txt,"%d",yoffset);
	SendDlgItemMessage(hDialogObjNames,IDC_OBJ_TXT_YOFFSET,WM_SETTEXT,0,(LPARAM)txt);

	//Not needed
//	::InvalidateRect(::GetDlgItem(hDialogObjNames,IDC_TXT_COL),NULL);
//	::InvalidateRect(::GetDlgItem(hDialogObjNames,IDC_TXT_OUT_COL),NULL);

}

BOOL cobjnames::DlgProc(HWND hDlg,UINT uMsg,WPARAM wParam,LPARAM lParam)
{
	switch (uMsg)
	{
	case WM_INITDIALOG:
		{
			hDialogObjNames=hDlg;
			UpdateObjParams();
		}
		return 1;

	case WM_COMMAND:
		{
			if(LOWORD(wParam)==IDC_OBJ_TXT_COL)
			{
				if(Interface::DlgPickColor(txtcol))
				{
					UpdateObjParams();
					Interface::UpdateViews();
				}
			}
			else if(LOWORD(wParam)==IDC_OBJ_TXT_OUT_COL)
			{
				if(Interface::DlgPickColor(txtoutcol))
				{
					UpdateObjParams();
					Interface::UpdateViews();
				}
			}

			else if(LOWORD(wParam)==IDC_OBJ_TXT_OUT) {bOutline=!bOutline;UpdateObjParams();Interface::UpdateViews();}
			else if(LOWORD(wParam)==IDC_OBJ_SEL_ONLY) {bSelected=!bSelected;UpdateObjParams();Interface::UpdateViews();}
			else if(LOWORD(wParam)==IDC_OBJ_PIVOT_LINES) {bLines=!bLines;UpdateObjParams();Interface::UpdateViews();}

			else if(LOWORD(wParam)==IDC_OBJ_MESHES) {bMeshes=!bMeshes;UpdateObjParams();Interface::UpdateViews();}
			else if(LOWORD(wParam)==IDC_OBJ_SKINS) {bSkins=!bSkins;UpdateObjParams();Interface::UpdateViews();}
			else if(LOWORD(wParam)==IDC_OBJ_SHAPES) {bShapes=!bShapes;UpdateObjParams();Interface::UpdateViews();}
			else if(LOWORD(wParam)==IDC_OBJ_BONES) {bBones=!bBones;UpdateObjParams();Interface::UpdateViews();}
			else if(LOWORD(wParam)==IDC_OBJ_LIGHTS) {bLights=!bLights;UpdateObjParams();Interface::UpdateViews();}
			else if(LOWORD(wParam)==IDC_OBJ_CAMERAS) {bCameras=!bCameras;UpdateObjParams();Interface::UpdateViews();}

			else if(LOWORD(wParam)==IDC_OBJ_ALL)
			{
				bMeshes=bSkins=bShapes=bBones=bLights=bCameras=TRUE;
				UpdateObjParams();
				Interface::UpdateViews();
			}
			else if(LOWORD(wParam)==IDC_OBJ_NONE)
			{
				bMeshes=bSkins=bShapes=bBones=bLights=bCameras=FALSE;
				UpdateObjParams();
				Interface::UpdateViews();
			}
			else if(LOWORD(wParam)==IDC_OBJ_INV)
			{
				bMeshes=!bMeshes;
				bSkins=!bSkins;
				bShapes=!bShapes;
				bBones=!bBones;
				bLights=!bLights;
				bCameras=!bCameras;
				UpdateObjParams();
				Interface::UpdateViews();
			}
			else
			{
				int id=0;
				if(LOWORD(wParam)==IDOK)
				{
					HWND fhwnd=::GetFocus();
					if(fhwnd) id=::GetDlgCtrlID(fhwnd);
				}
				else if(HIWORD(wParam)==EN_KILLFOCUS) id=LOWORD(wParam);

				if(id)
				{
					switch(id)
					{
					case IDC_OBJ_TXT_XOFFSET:
						xoffset=GetEditInt(hDlg,IDC_OBJ_TXT_XOFFSET);
						UpdateObjParams();
						Interface::UpdateViews();
						break;
					case IDC_OBJ_TXT_YOFFSET:
						yoffset=GetEditInt(hDlg,IDC_OBJ_TXT_YOFFSET);
						UpdateObjParams();
						Interface::UpdateViews();
						break;
					}
				}
			}
		}
		break;

		//WM_DRAWITEM message is handled by App's WndHook but this
		//message is posted for extra drawing
	case (WM_LR3D_HOOK_DRAWITEM):
		{
			LPDRAWITEMSTRUCT lpdis = (LPDRAWITEMSTRUCT)lParam;
			if(lpdis->hwndItem==::GetDlgItem(hDlg,IDC_OBJ_TXT_COL))
			{
				RECT rc=lpdis->rcItem;
				rc.left+=3;rc.right-=3;
				rc.top+=3;rc.bottom-=3;
				::SetBkColor(lpdis->hDC, txtcol);
				::ExtTextOut(lpdis->hDC, 0, 0, ETO_OPAQUE, &rc, NULL, 0, NULL);
				return 1;
			}
			else if(lpdis->hwndItem==::GetDlgItem(hDlg,IDC_OBJ_TXT_OUT_COL))
			{
				RECT rc=lpdis->rcItem;
				rc.left+=3;rc.right-=3;
				rc.top+=3;rc.bottom-=3;
				::SetBkColor(lpdis->hDC, txtoutcol);
				::ExtTextOut(lpdis->hDC, 0, 0, ETO_OPAQUE, &rc, NULL, 0, NULL);
				return 1;
			}
		}
	}

	return 0;
}

cobjnames::cobjnames()
{
	hInst=NULL;
	IDD=IDD_UTIL_OBJNAMES;
}

void cobjnames::Display(int attime,CGLWindow*glwnd,UINT renderflags)
{
	int nnodes=Interface::NumNodes();
	if(nnodes==0) return;

	glPushAttrib(GL_POLYGON_BIT|GL_ENABLE_BIT|GL_POINT_BIT|GL_DEPTH_BUFFER_BIT);

	glDisable(GL_TEXTURE_2D);
	glDisable(GL_BLEND);
	glDisable(GL_ALPHA_TEST);
	glDisable(GL_DITHER);
	glDisable(GL_LIGHTING);
	glDisable(GL_DEPTH_TEST);

	ccolor clr,out;
	clr.Set((int)RGB_BLUE(txtcol),(int)RGB_GREEN(txtcol),(int)RGB_RED(txtcol));
	out.Set((int)RGB_BLUE(txtoutcol),(int)RGB_GREEN(txtoutcol),(int)RGB_RED(txtoutcol));

	glwnd->Begin2D();

	for(int i=0;i<nnodes;i++)
	{
		cnode *node=Interface::GetNodeAt(i);
		if((node->GetFlags()&FLAG_HIDDEN)||
			(node->GetFlags()&FLAG_HOLD)) continue;

		if(bSelected && !(node->GetFlags()&FLAG_SELECTED)) continue;

		UINT type=node->GetTypeID();
		if(type==TYPE_MESH && !bMeshes) continue;
		if(type==TYPE_SKIN && !bSkins) continue;
		if(type==TYPE_SHAPE && !bShapes) continue;
		if(type==TYPE_BONE && !bBones) continue;
		if(type==TYPE_LIGHT && !bLights) continue;
		if(type==TYPE_CAMERA)
		{
			if(!bCameras) continue;
			if(glwnd->GetCamera()==((ccamera*)node)) continue;
		}

		cpoint3 scrpos,pos=node->GetPosition(attime);
		//Project to screen
		glwnd->Project(scrpos,pos.x,pos.y,pos.z);

		if(bOutline)
			glwnd->DrawStringOutline(GLWND_FONT_RES,node->GetName(),scrpos.x+xoffset,scrpos.y-yoffset,clr,out);
		else
			glwnd->DrawString(GLWND_FONT_RES,node->GetName(),scrpos.x+xoffset,scrpos.y-yoffset,clr);
	}

	//Lines To Pivot
	if(bLines)
	{
		glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);//Is this necessary???
		glBegin(GL_LINES);
		glColor3f(clr.r,clr.g,clr.b);
		for(i=0;i<nnodes;i++)
		{
			cnode *node=Interface::GetNodeAt(i);
			if((node->GetFlags()&FLAG_HIDDEN)||
				(node->GetFlags()&FLAG_HOLD)) continue;

			if(bSelected && !(node->GetFlags()&FLAG_SELECTED)) continue;

			UINT type=node->GetTypeID();
			if(type==TYPE_MESH && !bMeshes) continue;
			if(type==TYPE_SKIN && !bSkins) continue;
			if(type==TYPE_SHAPE && !bShapes) continue;
			if(type==TYPE_BONE && !bBones) continue;
			if(type==TYPE_LIGHT && !bLights) continue;
			if(type==TYPE_CAMERA)
			{
				if(!bCameras) continue;
				if(glwnd->GetCamera()==((ccamera*)node)) continue;
			}

			cpoint3 scrpos,pos=node->GetPosition(attime);
			//Project to screen
			glwnd->Project(scrpos,pos.x,pos.y,pos.z);

			glVertex3f(scrpos.x,scrpos.y,0.0f);
			glVertex3f(scrpos.x+xoffset,scrpos.y-yoffset,0.0f);
		}
		glEnd();
	}

	glwnd->End2D();
	glPopAttrib();
}

void cobjnames::Close()
{
}

BOOL cobjnames::Run()
{
	return TRUE;
}






//---------------------------------------------------------------------------------------------
//Scrshot utility
//---------------------------------------------------------------------------------------------
#include "sxImgLib.h"

static HWND hDialogScrshot=NULL;
static BOOL bActive=TRUE,bAll=FALSE,bWnd=FALSE;
static BOOL ParseFileDlg(char *filename,char *Title,char *szFilter,char *szDefExt);

static void UpdateScrParams()
{
	if(!hDialogScrshot) return;

	SendDlgItemMessage(hDialogScrshot,IDC_SCR_ACTIVE,BM_SETCHECK,(bActive)?BST_CHECKED:BST_UNCHECKED,0);
	SendDlgItemMessage(hDialogScrshot,IDC_SCR_ALL,BM_SETCHECK,(bAll)?BST_CHECKED:BST_UNCHECKED,0);
	SendDlgItemMessage(hDialogScrshot,IDC_SCR_WND,BM_SETCHECK,(bWnd)?BST_CHECKED:BST_UNCHECKED,0);
}

BOOL cscrcapture::DlgProc(HWND hDlg,UINT uMsg,WPARAM wParam,LPARAM lParam)
{
	switch (uMsg)
	{
	case WM_INITDIALOG:
		{
			hDialogScrshot=hDlg;
			UpdateScrParams();
		}
		return 1;

	case WM_COMMAND:
		{
			if(LOWORD(wParam)==IDC_SCR_ACTIVE)
			{
				bActive=TRUE;
				bAll=FALSE;
				bWnd=FALSE;
				UpdateScrParams();
			}
			else if(LOWORD(wParam)==IDC_SCR_ALL)
			{
				bActive=FALSE;
				bAll=TRUE;
				bWnd=FALSE;
				UpdateScrParams();
			}
			else if(LOWORD(wParam)==IDC_SCR_WND)
			{
				bActive=FALSE;
				bAll=FALSE;
				bWnd=TRUE;
				UpdateScrParams();
			}
			else if(LOWORD(wParam)==IDC_SCR_SAVE)
			{
				char filename[256]="";
				char *szFilter="(*.JPG)\0*.JPG\0""(*.BMP)\0*.BMP\0""All Files (*.*)\0*.*\0";
				char *szDefExt="*.JPG";

				//Active view
				if(bActive)
				{
					CGLWindow *glwnd=Interface::GetActiveGLView();
					glwnd->SetFocus(glwnd->HasFocus());
					int width=glwnd->GetWidth();
					int height=glwnd->GetHeight();
					BYTE *bits=(BYTE*)malloc(height*width*4);//height*((width*3 + 3) & ~3));
					assert(bits);

					//(ver.>=1.3.6):Had problems with Radeon 9200 and AVIs, so better set Read Buffer to GL_FRONT
					//NOTE: These funcs have no problems but just in case...
					int iReadBuffer;
					glGetIntegerv(GL_READ_BUFFER,&iReadBuffer);
					glReadBuffer(GL_FRONT);// Reading front buffer
					glReadPixels(0,0,width,height,GL_BGRA_EXT,GL_UNSIGNED_BYTE,bits);
					glReadBuffer(iReadBuffer);// Set old value back
					
					if(ParseFileDlg(filename,"Save Image As",szFilter,szDefExt)
						&& filename[0]!=0)
					{
						//JPG case
						int quality=100;
						char *ext=strrchr(filename,'.');
						if(ext && (stricmp(ext,".jpg")==0 || stricmp(ext,".JPG")==0))
							quality=Interface::JPGQualityDlg();
						
						sxImgLib_Save (filename, (unsigned char*)bits, NULL, width, height, 32/*24*/, quality, 0);
						Interface::CreateImageWnd(width,height);
						Interface::ShowImage(bits,NULL,width,height,32,FALSE,FALSE);
					//	char *title=strrchr(filename,'\\');if(title==NULL) title=strrchr(filename,'/');
					//	Interface::ShowImage(filename,title);
					}
					free(bits);
				}
				//All views
				else if(bAll)
				{
					RECT rect,rcv={10000,10000,-10000,-10000};
					//Calc rect
					for(int i=0;i<I_NUM_VIEWS;i++)
					{
						if(Interface::IsViewEnabled(i)==FALSE) continue;
						CGLWindow *glwnd=Interface::GetGLView(i);
						HWND hwnd=glwnd->GetHWND();
						::GetWindowRect(hwnd,&rect);
						if(rcv.left>rect.left) rcv.left=rect.left;
						if(rcv.right<rect.right) rcv.right=rect.right;
						if(rcv.top>rect.top) rcv.top=rect.top;
						if(rcv.bottom<rect.bottom) rcv.bottom=rect.bottom;
					}
					int width=rcv.right-rcv.left;
					int height=rcv.bottom-rcv.top;
					if(width>=10000 || height>=10000)
					{
						MessageBox(NULL,"No views visible!??!",PLUGNAME_SCRSHOT,MB_ICONERROR);
						break;
					}

					CGLWindow *act_glwnd=Interface::GetActiveGLView();

					BYTE *bits=(BYTE*)malloc(height*width*4);//height*((width*3 + 3) & ~3));
					assert(bits);

					//Parse and copy bits
					for(i=0;i<I_NUM_VIEWS;i++)
					{
						if(Interface::IsViewEnabled(i)==FALSE) continue;
						CGLWindow *glwnd=Interface::GetGLView(i);
						HWND hwnd=glwnd->GetHWND();
						::GetWindowRect(hwnd,&rect);

						//set focus
					//	glwnd->SetFocus(TRUE);
						wglMakeCurrent(glwnd->GetDC(),glwnd->GetGLRC());//hack to avoid active frame drawing

						int vwidth=glwnd->GetWidth();
						int vheight=glwnd->GetHeight();
						BYTE *vbits=(BYTE*)malloc(vheight*vwidth*4);//vheight*((vwidth*3 + 3) & ~3));
						assert(vbits);

						//(ver.>=1.3.6):Had problems with Radeon 9200 and AVIs, so better set Read Buffer to GL_FRONT
						//NOTE: These funcs have no problems but just in case...
						int iReadBuffer;
						glGetIntegerv(GL_READ_BUFFER,&iReadBuffer);
						glReadBuffer(GL_FRONT);// Reading front buffer
						glReadPixels(0,0,vwidth,vheight,GL_BGRA_EXT,GL_UNSIGNED_BYTE,vbits);
						glReadBuffer(iReadBuffer);// Set old value back

						int startx=rect.left-rcv.left;
					//	int starty=rect.top-rcv.top;
						int starty=(height-(rect.top-rcv.top))-vheight;//reverse
						if(starty<0) starty=0;//just in case...

						//Copy scanlines
						for(int y=0;y<vheight;y++)
						{
							for(int x=0;x<vwidth;x++)
							{
								for(int ii=0;ii<4/*3*/;ii++)
								{
									BYTE pixb=vbits[y*vwidth*4/*3*/+x*4/*3*/+ii];
									bits[(starty+y)*width*4/*3*/+(startx+x)*4/*3*/+ii]=pixb;
								}
							}
						}

						free(vbits);
					}
					act_glwnd->SetFocus(TRUE/*act_glwnd->HasFocus()*/);//set normal focus
					
					if(ParseFileDlg(filename,"Save Image As",szFilter,szDefExt)
						&& filename[0]!=0)
					{
						//JPG case
						int quality=100;
						char *ext=strrchr(filename,'.');
						if(ext && (stricmp(ext,".jpg")==0 || stricmp(ext,".JPG")==0))
							quality=Interface::JPGQualityDlg();
						
						sxImgLib_Save (filename, (unsigned char*)bits, NULL, width, height, 32/*24*/, quality, 0);
						Interface::CreateImageWnd(width,height);
						Interface::ShowImage(bits,NULL,width,height,32,FALSE,FALSE);
					//	char *title=strrchr(filename,'\\');if(title==NULL) title=strrchr(filename,'/');
					//	Interface::ShowImage(filename,title);
					}
					free(bits);
				}
				//window
				else if(bWnd)
				{
					HDC dc=::GetDC(::GetDesktopWindow());
					int bpp=::GetDeviceCaps(dc, BITSPIXEL)*GetDeviceCaps(dc, PLANES);
					int bytespp=bpp/8;
					::ReleaseDC(::GetDesktopWindow(), dc);
					
					if(bpp<=8)
					{
						MessageBox(NULL,"Screen colors must be greater than 8bit(256col)",PLUGNAME_SCRSHOT,MB_ICONERROR);
						break;
					}
					
					HWND hwnd=(HWND)Interface::GetMainHWND();
					HDC memDC=::GetWindowDC(hwnd);
					RECT rect;
					::GetWindowRect(hwnd,&rect);
					int width=(rect.right-rect.left);
					int height=(rect.bottom-rect.top);
					dc=::CreateCompatibleDC(memDC);
					HBITMAP hbmp=::CreateCompatibleBitmap(memDC/*dc*/, width, height);
					HBITMAP pOldBitmap = (HBITMAP)::SelectObject(/*memDC*/dc,hbmp);
					RECT rcc={0,0,width,height};::FillRect(/*memDC*/dc,&rcc,(HBRUSH)GetStockObject(WHITE_BRUSH));
					::BitBlt(/*memDC*/dc, 0, 0, width, height-4, /*dc*/memDC, 0, 0, SRCCOPY);
					//no palette support
					::SelectObject(/*memDC*/dc,pOldBitmap);
					::DeleteDC(dc);
					::ReleaseDC(hwnd,memDC);

					BYTE* bits = (BYTE*)malloc(height*width*bytespp);
					assert(bits);
					if(::GetBitmapBits(hbmp, height*width*bytespp, bits))
					{
						if(ParseFileDlg(filename,"Save Image As",szFilter,szDefExt)
							&& filename[0]!=0)
						{
							//JPG case
							int quality=100;
							char *ext=strrchr(filename,'.');
							if(ext && (stricmp(ext,".jpg")==0 || stricmp(ext,".JPG")==0))
								quality=Interface::JPGQualityDlg();
								
							//reverse scanlines
							for(int y=0;y<height/2;y++)
							{
								for(int x=0;x<width;x++)
								{
									for(int i=0;i<bytespp;i++)
									{
										BYTE top=bits[y*width*bytespp+x*bytespp+i];
										BYTE bottom=bits[(height-1-y)*width*bytespp+x*bytespp+i];
										bits[y*width*bytespp+x*bytespp+i]=bottom;
										bits[(height-1-y)*width*bytespp+x*bytespp+i]=top;
									}
								}
							}
							sxImgLib_Save (filename, (unsigned char*)bits, NULL, width, height, bpp, quality, 0);
							Interface::CreateImageWnd(width,height);
							Interface::ShowImage(bits,NULL,width,height,bpp,FALSE,FALSE);
						//	char *title=strrchr(filename,'\\');if(title==NULL) title=strrchr(filename,'/');
						//	Interface::ShowImage(filename,title);
						}
					}
					else
					{
						MessageBox(NULL,"Internal Error-GetBitmapBits() Failed!!?",PLUGNAME_SCRSHOT,MB_ICONERROR);
					}
					free(bits);
					::DeleteObject(hbmp);
				}
			}
		}
		break;
	}

	return 0;
}

BOOL ParseFileDlg(char *filename,char *Title,char *szFilter,char *szDefExt)
{
	OPENFILENAME ofn;
	memset (&ofn, 0, sizeof (OPENFILENAME));
    
	char szFile[256];
	char szFileTitle[256];
	szFile[0] = '\0';
	szFileTitle[0] = '\0';
	
	ofn.lStructSize = sizeof (OPENFILENAME);
	ofn.lpstrDefExt = szDefExt;
	ofn.lpstrFilter = szFilter;
	ofn.nFilterIndex=1;
	ofn.lpstrFile = szFile;
	ofn.nMaxFile = 256;
	ofn.lpstrFileTitle = szFileTitle;
	ofn.nMaxFileTitle = 256;
	ofn.Flags = OFN_OVERWRITEPROMPT | OFN_HIDEREADONLY | OFN_FILEMUSTEXIST | OFN_PATHMUSTEXIST;
	ofn.lpstrTitle = Title;
	
	if (::GetSaveFileName (&ofn) && szFile[0]!=0)
	{
		if(!strchr(szFile,'.')) {strcat(szFile,".");strcat(szFile,szDefExt);}
		strcpy(filename,szFile);
		return TRUE;
	}

	return FALSE;
}

cscrcapture::cscrcapture()
{
	hInst=NULL;
	IDD=IDD_UTIL_SCRSHOT;
}

void cscrcapture::Close()
{
}

BOOL cscrcapture::Run()
{
	return TRUE;
}


static BOOL GetLine(char * szLine, int size, FILE * file)
{
	BOOL bError=FALSE;

	szLine[0]=NULL;

	while (!feof(file))
	{
	    if (!fgets (szLine, size, file))
		{
			if(!feof(file)) bError = TRUE;
			break;
		}

		if (strlen(szLine)<2) // skip blank lines
			continue;

	//	AfxMessageBox(szLine);

		if (!strncmp (szLine, "//", 2)) // skip comments
			continue;

		break;
	}

	return(bError);
}

static void ReadObjNamesINI(FILE *file)
{
	BOOL bMshs,bShps,bSkns,bBns,bLts,bCams;
	BOOL bOut,bSel,bLns;
	COLORREF col,outcol;
	int xoff,yoff;

	int idat=0;

	BOOL bError = FALSE;
	char szLine[1024];
	while (!feof(file) && !bError)
	{
		memset(szLine,0,sizeof(szLine));
		bError = GetLine(szLine,sizeof(szLine),file);
		if (bError==TRUE) break;
		if(idat==0 && sscanf(szLine,"%d %d %d %d %d %d",
			&bMshs,&bShps,&bSkns,&bBns,&bLts,&bCams)==6)
		{
			bMeshes=bMshs;
			bShapes=bShps;
			bSkins=bSkns;
			bBones=bBns;
			bLights=bLts;
			bCameras=bCams;
		}
		else if(idat==1 && sscanf(szLine,"%d %d %d",&bOut,&bSel,&bLns)==3)
		{
			bOutline=bOut;
			bSelected=bSel;
			bLines=bLns;
		}
		else if(idat==2 && sscanf(szLine,"%X %X",&col,&outcol)==2)
		{
			txtcol=col;
			txtoutcol=outcol;
		}
		else if(idat==3 && sscanf(szLine,"%d %d",&xoff,&yoff)==2)
		{
			xoffset=xoff;
			yoffset=yoff;
		}

		if(idat==3) break;
		idat++;
	}
}

static void ReadScrShotINI(FILE *file)
{
	BOOL b1,b2,b3,bError=FALSE;
	char szLine[1024];
	while (!feof(file) && !bError)
	{
		memset(szLine,0,sizeof(szLine));
		bError = GetLine(szLine,sizeof(szLine),file);
		if (bError==TRUE) break;
		if(sscanf(szLine,"%d %d %d",&b1,&b2,&b3)==3)
		{
			bActive=b1;
			bAll=b2;
			bWnd=b3;
			break;
		}
	}
}

void ReadINI()
{
	char inifile[512];
	strcpy(inifile,Interface::GetMainPath());
	strcat(inifile,"/stdplugs/stdutil.ini");

	FILE *hFile=fopen(inifile,"rt");
	if(!hFile)
	{
		return;
	}

	BOOL bError = FALSE;
	char szLine[1024];
	while (!feof(hFile) && !bError)
	{
		memset(szLine,0,sizeof(szLine));
		bError = GetLine(szLine,sizeof(szLine),hFile);
		if (bError==TRUE) break;
		
		if(strstr(szLine, "[ObjectNames]"))//found object names
			ReadObjNamesINI(hFile);
		else if(strstr(szLine, "[ScreenShot]"))//found object names
			ReadScrShotINI(hFile);
	}

	fclose(hFile);
}

void SaveINI()
{
	char inifile[512];
	strcpy(inifile,Interface::GetMainPath());
	strcat(inifile,"/stdplugs/stdutil.ini");

	FILE *hFile=fopen(inifile,"wt");
	if(!hFile)
	{
		return;
	}

	fprintf(hFile,"// LightRay3D\n");
	fprintf(hFile,"// stdutil.dll ini file(DO NOT EDIT)\n\n");
	//Object Names
	fprintf(hFile,"[ObjectNames]\n");
	fprintf(hFile,"%d %d %d %d %d %d\n",bMeshes,bShapes,bSkins,bBones,bLights,bCameras);
	fprintf(hFile,"%d %d %d\n",bOutline,bSelected,bLines);
	fprintf(hFile,"%X %X\n",txtcol,txtoutcol);
	fprintf(hFile,"%d %d\n",xoffset,yoffset);

	//ScreenShot
	fprintf(hFile,"\n[ScreenShot]\n");
	fprintf(hFile,"%d %d %d\n",bActive,bAll,bWnd);

	fclose(hFile);
}





//Utility plugins can handle notification messages posted by the application
// Declare BOOL HandleMessage(DWORD msg,DWORD param1,DWORD param2) in your plugin.h file
// See Iutility.h for message explanation.
/*
BOOL cmyplugin::HandleMessage(DWORD msg,DWORD param1,DWORD param2)
{
	switch(msg)
	{
	case UTIL_MSG_UPDATE:
		return TRUE;

	case UTIL_MSG_APP_SEL_CHANGED:
		return TRUE;

	case UTIL_MSG_APP_SUBSEL_CHANGED:
		return TRUE;

	case UTIL_MSG_APP_SEL_PREDELETE:
		return TRUE;

	case UTIL_MSG_APP_SEL_POSTDELETE:
		return TRUE;

	case UTIL_MSG_APP_SUBSEL_PREDELETE:
		return TRUE;

	case UTIL_MSG_APP_SUBSEL_POSTDELETE:
		return TRUE;

	case UTIL_MSG_APP_SEL_VISCHANGED:
		return TRUE;

	case UTIL_MSG_APP_SUBSEL_VISCHANGED:
		return TRUE;

	case UTIL_MSG_APP_OBJ_ADDED:
		return TRUE;

	case UTIL_MSG_APP_OBJ_CHANGED:
		if(LOWORD(param1)==UTIL_PARAM_GEOM);
		return TRUE;

	case UTIL_MSG_APP_OBJ_TYPECONVERTED:
		return TRUE;

	case UTIL_MSG_APP_MAT_ADDED:
		return TRUE;

	case UTIL_MSG_APP_MAT_CHANGED:
		switch(LOWORD(param1))
		{
		case UTIL_PARAM_IMAGE:
			{
				switch(HIWORD(param1))
				{
				case MAT_IMG_DIFFUSE:
					break;
				case MAT_IMG_OPACITY:
					break;
				}
			}
			break;

		default:
			MessageBeep(0);
		}
		return TRUE;

	case UTIL_MSG_APP_MAT_PREDELETE:
		return TRUE;

	case UTIL_MSG_APP_MAT_POSTDELETE:
		return TRUE;

	case UTIL_MSG_APP_MAT_UPDATE:
		return TRUE;

	case UTIL_MSG_APP_ANIMCHANGED:
		return TRUE;

	case UTIL_MSG_APP_PREEDITANIM:
		return TRUE;

	case UTIL_MSG_APP_POSTEDITANIM:
		return TRUE;

	case UTIL_MSG_APP_PRENEW:
		return TRUE;

	case UTIL_MSG_APP_POSTNEW:
		return TRUE;

	case UTIL_MSG_APP_PRELOAD:
		return TRUE;

	case UTIL_MSG_APP_POSTLOAD:
		return TRUE;

	case UTIL_MSG_APP_PREMERGE:
		return TRUE;

	case UTIL_MSG_APP_POSTMERGE:
		return TRUE;

	case UTIL_MSG_APP_IMPORT:
		return TRUE;

	case UTIL_MSG_APP_CREATESTART:
		return TRUE;

	case UTIL_MSG_APP_CREATEEND:
		return TRUE;

	case UTIL_MSG_APP_MODIFY:
		if(param1==UTIL_PARAM_INIT) MessageBeep(0);
		else if(param1==UTIL_PARAM_APPLY) MessageBeep(0);
		else if(param1==UTIL_PARAM_DONE) MessageBeep(0);
		return TRUE;

	case UTIL_MSG_APP_ENV_CHANGED:
		return TRUE;

	case UTIL_MSG_APP_VIEWCHANGED:
		return TRUE;

	case UTIL_MSG_RAW_OBJ_ADDED:
		return TRUE;

	case UTIL_MSG_RAW_OBJ_DELETED:
		return TRUE;

	case UTIL_MSG_RAW_MAT_ADDED:
		return TRUE;

	case UTIL_MSG_RAW_SEL_CHANGED:
		return TRUE;

	case UTIL_MSG_RAW_VISCHANGED:
		return TRUE;
	}

	return FALSE;
}
*/